\\ Copyright 2019, 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
read("../vpar.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");


print("===============================================================");

option_verbose = 0;
option_progress = 1;


\\-----------------------------------------------------------------------------



\\-----------------------------------------------------------------------------
\\ non-triangular-ness

\\ A023532 = n+1 not triangular
A023532(n) = ! test_is_triangular(n+1);
{
  my(want=OEIS_data("A023532")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A023532(n)),
              want,
              "A023532() samples");
}


\\-----------------------------------------------------------------------------
progress("Bulgarian Solitaire - Partitions");

{
  \\ triangular are trees
  for(t=1,6,
     my(n=test_triangular(t),
        vpar=test_make_Bulgarian_solitaire(n));
     check_equal(#vpar, numbpart(n),
                "Bulgarian solitaire triangular numbpart");
     check_equal(vpar_is_cyclic(vpar), 0,
                "Bulgarian solitaire triangular acyclic");
     check_equal(vpar_num_components(vpar), 1,
                "Bulgarian solitaire triangular one component");
     my(height=vpar_height(vpar));
     check_equal(height, t*(t-1),
                "Bulgarian solitaire triangular height pronic");
     check_equal(height, A002378_samples[t],
                "Bulgarian solitaire triangular height pronic = A002378");

     check_equal(vpar_is_asymmetric(vpar), n<10,
                 "Bulgarian solitaire triangular asymmetric n<10");
     check_equal(vpar_is_free_asymmetric(vpar), n!=3 && n<10,
                 "Bulgarian solitaire triangular free asymmetric n!=3 and <10");
     check_equal(vpar_num_automorphisms(vpar) % 3, n<15,
                 "Bulgarian solitaire no automorphisms order 3 for n<15");
  );
}

{
  \\ N=6

  my(n=6,
     vpar=test_make_Bulgarian_solitaire(n));
  check_equal(vpar_is_cyclic(vpar), 0);
  check_equal(vpar_is_tree(vpar), 1);
  check_equal(vpar_num_leaves(vpar), 4);
  check_equal(vpar_num_automorphisms(vpar), 1);
  check_equal(vpar_is_asymmetric(vpar), 1);

  my(centre=vpar_centres(vpar));
  check_equal(#centre,1, "Bulgarian solitaire N=6 unicentral");
  centre = centre[1];

  my(r=vpar_root(vpar));
  check_equal(r,6);

  \\ root is leaf nearest the centre
  my(leaves=vpar_leaves(vpar));
  check_equal(leaves, [6, 9, 10, 11]);
  my(dists=apply(v->vpar_path_length(vpar,v,centre), leaves));
  check_equal(dists, [2, 4, 3, 4]);
  my(min_pos, min_dist=vecmin(dists,&min_pos));
  check_equal(min_dist, 2);
  check_equal(min_pos, 1);
  check_equal(leaves[min_pos], r);
}
{
  \\ N=7

  my(n=7,
     vpar=test_make_Bulgarian_solitaire(n));
  check_equal(#vpar, 15);
  check_equal(vpar_is_cyclic(vpar), 1);
  check_equal(vpar_num_components(vpar), 1);
  check_equal(vpar_component_cycle_lengths(vpar), [4]);
  check_equal(vpar_num_leaves(vpar), 5);
}
{
  \\ N=9

  my(n=9,
     vpar=test_make_Bulgarian_solitaire(n));
  check_equal(#vpar, 30);
  check_equal(vpar_is_cyclic(vpar), 1);
  check_equal(vpar_num_components(vpar), 1);
  check_equal(vpar_component_cycle_lengths(vpar), [4]);
  check_equal(vpar_num_leaves(vpar), 10);
}
{
  \\ N=10

  my(n=10,
     vpar=test_make_Bulgarian_solitaire(n));
  check_equal(#vpar, 42);
  check_equal(numbpart(n), 42);
  check_equal(vpar_is_cyclic(vpar), 0);
  check_equal(vpar_num_roots(vpar), 1);
  check_equal(vpar_num_leaves(vpar), 15);
  check_equal(vpar_indnum(vpar), 23);
  check_equal(vpar_domnum(vpar), 17);

  check_equal(vpar_num_automorphisms(vpar), 4);
  my(perms=vpar_automorphisms(vpar),
     S2=test_perms_Sn(2),
     S2xS2=test_perms_direct_product(S2,S2));
  check_equal(test_perms_is_isomorphic(S2xS2,perms), 1);
  check_equal(apply(test_perm_to_cycles_sans_fixed,perms),
              [[],
               [Vecsmall([15, 31]), Vecsmall([30, 38])],
               [Vecsmall([16, 25]), Vecsmall([29, 34])],
               [Vecsmall([15, 31]), Vecsmall([16, 25]), Vecsmall([29, 34]), Vecsmall([30, 38])]]);

  my(r=vpar_root(vpar));
  check_equal(r,20);
  check_equal(vpar_num_children_of_vertex(vpar,r), 1);
  my(c=vpar_children_of_vertex(vpar,r)[1]);
  check_equal(c,12);
  check_equal(vpar_degree_of_vertex(vpar,c), 4);

  \\ root is the sole leaf with degree=4 neighbour
  for(v=1,#vpar,
     vpar_is_leaf(vpar,v) || next;
     my(a=vpar_neighbours_of_vertex(vpar,v)[1]); \\ attachment vertex
     check_equal(vpar_degree_of_vertex(vpar,a) == 4,
                 v==r));
}

{
  \\ N=12

  my(n=12,
     vpar=test_make_Bulgarian_solitaire(n));

  \\ Probably doesn't work properly for cyclics.
  \\ check_equal(vpar_num_automorphisms(vpar), 96);
}

\\ test_distance_to_leaf(vpar,v) =
\\ {
\\   my(path_length=vpar_path_lengths_vector(vpar,v),
\\      leaves=vpar_leaves(vpar),
\\      ret=#vpar);
\\   for(i=1,#leaves,
\\      my(l=leaves[i]);
\\      if(l!=v, ret=min(ret,path_length[l])));
\\   ret;
\\ }
\\ test_num_vertices_below_distance(vpar,v,dist) =
\\ {
\\   my(path_length=vpar_path_lengths_vector(vpar,v));
\\   sum(i=1,#vpar, v!=i && path_length[i]<=dist);
\\ }
{
  \\ N=15

  my(n=15,
     vpar=test_make_Bulgarian_solitaire(n));
  check_equal(#vpar, 176);
  check_equal(numbpart(n), 176);
  check_equal(vpar_indnum(vpar), 92);
  check_equal(vpar_domnum(vpar), 74);
  check_equal(vpar_diameter(vpar), 29);
  check_equal(vpar_matchnum(vpar), 84);
  check_equal(vpar_num_automorphisms(vpar), 12288);
  \\ vpar_dot(vpar);

  my(r=vpar_root(vpar));
  check_equal(r,74);
  check_equal(vpar_num_children_of_vertex(vpar,r), 1);
  my(c=vpar_children_of_vertex(vpar,r)[1]);
  check_equal(c,49);
  check_equal(vpar_degree_of_vertex(vpar,c), 4);

  my(centre=vpar_centres(vpar));
  check_equal(#centre,2, "Bulgarian solitaire N=15 bicentral");

  \\ for(v=1,#vpar,
  \\    vpar_is_leaf(vpar,v) || next;
  \\    my(a=vpar_neighbours_of_vertex(vpar,v)[1]); \\ attachment vertex
  \\    vpar_degree_of_vertex(vpar,a)==4 || next;
  \\    print(v"  "vpar_path_length(vpar,v,centre[1])" "vpar_path_length(vpar,v,centre[2]));
  \\ );
  \\    if(1 || vpar_degree_of_vertex(vpar,a)==4,
  \\       print(v"  "vpar_eccentricity_of_vertex(vpar,v)"  "test_num_vertices_below_distance(vpar,v,3)
  \\            "  depth="vpar_depth_of_vertex(vpar,v));
  \\       );
  \\    \\ check_equal(vpar_degree_of_vertex(vpar,a) == 4,
  \\    \\             v==r);
}

test_longest_cycle_length(vpar) =
{
  my(vec=vpar_component_cycle_lengths(vpar));
  vec=select(n->n, vec);  \\ non-zeros
  if(#vec, vecmax(vec));
}
{
  check_equal(numbpart(30), 5604);
  my(samples = A183110_samples[1..min(30,#A183110_samples)]);
  check_equal(vector(#samples,n, test_longest_cycle_length(test_make_Bulgarian_solitaire(n,1))),
              samples,
              "A183110 Bulgarian solitaire longest cycle length");
}

test_vector_rotate_up(vec,s) = vector(#vec,i, vec[((i-s-1)%#vec) + 1]);
check_equal(test_vector_rotate_up([],10), []);
check_equal(test_vector_rotate_up([10,11,12,13,14],0), [10,11,12,13,14]);
check_equal(test_vector_rotate_up([10,11,12,13,14],1), [14,10,11,12,13]);
check_equal(test_vector_rotate_up([10,11,12,13,14],3), [12,13,14,10,11]);
check_equal(test_vector_rotate_up([10,11,12,13,14],-1), [11,12,13,14,10]);

\\ n is an integer.
\\ Return it with its low k many bits rotated upwards by s steps.
test_binary_rotate_up(n,s,k) =
{
  s %= k;
  n <<= s;
  bitor(bitand(n,2^k-1), n>>k);
}
check_equal(test_binary_rotate_up(0,0,1), 0);
check_equal(test_binary_rotate_up(test_from_binary(1101),1,4), \
            test_from_binary(1011));

\\ formula as given by Paul Weisenhorn in OEIS A037306
test_num_compositions_up_to_rotation(t,k) = \
  sumdiv(gcd(t,k),d, eulerphi(d)*binomial(t/d,k/d)) / t;
{
  my(want=OEIS_data("A037306"),
     l=List([]));
  for(t=1,#want,
     for(k=1,t,
        if(#l >= #want, break(2));
        listput(l, test_num_compositions_up_to_rotation(t,k))));
  check_equal(Vec(l), want,
              "test_num_compositions_up_to_rotation() = A037306");
}

\\ Return the number of compositions of t (partitions with order) into k
\\ parts which are different up to rotation.
test_num_compositions_up_to_rotation_by_bits(t,k) =
{
  my(ret=0);
  for(i=0,2^t-1,
     if(hammingweight(i)!=k, next);
     for(s=1,t, if(test_binary_rotate_up(i,s,t) < i, next(2)));
     ret++);
  ret;
}
{
  for(t=1,10,
     for(k=1,t,
        check_equal(test_num_compositions_up_to_rotation_by_bits(t,k),
                    test_num_compositions_up_to_rotation(t,k))));
}

test_partition_rank(p) = if(#p, p[#p] - #p, 0);
check_equal(test_partition_rank(Vecsmall([])), 0);
check_equal(test_partition_rank(Vecsmall([2,3,4])), 1);

test_num_partitions_positive_rank(n) =   \\ meaning >=1
{
  my(total=0);
  forpart(p=n, total += (test_partition_rank(p) > 0));
  total;
}
{
  check_equal(numbpart(40), 37338);
  my(want=OEIS_data("A064173"));  \\ OFFSET=1
  if(#want>40, want=want[1..40]);
  check_equal(vector(#want,n, test_num_partitions_positive_rank(n)),
              want,
              "A064173 test_num_partitions_positive_rank()");
}

test_num_partitions_rank1(n) =
{
  my(total=0);
  forpart(p=n, total += (test_partition_rank(p)==1));
  total;
}
{
  check_equal(numbpart(40), 37338);

  my(want=OEIS_data("A101198"));  \\ OFFSET=1
  want=want[1..min(40,#want)];
  check_equal(vector(#want,n, test_num_partitions_rank1(n)),
              want,
              "A101198 test_num_partitions_rank1()");
}

\\ test_Bulgarian_solitaire_num_Eden() returns the number of Garden of Eden
\\ partitions count per the formula of
\\
\\     Brian Hopkins and James A. Sellers, "Exact Enumeration of Garden of
\\     Eden Partitions", INTEGERS: Electronic Journal of Combinatorial
\\     Number Theory 7(2), 2007, #A19.
\\
\\ They show the Garden of Eden partitions are those of rank <= -2, and
\\ count such of given n by formula or generating function.
\\
test_Bulgarian_solitaire_num_Eden(n) = \
  sum(j=1,sqrtint(n), (-1)^(j+1) * numbpart(n - 3*j*(j+1)/2));
{
  \\ j = sqrtint(n)
  \\ n = j^2 + r  with 0 <= r < 2j+1
  \\ 3*j*(j+1)/2
  \\  = 3/2*j*(j+1)
  \\  = 3/2*(j^2 + j)
  \\  = 3/2*(n - r + j)
  \\  = n + 1/2*(n - 2j + j)
  \\  = n + 1/2*(n - j)
  \\ is >= n when n>=j which is n>=1
  \\
  for(n=1,500,
     my(j = sqrtint(n));
     check_equal(3*j*(j+1)/2 >= n, 1));
}
{
  check_equal(3*x*(x+1)/2 - 'n,  3/2*x^2 + 3/2*x - 'n);
  \\ x = (-3/2 + sqrt((3/2)^2 + 4*3/2*n))/(2*3/2)
  \\ x = sqrt(2/3*n + 1/4) - 1/2
  \\ (i+f-1)/2 - (i-1)/2 == 1/2*f
  for(n=1,5000,
     my(j = (sqrtint(8*n\3 + 1) - 1)\2);   \\ inverse
     check_equal(n >= 3*j*(j+1)/2, 1);
     j++;
     check_equal(n < 3*j*(j+1)/2, 1));

  my(want=OEIS_data("A123975"));  \\  OFFSET=1
  check_equal(vector(#want,n, test_Bulgarian_solitaire_num_Eden(n)),
              want,    \\ OFFSET=1
              "A123975 test_Bulgarian_solitaire_num_Eden()");
}

\\ Generating function per Hopkins and Jones above.
test_Bulgarian_solitaire_num_Eden_gf(limit) =
{
  my(L=O(x^limit));
  prod(i=1,limit, 1/(1-x^i + L))
  * sum(r=1,limit, (-1)^(r-1) * x^((3*r^2+3*r) / 2));
}
check_equal(vector(50,n,n--; test_Bulgarian_solitaire_num_Eden(n)), \
            test_gf_terms(test_Bulgarian_solitaire_num_Eden_gf(50),50));

test_num_partitions_rank_le_neg2_by_count(n) =
{
  my(total=0);
  forpart(p=n, total += (test_partition_rank(p) <= -2));
  total;
}
{
  check_equal(numbpart(40), 37338);
  my(samples=A123975_samples[1..min(40,#A123975_samples)]);
  check_equal(vector(#samples,n, test_num_partitions_rank_le_neg2_by_count(n)),
              samples,
              "A123975 test_num_partitions_rank_le_neg2_by_count()");
}
{
  \\ A064173 = num partitions rank >= 1
  \\ A101198 = num partitions rank == 1
  \\ A123975 = num partitions rank >= 2
  my(len=vecmin([#A064173_samples, #A101198_samples, #A123975_samples]));
  check_equal(A064173_samples[1..len] - A101198_samples[1..len],
              A123975_samples[1..len],
              "A123975 = A064173 - A101198");
}

test_num_partitions_rank_ge_neg1_by_count(n) =
{
  my(total=0);
  forpart(p=n, total += (test_partition_rank(p) >= -1));
  total;
}
test_gA260894(limit) =
{
  \\ formula per A260894
  my(L=O(x^limit));
  sum(m=0,limit, x^m*prod(k=1,m, (1-x^(m+k))/(1-x^k + L)));
}
test_A260894(n) = polcoeff(test_gA260894(n+1),n);
{
  check_equal(test_gf_terms(test_gA260894(#A260894_samples), #A260894_samples),
              A260894_samples,
              "A260894 test_gA260894()");
  check_equal(vector(20,n,n--; test_A260894(n)),
              A260894_samples[1..20],
              "A260894 test_A260894()");

  check_equal(numbpart(40), 37338);
  my(samples=A260894_samples[1..min(40,#A260894_samples)]);
  check_equal(vector(#samples,n,n--; test_num_partitions_rank_ge_neg1_by_count(n)),
              samples,
              "A260894 test_num_partitions_rank_ge_neg1_by_count()");

  \\ A000041 number of partitions
  \\ A123975 number of partitions rank <= -2
  \\ A260894 number of partitions rank >= -1
  check_equal(vector(20,n,n--; test_Bulgarian_solitaire_num_Eden(n) + test_A260894(n)),
              vector(20,n,n--; numbpart(n)),
              "A123975 + A260894 = A000041, functions");

  \\ A123975 OFFSET=1
  \\ A260894 OFFSET=0
  \\ A000041 OFFSET=0
  check_equal(test_Bulgarian_solitaire_num_Eden(0),  0,
              "test_Bulgarian_solitaire_num_Eden(0)");
  check_equal(test_A260894(0),  1,
              "A260894(0)");
  my(A123975_samples_with_zero = concat(0,A123975_samples),
     len=vecmin([#A123975_samples_with_zero, #A260894_samples, #A000041_samples]));
  check_equal(A123975_samples_with_zero[1..len] + A260894_samples[1..len],
              A000041_samples[1..len],
              "A123975 + A260894 = A000041, samples");
}
\\ adapting Hopkins and Jones
test_A260894_by_sum(n) = \
  numbpart(n) + sum(j=1,sqrtint(n), (-1)^j * numbpart(n - 3*j*(j+1)/2));
{
  check_equal(vector(#A260894_samples,n,n--; test_A260894_by_sum(n)),
              A260894_samples,
              "A260894 test_A260894_by_sum()");
}

\\ Return the depth of v, or distance up to a cyclic vertex if v is in a
\\ cyclic component.
test_depth_below_cycle_of_vertex(vpar,v) =
{
  my(seen=vectorsmall(#vpar),
     depth=0);
  while(vpar[v],
        v=vpar[v];
        if(seen[v], return(seen[v]));
        seen[v]=depth++);
  depth;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     for(v=1,#vpar,
        check_equal(test_depth_below_cycle_of_vertex(vpar,v),
                    vpar_depth_of_vertex(vpar,v))));
}
check_equal(test_depth_below_cycle_of_vertex([2,3,2],1), 1);
check_equal(test_depth_below_cycle_of_vertex([2,3,2,1],4), 2);

\\ Return the number of non-triangular numbers <= n.
A083920(n) = sum(i=0,n, !test_is_triangular(i));
{
  my(want=OEIS_data("A083920")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A083920(n)),
              want);
}

\\ A002024 = n appears n times
A002024(n) = test_untriangular_floor(n-1) + 1;
{
  my(want=OEIS_data("A002024")); \\ OFFSET=1
  check_equal(vector(#want,n, A002024(n)),
              want);
}

\\ A002260 = runs 1..t
A002260(n) = my(t=A002024(n)); n - test_triangular(t-1);
{
  my(want=OEIS_data("A002260")); \\ OFFSET=1
  check_equal(vector(#want,n, A002260(n)),
              want);
}

{
  \\ all n
  for(n=1,20,
     my(vpar=test_make_Bulgarian_solitaire(n));
     check_equal(#vpar, numbpart(n),
                "Bulgarian solitaire numbpart");
     check_equal(vpar_is_cyclic(vpar), ! test_is_triangular(n),
                "Bulgarian solitaire cyclic when not triangular");

     \\ Ethan Akin and Morton Davis, "Bulgarian Solitaire", American
     \\ Mathematical Monthly, volume 92, number 4, April 1985, pages
     \\ 237-250.  http://www.jstor.org/stable/2323643
     \\ Num components is number of compositions of t into r pieces, up to
     \\ rotation of those compositions.
     \\
     my(num_components=vpar_num_components(vpar));
     check_equal(num_components,
                 A037306_samples[n],   \\ offset 1
                "Bulgarian solitaire num components = A037306");
     my(t = test_untriangular_floor(n-1) + 1,
        r = n - test_triangular(t-1));
     check_equal(r >= 1, 1);
     check_equal(r <= t, 1);
     check_equal(n, test_triangular(t-1) + r);
     check_equal(num_components,
                 test_num_compositions_up_to_rotation(t,r),
                "Bulgarian solitaire num components = compositions up to rotation");

     my(num_cyclic_vertices=vpar_num_cyclic_vertices(vpar),
        roots = vpar_roots(vpar),
        num_roots = #roots);
     check_equal(num_cyclic_vertices + num_roots,
                 binomial(t,r),
                 "Bulgarian solitaire num cyclic + root vertices = binomial");
     check_equal(num_cyclic_vertices,
                 if(t==r,0,binomial(t,r)),
                 "Bulgarian solitaire num cyclic vertices");
     check_equal(num_cyclic_vertices,
                 if(test_is_triangular(n),0, binomial(A002024(n),A002260(n))),
                 "Bulgarian solitaire num cyclic vertices by A-numbers");
     check_equal(num_cyclic_vertices,
                 A023532(n-1) * binomial(A002024(n),A002260(n)),
               "Bulgarian solitaire num cyclic vertices by A-numbers product");
     check_equal(num_cyclic_vertices,
                 A325191_samples[n+1],  \\ OFFSET=0
          "A325191 Bulgarian solitaire num cyclics, not self-loop");

     my(vpar_with_self_loops=vpar);
     for(i=1,#roots,
        vpar_with_self_loops[roots[i]] = roots[i]);  \\ root to self-loop
     check_equal(vpar_num_cyclic_vertices(vpar_with_self_loops),
                 binomial(t,r),
          "Bulgarian solitaire num cyclic vertices incl self-loop = binomial");
     check_equal(binomial(t,r),
                 A074909_samples[n+1],  \\ OFFSET=0
          "A074909 Bulgarian solitaire binomial(t,r)");

     \\ Garden of Eden = childless
     my(num_childless_with_self_loops=vpar_num_childless(vpar_with_self_loops));
     check_equal(num_childless_with_self_loops,
                 test_Bulgarian_solitaire_num_Eden(n),
               "Bulgarian solitaire num childless (Garden of Eden)");
     check_equal(#vpar_with_self_loops - num_childless_with_self_loops,
                 test_A260894(n),
               "A260894 Bulgarian solitaire num childful (non Garden of Eden)");
     my(parts=partitions(n));
     check_equal(vpar_childless_vertices(vpar_with_self_loops),
                 Vec(select(p->test_partition_rank(p)<=-2, parts, 1)));

     \\ Depth of partition [n], asked by Cranston and West
     \\ print(test_depth_below_cycle_of_vertex(vpar,1)," ",A083920(n-1));
     check_equal(test_depth_below_cycle_of_vertex(vpar,1),
                 if(n==2,1, A083920(n-1)),
                 "Bulgarian solitaire depth of [n]");
  );
}


\\-----------------------------------------------------------------------------
progress("Bulgarian Solitaire - Compositions");

\\ num = integer 1 to 2^(n-1) inclusive
\\ vec = composition summing to n
\\
\\ There are various ways to extract bit run lengths.  The form here uses
\\ num-1 so range 0 to 2^(n-1)-1, adds bit 2^(n-1) so
\\     high                low
\\      1  00 1 0000 1 000 1 0
\\         |----- num-1 -----|
\\ Start at the high 1-bit, then composition term vec[1] says how many bit
\\ positions to go down for the first 1 within value num-1, vec[2] how many
\\ to go down to the next, etc.  b[] is binary high to low, with high 1
\\ included in num_to_composition() but excluded in composition_to_num().
\\
test_composition_to_num(vec) =
{
  my(b=vector(vecsum(vec)-1), pos=vec[1]);
  for(i=2,#vec, b[pos]=1; pos+=vec[i]);
  test_fromdigits(b,2) + 1;
}
test_num_to_composition(num,n) =
{
  my(b=binary(num + (1<<(n-1)) - 1),
     vec=vector(hammingweight(b)),
     pos=0);
  for(i=1,#b, vec[pos+=b[i]]++);
  vec;
}

test_Bulgarian_composition_step(p) =
{
  \\ like Bulgarian_solitaire_step(), but no vecsort()
  p=Vec(p);
  for(i=1,#p, p[i]--);
  select(x->x!=0, concat(p,[#p]));
}
test_make_Bulgarian_compositions(n) =
{
  vector(2^(n-1),v,
         my(p=test_composition_to_num
              (test_Bulgarian_composition_step(test_num_to_composition(v,n))));
         if(p==v, 0, p));
}

{
  \\ all n
  for(n=1,14,
     my(vpar=test_make_Bulgarian_compositions(n));
     check_equal(#vpar, test_num_compositions(n),
                "Bulgarian compositions test_num_compositions()");

     \\ Brian Hopkins, Michael A. Jones, "Shift-Induced Dynamical Systems on
     \\ Partitions and Compositions", The Electronic Journal of Combinatorics,
     \\ volume 13, 2006.
     \\ Number of "Garden of Eden" compositions (those without predecessors)
     \\ is 2^(n-1) - fibonacci(n+1).
     check_equal(vpar_num_childless(vpar),
                 if(n==1,1, 2^(n-1) - fibonacci(n+1)),
                "Bulgarian compositions vpar_num_childless()");
  );
}
{
  check_equal(vector(#A008466_samples,n,
                     if(n==0,1, 2^(n-1) - fibonacci(n+1))),
              A008466_samples,
              "A008466 = 2^n - fibonacci(n+2) shifted by 1");
}


\\-----------------------------------------------------------------------------
progress("end");
