\\ Copyright 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../examples/complete-binary-matchings.gp");
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
read("interv.gp");
read("interv-wip.gp");

print("===============================================================\n\n");

option_verbose=0;
option_progress = 1;


\\ Other copies of Wolfram:
\\ http://new.math.uiuc.edu/im2008/dakkak/papers/files/wolfram.smca.pdf
\\ https://www-users.york.ac.uk/~gt512/BIC/wolfram83.pdf
\\ http://lattice.ifsc.usp.br/~lattice/oldlattice/artigo-wolfram-cellular-autom.pdf



\\-----------------------------------------------------------------------------
\\ vpar_matchnum() of complete binary tree
\\ 0,0,1,2,5,10,21,42,85,170,341,
\\ A000975
\\   vertex cover = min num vertices to delete to obtain an independent set
\\   a(n)   = |VC| 
\\   a(n+1) = indnum

A000975(n) =
{
  n>=0 || error();
  2^(n+1)\3;
}
{
  my(want=OEIS_data("A000975")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A000975(n)),
              want);
}

\\          *         row 1  (root)
\\        /   \
\\       *     *      row 2
\\      / \   / \
\\     *   * *   *    row 3
\\ indnum = 5
\\ always take leaves, drop their attachment points

{
  for(rows=0,10,
     my(height=rows-1,
        vpar=vpar_make_complete_tree(2,height));

     \\ Sen-Peng Eu in A000975
     check_equal(#vpar, 2^rows-1);
     check_equal(vpar_indnum(vpar), A000975(rows),
                 "indnum");
        
     if(rows >= 1,
        check_equal(vpar_matchnum(vpar), A000975(rows-1),
                    "matchnum"));
     );
}

\\-----------------------------------------------------------------------------
\\ A338293 = Num Matchings

{
  OEIS_check_func("A338293","a");
  check_equal(a(8),
              316789536051348354157896789538095888519287109375,
              "a(8)");
  check_equal(a(9),
              200124345390089658803186940444206761926203919781706304118599611508057615137659013271331787109375,
              "a(9)");
  check_equal(#digits(a(9)), 96,
              "a(9) num digits");
}
{
  \\ b-file  ~/OEIS/b338293.txt
  my(want=OEIS_data("A338293",'bfile));
  check_equal(want[1], 1);  \\ OFFSET=0
  check_equal(want[2], 1);
  check_equal(want[3], 3);
  print("A338293 bfile "#want" terms, "
        " last value length ",#digits(want[#want])," digits");
  check_equal(vector(#want,n,n--; a(n)),
              want);
}
{
  \\ some formulas
  for(n=2,10,
     check_equal(a(n-1)^2     +       2 * a(n-1)*a(n-2)^2,
                 a(n));
     check_equal(a(n),
                 a(n-1)^2 + 2*a(n-1)*a(n-2)^2);
  );
  for(n=1,100,
     check_equal(J(n+1),
                 J(n) + 2*J(n-1));
     check_equal(J(n),
                 (2^n - (-1)^n)/3));


  my(aa = (n) -> my(x=1,y=1); for(i=2,n, [x,y] = [x^2 + 2*x*y^2, x]); x);
  for(n=0,10,
     check_equal(a(n), aa(n)));
  check_equal(a(0), 1);
  check_equal(a(1), 1);
  check_equal(a(2), 3);

  check_equal(a(4), J(5) * J(4) * J(3)^2 * J(2)^4 * J(1)^8);
  check_equal(a(4), 11  *   5  *    3^2 *    1^4 *    1^8);
  check_equal(a(4), 495);
}
{
  \\ Jacobsthals
  OEIS_check_func("A001045","J");
}

\\-----------------------------------------------------------------------------
\\ A228607 Relation by R. J. Mathar

A228607_c(n) =
{
  n>=1 || error();
  if(n==1,1, A228607_d(n-1)^2);
}
A228607_d(n) =
{
  n>=1 || error();
  if(n==1,3,
     2*A228607_c(n-1)*A228607_d(n-1)
     + A228607_d(n-1)^2);
}
A228607(n) =
{
  n>=1 || error();
  A228607_d(n)^3 + 3*A228607_c(n)*A228607_d(n)^2;
}
{
  OEIS_check_func("A228607",,['offset,1]);
  check_equal(vector(3,n, A228607(n)),
              vector(3,n, 
                     a(n+1)^2*(a(n+1)+3*a(n)^2)));
}

\\-----------------------------------------------------------------------------
\\ A242049 = log(C)

A242049_samples = OEIS_data("A242049");
A242049_samples_frac = fromdigits(A242049_samples)/10^#A242049_samples;
{
  check_equal(logC_digits_vector(#A242049_samples), A242049_samples,
             "logC_digits_vector() vs A242049 samples");
}

\\-----------------------------------------------------------------------------
\\ A338294 = C growth power
 
{
  \\ A338294 growth power decimals
  my(want_digits=OEIS_data("A338294"), \\ OFFSET=1
     want_frac = fromdigits(want_digits)/10^(#want_digits-1));
  check_equal(nearly_equal(want_frac, 1.5371767171823579495901403289),
              1, "want_frac");

  check_equal(want_digits[1..51],
              [1,5,3,7,1,7,6,7,1,7,1,8,2,3,5,7,9,4,9,5,9,0,1,4,0,3,2,8,9,5,5,2,2,1,6,0,2,5,0,1,5,0,8,0,9,3,4,3,2,3,6],
              "example in A242049");

  my(len=200,
     l=logC_digits_vector(len));
  l=fromdigits(l);
  l=[l,l+1];
  l /= 10^len;
  l = interv_exp(l,70);
  \\ check_equal(interv_contains_constant(l, want_frac), 1);
 
  check_equal(want_digits,
              interv_frac_digits(l,,#want_digits));

  default(realprecision,200);
  print(exp(A242049_samples_frac));
  print(concat(apply(d->Str(d),want_digits)));

  l=logC_digits_vector(200);
  l=fromdigits(l)/10^#l;
  print(exp(l));
}

\\-----------------------------------------------------------------------------
progress("Kropf and Wagner");

\\ log(t_n) = sum numruns_i(n) * log(s_i)
{
  my(s=(n)->1/3*(2^(n+2) - (-1)^n));    \\ s_n = Chi(n)
  check_equal(vector(100,n, s(n)),
              vector(100,n, Chi(n)));
}


\\-----------------------------------------------------------------------------
progress("Run Lengths");

\\    11         00 111    0 11
\\ 11 ? 11    00111 00111 011 011
\\
\\ run of 1s and the 0s below it
\\    11 00        111 0    11
\\  1100 1100   1110 1110 11 11
to_run_lengths(n) =
{
  my(l=List([]));
  if(n,
     my(k=logint(n,2));
     while(k>=0,
           my(x=k);
           while(bittest(n,k), k--);
           listput(l,x-k);
           x=k;
           while(k>=0 && !bittest(n,k), k--);
           listput(l,x-k)));
  Vec(l);
}
check_equal(to_run_lengths(10), [1,1, 1,1]);
check_equal(to_run_lengths(15), [4,0]);
check_equal(to_run_lengths(30), [4,1]);

from_run_lengths(v) =
{
  my(bits=vector(vecsum(v)),upto=0,i=1);
  for(i=1,#v,
        for(j=1,v[i], bits[upto++]=1);
        upto += v[i++]);
  upto==vecsum(v) || error();
  fromdigits(bits,2);
}
check_equal(from_run_lengths([1,1, 1,1]), 10);
check_equal(from_run_lengths([4,0]), 15);


\\-----------------------------------------------------------------------------
progress("A161820 minimum of its run lengths");

\\ v = [run_1s_length,   \\ of even length.
\\      run_0s_length,
\\      run_1s_length,
\\      run_0s_length,
\\      ...]
\\ Return a similar such vector with same multiset of 1s lengths, a single 0
\\ separator between each, and sorted to give the smallest numerical value,
\\ so 1s lengths smallest first.
run_lengths_minimum_vector(v) =
{
  #v%2==0 || error("run_lengths_minimum_vector() is for even length vector");
  my(ones=vector(#v/2,i, v[2*i-1]));
  ones=vecsort(ones);
  vector(#v,i, if(i%2, ones[i\2+1], i==#v,0,1));
}
{
  check_equal(run_lengths_minimum_vector([1,2]), [1,0]);
  check_equal(run_lengths_minimum_vector([7,1,1,2]), [1,1,7,0]);
  check_equal(run_lengths_minimum_vector([8,100,9,100,5,100]), [5,1,8,1,9,0]);

  \\ check_equal(vector(10,n,n--; run_lengths_minimum_vector(first_trinomial_index_vector(n))),
  \\             vector(10,n,n--; first_trinomial_index_vector(n)),
  \\             "first_trinomial_index_vector() is minimal");
}

\\ return an integer with the same multiset of runs of 1s lengths, but of
\\ smallest numerical value (A161820)
run_lengths_minimum(n) = from_run_lengths(run_lengths_minimum_vector(to_run_lengths(n)));
\\ vector(20,n, run_lengths_minimum(n))
{
  check_equal(vector(#A161820_samples,n, run_lengths_minimum(n)),
              A161820_samples,  \\ OFFSET=1
              "run_lengths_minimum() vs A161820_samples");

  \\ a_index_T() is already the minimum for its run lengths of 1s
  check_equal(vector(10,n,n--; run_lengths_minimum(a_index_T(n))),
              vector(10,n,n--; a_index_T(n)),
              "a_index_T() is minimal");
}

\\ ENHANCE-ME: Might like to avoid quadratic time shifting up successively.
A161820_compact(n) =
{
  if(n,
     my(v=select(k->bittest(n,k)!=bittest(n,k-1),[0..logint(n,2)+1]));
     v=vector(#v/2,i,v[2*i]-v[2*i-1]);
     v=vecsort(v,,4);
     my(p=1); sum(i=1,#v, my(l=p,h=p<<v[i]);p=h<<1; h-l);

     \\ my(p=1,ret=0);
     \\ for(i=1,#v, my(h=p<<v[i]); ret+=h-p; p=h<<1);
     \\ for(i=2,#v, v[i]+=v[i-1]+1);
     \\ sum(i=0,#v-1, 2^v[i+1] - 2^if(i,v[i]+1));
     \\ v=apply(r->vector(r+1,i,i>1),v);
     \\ v=concat(v);
     \\ fromdigits(v,2);
     ,
     0);


\\ concat([]) is no good, so a dummy 0 length run
 \\ vector(hammingweight(n)-1)List([0])); for(k=0,if(n,logint(n,2)),
 \\     if(bittest(n,k), my(s=k); while(bittest(n,k++),); listput(v,k-s)));
 \\  fromdigits(concat(apply(len->vector(len+1,i,i>1), vecsort(v))), 2);
}
{
  check_equal(vector(#A161820_samples,n,n--; A161820_compact(n)),
              vector(#A161820_samples,n,n--; run_lengths_minimum(n)),
              "A161820_compact() vs run_lengths_minimum()");
}


\\-----------------------------------------------------------------------------
\\ Jacobsthals Bits

{
  check_equal(vector(256,n,n--; bittest(J(n),0)),
              vector(256,n,n--; if(n==0,0,
                                   1)),
              "Jacobsthal bit 0");

  check_equal(vector(256,n,n--; bittest(J(n),1)),
              vector(256,n,n--; if(n==1,   0,
                                   n%2==0, 0,
                                   n%2==1, 1,
                                   error())),
              "Jacobsthal bit 1");

  check_equal(vector(256,n,n--; bittest(J(n),2)),
              vector(256,n,n--; if(n==0,0,
                                   n==2,0,
                                   n%2==0, 1, 0)),
              "Jacobsthal bit 2, is 1 when n even, except n=0,2");
  check_equal(vector(256,n,n--; J(n)%8),
              vector(256,n,n--; if(n==0,0,
                                   n==1,1,
                                   n==2,   1,    \\ even
                                   n%2==1, 3,    \\ odd
                                   n%2==0, 5,    \\ even
                                   error())),
              "Jacobsthal mod 8, is 3 or 5 or 1");

  check_equal(matrix(2,2,r,c, bittest(if(r==1,3,5) * if(c==1,3,5), 2)),
              [0,1;     \\ *3 unchanged
               1,0]);   \\ *5 toggle
}

{
  \\ Chi bits
  check_equal(vector(256,n,n--; bittest(Chi(n),0)),
              vector(256,n,n--; 1),
              "Chi bit 0");

  check_equal(vector(256,n,n--; bittest(Chi(n),1)),
              vector(256,n,n--; if(n%2==0, 0,
                                   n%2==1, 1,
                                   error())),
              "Chi bit 1");

  check_equal(vector(256,n,n--; bittest(Chi(n),2)),
              vector(256,n,n--; if(n==0,   0,
                                   n%2==0, 1,
                                   n%2==1, 0,
                                   error())),
              "Chi bit 2, is 1 when n even, except n=0,2");
  check_equal(vector(256,n,n--; Chi(n)%8),
              vector(256,n,n--; if(n==0,   1,    \\ even
                                   n%2==1, 3,    \\ odd
                                   n%2==0, 5,    \\ even
                                   error())),
              "Chi mod 8, is 3 or 5 or 1");

  check_equal(matrix(2,2,r,c, bittest(if(r==1,3,5) * if(c==1,3,5), 2)),
              [0,1;     \\ *3 unchanged
               1,0]);   \\ *5 toggle
}


\\-----------------------------------------------------------------------------
\\ T() and GRS


ThueMorse(n) = hammingweight(n) % 2;
{
  check_equal(vector(#A010060_samples,n,n--; ThueMorse(n)),
              A010060_samples,  \\ OFFSET=0
              "ThueMorse() = A010060");
}

Count11Pairs(n) = hammingweight(bitand(n,n>>1));
{
  check_equal(vector(#A014081_samples,n,n--; Count11Pairs(n)),
              A014081_samples,  \\ OFFSET=0
              "Count11Pairs() = A014081");
}

GRS01(n) = Count11Pairs(n) % 2;
{
  check_equal(vector(#A020987_samples,n,n--; GRS01(n)),
              A020987_samples,  \\ OFFSET=0
              "GRS01() = A020987");
}

\\ A003188
Gray(n) =
{
  n>=0 || error();
  bitxor(n,n>>1);
}
check_equal(vector(#A003188_samples,n,n--; Gray(n)), A003188_samples);

\\ A006068
UnGray(g) =
{
  g>=0 || error();
  my(v=binary(g),r=0);
  for(i=2,#v,  \\ high to low
      v[i] = bitxor(v[i],v[i-1]));
  fromdigits(v,2);
}
check_equal(vector(#A006068_samples,n,n--; UnGray(n)), A006068_samples);

\\ A101080
hammingdistance(x,y) = hammingweight(bitxor(x,y));
{
  check_equal(test_antidiagonals_of_func(#A101080_samples,hammingdistance,0,0),
              A101080_samples,
              "hammingdistance() array");
}

\\ n    ...10000   1xxxx
\\ n-1  ...01111   1xyxy    low zeros 1010 instead of normal UnGray 1111
\\
A066194(n) = UnGray(n-1)+1;
{
  check_equal(vector(#A066194_samples,n, A066194(n)),
              A066194_samples,  \\ OFFSET=1
              "A066194() vs A066194_samples");
}
\\ {
\\   my(n=from_binary(111000111000000));
\\   printf("%10s\n", test_to_binary(n));
\\   printf("%10s\n",test_to_binary(UnGray(n)));
\\   printf("%10s\n",test_to_binary(A066194(n)));
\\   print();
\\   quit;
\\ }

\\ UnGray count low 0s,
\\ count low 0s, +1,  when even number of 1 bits
\\
A277808(n) = if(hammingweight(n)%2==1,0, 1+valuation(n,2));
{
  check_equal(vector(#A277808_samples,n, A277808(n)),
              A277808_samples,  \\ OFFSET=1
              "A277808() vs A277808_samples");
  check_equal(vector(256,n, A277808(n)),
              vector(256,n, valuation(UnGray(n), 2)),
              "A277808() vs count low 0s of UnGray");
}

A268726(n) = valuation(UnGray(n)+1, 2);
{
  check_equal(vector(#A268726_samples,n,n--; A268726(n)),
              A268726_samples,  \\ OFFSET=0
              "A268726() vs A268726_samples");
}

A268727(n) = A268726(n) + 1;     \\ 1-based
{
  check_equal(vector(#A268727_samples,n,n--; A268727(n)),
              A268727_samples,  \\ OFFSET=0
              "A268727() vs A268727_samples");
}

\\ Gray, UnGray offset perm
A268717(n) =
{
  n>=0 || error();
  if(n==0,0, Gray(1+UnGray(n-1)));
}
{
  check_equal(vector(#A268717_samples,n,n--; A268717(n)),
              A268717_samples,  \\ OFFSET=0
              "A268717() vs A268717_samples");

  \\ Antti Karttunen in A268717
  check_equal(vector(256,n,n--; hammingdistance(n,A268717(n+1))),
              vector(256,n,n--; 1),
              "A268717(n+1) one bit toggled from n");

  \\ Antti Karttunen in A268717, bit toggled position
  check_equal(vector(256,n,n--; logint(bitxor(n,A268717(n+1)),2)),
              vector(256,n,n--; A268726(n)),
              "A268717(n+1) one bit toggled from n, position A268726");

  \\ for(n=256,280,
  \\    printf("%10s\n", test_to_binary(n));
  \\    printf("%10s\n", test_to_binary((1<<A268726(n))));
  \\ );
  \\ quit;
}

\\ triple steps of A268717

A268823(n) =
{
  n>=0 || error();
  if(n==0,0,
     n==1,1,
     A268717(1 + A268717(1 + A268717(n-2))));
}
{
  check_equal(vector(#A268823_samples,n,n--; A268823(n)),
              A268823_samples,  \\ OFFSET=0
              "A268823() vs A268823_samples");
}

A268676(n) =
{
  n>=0 || error();
  hammingdistance(n, A268823(n+3));
}
{
  check_equal(vector(#A268676_samples,n,n--; A268676(n)),
              A268676_samples,  \\ OFFSET=0
              "A268676() vs A268676_samples");

  \\ Antti Karttunen apparent 3,1 as ThueMorse
  check_equal(vector(1024,n,n--; A268676(n)),
              vector(1024,n,n--; if(ThueMorse(n),3,1)),
              "A268676() vs ThueMorse()");
}

{
  \\ T(n) bits
  for(n=0,1024,
     my(t=T(n));
     check_equal(bittest(t,0), 1,            "T bit 0 = 1");
     check_equal(bittest(t,1), ThueMorse(n), "T bit 1 = ThueMorse");
     check_equal(t%4, A268676(n),            "T bits 1,0 = A268676");
     check_equal(bittest(t,2), GRS01(n),     "T bit 2 = GRS01");
  );
}

\\ vector(50,n,n--;bittest(T(n),0))
\\ vector(150,n,n--;bittest(T(n),1))
\\vector(50,n,n--;bittest(T(n),3))
\\ vector(50,n,n--;T(n)%8)
\\ quit
\\ vector(50,n,J(n)%4)
\\ A153284 a(n) = n + sum((-1)^(j))*a(j); for j=1 to n-1; with a(1)=1.
\\ A153284 ,1,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,
\\ quit
\\ vector(20,n,T(n))
\\ vector(20,n,T(n)%4)
\\ vector(20,n,(T(n)\4)%2)
\\ vector(20,n,GRS01(n))
\\ apply(n->n==1,[3, 3, 1, 3, 1, 1, 3, 3, 1, 1, 3, 1, 3, 3, 1, 3, 1, 1, 3, 1])
\\ A268676 a(n) = A101080(n,A268823(3+n)), where A101080(x,y) gives the Hamming distance between binary expansions of x and y.
\\ A268823 Permutation of nonnegative integers: a(0) = 0, a(n) = A268717(1 + A268821(n-1)).


\\-----------------------------------------------------------------------------
progress("C by interval log(C) and exp()");

{
  if(0,
     my(target_bits=512);

     my(t_log43 = interv_sumalt(t-> -1/(t*(-3)^t),
                               ceil(target_bits*log(2)/log(3)),
                               1)); \\ n_start
     t_log43=interv_widen_bits(t_log43,target_bits);
     print("log(4/3) ",t_log43*1.0);
     print("  ",interv_frac_digits(t_log43));
     my(t_other = interv_sumalt(t-> 1/(t*((-2)^(t+1) + 1)),
                               target_bits,
                               1)); \\ n_start
     t_other=interv_widen_bits(t_other,target_bits);
     print("log other ",t_other*1.0);
     print("  ",interv_frac_digits(t_other));

     my(t=t_log43 + t_other);
     print("log(C) ",t*1.0," digits ",#interv_frac_digits(t));
     print("  "interv_frac_digits(t));

     t=interv_exp(t,ceil(target_bits*.3));
     print(t*1.0);
     print("C digits ",#interv_frac_digits(t));
     my(v=interv_frac_digits(t,,87));
     for(i=1,#v,print1(v[i],if(i==#v,"",","));if(i==29||i==58,print()));print();
     print(v[1]"."fromdigits(v[2..#v]));
    );
}

\\------------------------------------------------------------------------------
progress("end");
