\\ Copyright 2018, 2019, 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Tests related to
\\     - trees with childless vertices all at odd depths
\\     - trees with childless vertices all at even depths
\\     - free trees with all leaves in the same bipartite half

default(recover,0);
default(strictargs,1);

option_verbose=0;
option_progress=1;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");


\\ Return 0 if all childless vertices in vpar are at even depths.
\\ Return 1 if all childless vertices in vpar are at odd depths.
\\ Return 'none if a mixture of odd and even.
test_childless_depths_parity(vpar) =
{
  my(depths=vpar_depths_vector(vpar),
     childless=vpar_childless_vertices(vpar));
  for(want=0,1,
     for(i=1,#childless,
        if(depths[childless[i]]%2 != want,
           next(2)));
     return(want));
  'none;
}
check_equal(test_childless_depths_parity([0]),     0);
check_equal(test_childless_depths_parity([0,1]),   1);
check_equal(test_childless_depths_parity([0,1,0]), 'none);

{
  if(0,
     my(n=5);
     for(want=0,1,
        print("want "want);
        my(vpar=vpar_unlabelled_first(n));
        until((vpar=vpar_unlabelled_next(vpar))=='none,  \\ trees
              if(test_childless_depths_parity(vpar)==want,
                 print(vpar)))));
}


\\-----------------------------------------------------------------------------
progress("Unlabelled Trees Childless at Even or Odd");

\\ Return a vector of vectors [ [evens counts, ...], [odds counts, ...] ].
\\ Each of the two vectors v[n] = number of rooted unlabelled trees of n
\\ vertices with all childless vertices at even depths for the evens vector,
\\ or at odd depths for the odds vector.
\\
\\ This implementation poor due to repeat transforming.
\\
test_count_unlabelled_childless_parity_vectors(n) =
{
  my(evens=[],
     odds=[]);
  for(i=1,n,
     my(new_odds =concat(0, vpar_INTERNAL_Euler_transform(evens)),
        new_evens=concat(1, vpar_INTERNAL_Euler_transform(odds)));
     evens=new_evens;
     odds=new_odds);
  return([evens,odds]);
}
{
  check_equal(test_count_unlabelled_childless_parity_vectors(1),
              [ [1], [0] ]);
  check_equal(test_count_unlabelled_childless_parity_vectors(2),
              [ [1,0], [0,1] ]);
  check_equal(test_count_unlabelled_childless_parity_vectors(5),
              [[1,0,1,1,3],    \\ even
               [0,1,1,2,3] ]); \\ odd
  \\        n = 1 2 3 4 5
  \\
  \\        n=1  n=2     n=2     n=2
  \\ empty   *    *       *       *
  \\              |       |      / \
  \\              * ODD   *     *   * ODD
  \\                      |
  \\                      * EVEN
}

{
  my(want_evens=[
 0,1,0,1,1,3,4,10,17,38,73,158,324,700,1483,3224,6979,15300,33571,74219,
 164476,366302,817999,1833280,4119266,9281867,20962757,47453359,107637494,
 244630449,556964670,1270218355,2901393727,6637071449,15203568955,34872363374
                 ],
     want_odds=[
 0,0,1,1,2,3,6,11,22,43,89,183,384,812,1738,3742,8125,17735,38941,85898,
 190328,423320,944933,2115941,4752138,10701191,24157460,54658278,123930534,
 281546031,640785749,1460879893,3335858947,7628666743,17470228499,40060975624
               ]);
  check_equal(#want_evens,#want_odds);
  my(got=test_count_unlabelled_childless_parity_vectors(#want_evens-1));
  check_equal(concat(0,got[1]), want_evens);
  check_equal(concat(0,got[2]), want_odds);
}

A337089_A337090_vectors_compact(len) =
{
  my(evens=vector(len), ec=vector(len)); evens[1]=1;
  my(odds=vector(len),  oc=vector(len));
  for(n=1,len-1,
    ec[n] = sumdiv(n,d, d*evens[d]);
    oc[n] = sumdiv(n,d, d*odds[d]);
    evens[n+1] =          sum(k=1,n,   oc[k]*evens[n+1-k])  /n;
    odds[n+1]  = (ec[n] + sum(k=1,n-1, ec[k]* odds[n+1-k])) /n);
  return([evens,odds]);
}

{
  \\ A337089 b-files
  \\ A337090
  \\ size cf A304867 ~/OEIS/b304867.txt to 500
  \\ 
  my(limit=600);
  print("limit "limit);
  my(both_vecs=A337089_A337090_vectors_compact(limit),
     even_vec, odd_vec);
  [even_vec,odd_vec] = both_vecs;
  even_vec=concat(0,even_vec);    \\ with initial a(0)=0 term in each
  odd_vec =concat(0,odd_vec);

  \\ 0, 1, 0, 1, 1, 3, 4, 10, 17  offset 0
  check_equal(even_vec[1..#OEIS_data("A337089")],
              OEIS_data("A337089"));
  \\ 0, 0, 1, 1, 2, 3, 6, 11, 22  offset 0
  check_equal(odd_vec[1..#OEIS_data("A337090")],
              OEIS_data("A337090"));
  if(0,
     check_equal(#even_vec, limit+1);
     check_equal(#odd_vec,  limit+1);
     print("  max "even_vec[#even_vec]);
     print("      "odd_vec[#odd_vec]);
     print("  digits "#digits(even_vec[#even_vec]));
     print("         "#digits(odd_vec[#odd_vec]));

     system("rm /tmp/b337089.txt");
     system("rm /tmp/b337090.txt");
     for(i=1,#even_vec,my(n=i-1); write("/tmp/b337089.txt", n," ",even_vec[i]));
     for(i=1,#odd_vec, my(n=i-1); write("/tmp/b337090.txt", n," ",odd_vec[i]));
     system("ls -l /tmp/b3370*.txt");
     );

  my(g=OEIS_data_gf("A337089",0));
  print("~/OEIS/b337089.txt A337089 bfile poly degree ",poldegree(Pol(g)),"  terms ",g+O(x^7));
  check_equal(g == Polrev(even_vec), 1, "b337089.txt");
  my(g=OEIS_data_gf("A337090",0));
  print("~/OEIS/b337090.txt A337090 bfile poldegree ",poldegree(Pol(g))," terms  ",g+O(x^7));
  check_equal(g == Polrev(odd_vec), 1,  "b337090.txt");
}


\\ Andrew Howroyd in A304867
EulerT(v) = Vec(exp(x*Ser(dirmul(v, vector(#v, n, 1/n))))-1, -#v);
c(n) = {
   my(v=[1]);
   for(i=1,ceil(n/2),
       v=concat(1, EulerT(concat(0, EulerT(v))))); v;
}

{
  \\ versus search of all rooted unlabelled trees
  my(len=10,
     both_vecs=test_count_unlabelled_childless_parity_vectors(len),
     even_vec, odd_vec);
  [even_vec,odd_vec] = both_vecs;
  my(A337089=(n)->even_vec[n]);
  my(A337090=(n)->odd_vec[n]);
  for(n=1,min(12,len),
     my(vpar=vpar_unlabelled_first(n),
        evens=0, odds=0);
     until((vpar=vpar_unlabelled_next(vpar))=='none,  \\ trees
           my(t=test_childless_depths_parity(vpar));
           if(t==0, evens++,
              t==1, odds++));
     check_equal(evens, even_vec[n]);
     check_equal(odds,  odd_vec[n]));
}

{
  my(len=50,
     both_vecs=test_count_unlabelled_childless_parity_vectors(len),
     even_vec, odd_vec);
  [even_vec,odd_vec] = both_vecs;
  my(A337089=(n)->even_vec[n]);
  my(A337090=(n)->odd_vec[n]);

  check_equal(A337089_A337090_vectors_compact(len),
              both_vecs,
              "A337089_A337090_vectors_compact()");

  \\    [1, 0, 1, 1, 3, 4, 10, 17, 38, 73]
  \\    [0, 1, 1, 2, 3, 6, 11, 22, 43, 89]
  \\ n = 1  2  3  4  5  6   vertices
  \\
  \\ print(even_vec);
  \\ print(odd_vec);
  \\ A337089 = 1, 3, 4, 10, 17, 38, 73, 158, 324
  \\ A337090 = 2, 3, 6, 11, 22, 43, 89, 183, 384
  \\ print(even_vec + odd_vec);
  \\ not in OEIS: 3, 6, 10, 21, 39, 81, 162, 341, 708

  check_equal(even_vec,
              concat(1, vpar_INTERNAL_Euler_transform(odd_vec[^#odd_vec])));
  check_equal(odd_vec,
              concat(0, vpar_INTERNAL_Euler_transform(even_vec[^#even_vec])));

  my(geven=x*Ser(even_vec),  \\ first term n=1
     godd =x*Ser(odd_vec));
  for(i=1,#even_vec,
     check_equal(polcoeff(geven,i), even_vec[i]);
     check_equal(polcoeff(godd,i),  odd_vec[i]));

  \\---------
  \\ mutual generating functions
  my(L=O(x^(#even_vec+1)));
  check_equal(x*prod(k=1,#odd_vec, 1/(1-'x^k + L)^odd_vec[k]) + L,
              geven,
              "geven by product");
  check_equal(x*(-1 + prod(k=1,#even_vec, 1/(1-'x^k + L)^even_vec[k]))  + L,
              godd,
              "godd by product");

  check_equal(x*exp(sum(k=1,#odd_vec, subst(godd,'x,'x^k+L) / k)) + L,
              geven,
              "geven by exp");
  check_equal(x*(-1 + exp(sum(k=1,#even_vec, subst(geven,'x,'x^k+L) / k))) + L,
              godd,
              "godd by exp");

  \\---------
  \\ evens formula
  for(n=2,len,
     check_equal( (sum(k=1,n-1, A337089(k) * sumdiv(n-k,d, d*A337090(d))))
                  / (n-1),
                 A337089(n));

     check_equal( sum(k=1,n-1, A337089(n-k) * sumdiv(k,d, d*A337090(d)))
                  / (n-1),
                 A337089(n));
  );
  if(1,
     my(n=2,k=1);
     check_equal(n-1,1);
     check_equal(n-k,1);
     check_equal(A337089(2), 0);
     check_equal(A337089(n-k), 1);
     check_equal(divisors(k), [1]);
     check_equal(A337090(1), 0));

  \\ odds formula
  for(n=2,len,
     check_equal(sum(k=1,n-1, (A337090(k) + if(k==1,1)) * sumdiv(n-k,d, d*A337089(d)))
                 /(n-1),
                 A337090(n));

     check_equal( (1/(n-1)) * sum(k=1,n-1, (A337090(n-k) + if(k==n-1,1)) * sumdiv(k,d, d*A337089(d))),
                 A337090(n));
     check_equal((sumdiv(n-1,d, A337089(d)*d) + sum(k=1,n-2, A337090(n-k) * sumdiv(k,d, d*A337089(d))))/ (n-1),
                 A337090(n));
  );

  check_equal(A337089(1), 1);
  check_equal(A337090(1), 0);

  \\ evens formula for n+1 instead
  \\ k=1; n+1-k == n
  \\ k=n; n+1-k == 1
  for(n=1,len-1,
     check_equal( (sum(k=1,n, A337089(k) * sumdiv(n+1-k,d, d*A337090(d))))
                 / n,
                 A337089(n+1));
     check_equal( (sum(k=1,n, A337089(n+1-k) * sumdiv(k,d, d*A337090(d))))
                 / n,
                 A337089(n+1));

     my(a=odd_vec,
        c=vector(#a-1,k, sumdiv(k,d, d*A337090(d) )),
        b=vector(#a-1));
     for(n=1,#b, b[n] = (c[n] + sum(k=1,n-1, c[k]*b[n-k]))/ n);
     check_equal(concat(1,b), even_vec);
  );

  \\-------------------
  \\ vs Andrew Howroyd's c(n) = even
  \\ and          EulerT(c(n)) = odd
  check_equal(c(len)[1..len],                   even_vec, "c(len)");
  check_equal(concat(0,EulerT(c(len)))[1..len], odd_vec, "EulerT(c(len))");
}
{
  \\ speed comparison with c(n)
  if(0,
     gettime(); c(200); print("c ",gettime());
     gettime(); A337089_A337090_vectors_compact(200); print("A ",gettime());
     quit);
}
{
  \\ ec and oc values
  \\ do they mean anything?
  \\ not in OEIS: 1, 1, 4, 5, 16, 28, 71, 141, 346, 746
  \\ not in OEIS:    2, 3, 10, 15, 41, 77, 186, 390, 907
  if(0,
     my(len=15,
        both_vecs=test_count_unlabelled_childless_parity_vectors(len),
        even_vec, odd_vec);
     [even_vec,odd_vec] = both_vecs;
     print(vector(10,n,  sumdiv(n,d, d*even_vec[d])));
     print(vector(10,n,  sumdiv(n,d, d*odd_vec[d])));
     quit;
     );
}




{
  if(0,
     print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
     my(len=35,
        both_vecs=test_count_unlabelled_childless_parity_vectors(len),
        even_vec, odd_vec);
     [even_vec,odd_vec] = both_vecs;
     print1("A337089 evens\n%S ");
     for(n=0,len,
        my(c=if(n==0,0, even_vec[n]));
        if(n==20,print1("\n%T "), n==29,print1("\n%U "));
        print1(c, if(n==len,"",",")));
     print();
     print();

     print1("A337090 odds\n%S ");
     for(n=0,len,
        my(c=if(n==0,0, odd_vec[n]));
        if(n==20,print1("\n%T "), n==29,print1("\n%U "));
        print1(c, if(n==len,"",",")));
     print();

     print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
     quit
    );
}
{
  \\ n=1..500 about 41kb, 1000 about 190kb
  if(0,
     my(len=500,
        both_vecs=test_count_unlabelled_childless_parity_vectors(len),
        even_vec, odd_vec);
     [even_vec,odd_vec] = both_vecs;
     system("rm -f /tmp/b970001.txt");
     for(n=1,len, write("/tmp/b970001.txt",Str(n" "even_vec[n])));

     system("rm -f /tmp/b970002.txt");
     for(n=1,len, write("/tmp/b970002.txt",Str(n" "odd_vec[n])));

     system("ls -l /tmp/b970001.txt /tmp/b970002.txt");
    );
}


\\-----------------------------------------------------------------------------
progress("Free Trees Bipartite Leaves");

\\ bileaves 1, 0, 1, 1, 2, 2, 5, 6, 13, 20, 41, 70
\\ even     1, 0, 1, 1, 3,
\\ odd      0, 1, 1, 2, 3,
\\      n = 1  2  3  4  5

\\ n=4     E      claw        E           E
\\       / | \    sole        |           |
\\      O  O  O   bileaves    O           O
\\                            |          / \
\\        odd                 E         E   E   even
\\                            |
\\                            O  odd

\\ vec[1..len] where vec[n] = count bileaves trees of n vertices
test_count_bileaves_vector(len) =
{
  \\ n=4 i=1..2  e[1]*o[3] .. e[2]*o[2]
  \\             o[1]*e[3] .. o[2]*e[2]
  \\ n=5 i=1..2  e[1]*o[4] .. e[2]*o[3]
  \\             o[1]*e[4] .. o[2]*e[3]

  my(pair=test_count_unlabelled_childless_parity_vectors(len),
     evens=pair[1],
     odds =pair[2],
     both =evens+odds);
  vector(len,n,
         both[n]         \\ unicentroidal
         - sum(i=1,n-1,  \\ unbalanced combinations
               evens[n-i]*(odds[i] + if(i==1,1))));
}
\\ n=7 vertices (n=6 edges), 5 trees
\\                                      n = 1 2 3 4 5 6 7
check_equal(test_count_bileaves_vector(7), [1,0,1,1,2,2,5]);
{
  \\ A304867 offset 0 is of n+1 vertices, so 1.. vertices
  \\ exception A304867(1) = 1 is about 2 vertices but no such bileaves tree
  my(want=A304867_samples);
  check_equal(want[2],1, "A304867(1) exception");
  want[2] = 0;
  check_equal(test_count_bileaves_vector(#want),
              want,
              "test_count_bileaves_vector() vs A304867_samples");
}
{
  \\ 1, 1, 1, 1, 2, 2, 5, 6, 13, 20, 41, 70, 144, 266, 545, 1072, 2210,
  my(len=14,
     count_uni_vec=vector(len),
     count_bi_vec=vector(len));
  for(n=1,len,
     my(count_uni=0, count_bi=0,
        vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           if(vpar_is_bileaves(vpar),
              if(#vpar_centroids(vpar)==1,
                 count_uni++,
                 count_bi++)));
     count_uni_vec[n]=count_uni;
     count_bi_vec[n]=count_bi);
  my(count_vec=count_uni_vec + count_bi_vec);

  check_equal(count_vec,
              test_count_bileaves_vector(len),
              "test_count_bileaves_vector() vs vpar_is_bileaves()");
  \\ print(count_vec);
}

\\ terms n=0 ...
A304867_vector_selfcontained(len) =
{
  my(evens=vector(len), ec=evens); evens[1]=1;
  my(odds=evens, oc=ec); \\ with odds[1]=1
  for(n=1,len-1,
    ec[n] = sumdiv(n,d, d*evens[d]);
    oc[n] = sumdiv(n,d, d*odds[d]) - 1;
    evens[n+1] = sum(k=1,n, oc[k]*evens[n+1-k]) /n;
    odds[n+1]  = sum(k=1,n, ec[k]* odds[n+1-k]) /n);
  for(n=3,len, ec[n] = evens[n]+odds[n]
                - sum(i=1,n-1, evens[n-i]*odds[i]));
  ec;
}
{
  check_equal(A304867_vector_selfcontained(#A304867_samples),
              A304867_samples,
              "A304867_vector_selfcontained() vs A304867_samples");
}

{
  \\ 1, 1, 1, 1, 2, 2, 5, 6, 13, 20, 41, 70, 144, 266, 545, 1072, 2210,
  my(len=50,
     L = O(x^len),
     pair=test_count_unlabelled_childless_parity_vectors(len),
     even=pair[1],
     odd=pair[2],
     both=even+odd,
     geven=x*Ser(even),
     godd =x*Ser(odd),
     gboth=geven+godd,
     bileaves=test_count_bileaves_vector(len),
     gbileaves=x*Ser(bileaves) + 1);  \\ including empty tree

  my(evenf=(n)->if(n==0,0, even[n]),
     oddf=(n)->if(n==0,0, odd[n]));
  check_equal(evenf(8), 17, "evenf(8)");
  check_equal(oddf(8),  22, "oddf(8)");

  my(vA304867=A304867_vector_selfcontained(len),
     gA304867=Ser(vA304867),
     A304867f=(n)->vA304867[n+1]);
  check_equal(vA304867[1..#A304867_samples], A304867_samples);
  check_equal(gA304867 == Ser(A304867_samples), 1);
  check_equal(A304867f(6), 5);

  check_equal(vector(len,n, 
                     even[n] + odd[n]  \\ unicentroidal
                     - sum(i=1,n-1,    \\ unbalanced combinations
                           even[n-i]*(odd[i] + if(i==1,1)))),
              bileaves,
              "bileaves 1");
  
  \\ i=n case is 0 anyway
  check_equal(bileaves,
              vector(len,n, 
                     even[n] + odd[n]  \\ unicentroidal
                     - sum(i=1,n,    \\ unbalanced combinations
                           evenf(n-i)*(odd[i] + if(i==1,1)))));
  
  \\ A304867 a(n-1)     evens makes n vertices, n-1 edges
  check_equal(vector(len-1,n,
                     if(n==2, 1,  \\ A304867 exception
                        evenf(n) + oddf(n)
                        - evenf(n-1)
                        - sum(i=2,n, evenf(n-i)*oddf(i))
                        )),
              vector(len-1,n, A304867f(n-1)),
              "A304867f() by evenf,oddf");
  
  check_equal(bileaves,
              vector(len,n, 
                     both[n]               \\ unicentroidal
                     - if(n>=2, even[n-1])
                     - sum(i=1,n>>1,       \\ unbalanced combinations
                           even[i]*odd[n-i]
                           + odd[i]*even[n-i])
                     + if(n%2==0,                \\ when n even,
                          even[n/2]*odd[n/2])));  \\   bicentroidals

  check_equal(bileaves,
              vector(len,n,
                     \\ print("n="n"  even "even[n]"  odd "odd[n]);
                     even[n]      \\ unicentroidal even
                     - sum(i=1,n>>1,
                           \\ print("  even unbalanced "i" "n-i" counts "even[i]" "odd[n-i]);
                           even[i]*odd[n-i])  \\ unbalanced combinations
  
                     + odd[n]      \\ unicentroidal odd
                     - sum(i=1,n>>1,
                           my(o=if(i==1,1, odd[i]));
                           \\ print("  odd unbalanced "i" "n-i" counts "o" "even[n-i]);
                           o*even[n-i])  \\ unbalanced combinations
                     
                     
                     + if(n%2==0,                               \\ when n even,
                          \\ print("  bicentroidal "even[n/2]" "odd[n/2]);
                          even[n/2]*odd[n/2])));          \\   bicentroidals
              

  check_equal(gboth
              + 1
              - x*geven
              - geven*godd,
              gbileaves,
              "gbileaves 1");

  check_equal(gbileaves,
              gboth
              + 1
              + (1-x)*geven
              - geven*(godd+1));

  check_equal(gbileaves,
              godd+1
              + (2-x)*geven
              - geven*(godd+1));

  check_equal(gbileaves,
              (2-x)*geven
              + (1-geven)*(godd+1));

  check_equal(gbileaves,
              (1-x-godd)*geven + godd+1);
  check_equal(gbileaves,
              godd+1 - (godd-1+x)*geven);

  \\------------
  \\ Andrew Howroyd's seq(n) code
  my(u=c(len));
  check_equal(godd/x,
              x*Ser(EulerT(u)) + L);
  check_equal(1-geven    + L,
              1-x*Ser(u) + L);
  check_equal(geven/x    + L,
              Ser(u)     + L);

  check_equal((gbileaves-1)/x + x,
              + godd/x * (1-geven)
              + (1-x)*geven/x + x,
              "seq(n)");
  \\ x*Ser(EulerT(u)) * (1-x*Ser(u)) + (1 - x)*Ser(u) + x + O(x*x^n

  my(gA304867 = (gbileaves-1)/x + x);
  check_equal(Ser(A304867_samples) + O(x^20),
              gA304867 + O(x^20));
  check_equal(gA304867,
              + godd/x * (1-geven)
              + (1-x)*geven/x + x);

  check_equal(gA304867,
              1/x - 1 + x + (geven/x-1/x) * (1-x-godd) );
  check_equal(gA304867,
              ( 1 - x + x^2 + (geven-1) * (1-x-godd) )/x );
  check_equal(gA304867,
              -1 + x + ( geven - (godd+x) * (geven-1) )/x );
  check_equal(gA304867,
              -1 + x + ( 1 - (godd+x-1) * (geven-1) )/x );
  check_equal(gA304867,
              (1/x)*( 1 - x + x^2 - (1-geven) * (1-x-godd) ) );
}


\\-----------------------------------------------------------------------------
progress("~/OEIS/b304867.txt");

{
  \\ A304867 count hypertrees weight n = free bileaves of n edges n+1 vertices
  \\ OFFSET=0
  my(v=test_count_bileaves_vector(#A304867_samples));
  v[2]++;  \\ exception at hypertree n=1 which is free n=2
  check_equal(v, A304867_samples,
              "test_count_bileaves_vector() vs A304867_samples");

  \\ ~/OEIS/b304867
  my(g=OEIS_data_gf("A304867",'bfile));
  \\ g=Pol(g + O(x^20)); \\ to truncate
  print("A304867 bfile poldegree ",poldegree(Pol(g)));

  \\ local(option_verbose=0);   \\ test_count_bileaves_vector() a bit slow
  check_equal(g ==
              Polrev(A304867_vector_selfcontained(poldegree(Pol(g))+1)),
              1,
              "A304867_vector_selfcontained() vs b304867.txt");

  check_equal(g ==
              Polrev(test_count_bileaves_vector(poldegree(Pol(g))+1))
              + x,  \\ exeption at hypertree n=1
              1,
              "test_count_bileaves_vector() vs b304867.txt");
}


\\-----------------------------------------------------------------------------
progress("end");
