\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Various "CP" functions for rooted binary trees enumberated as per Colijn
\\ and Plazzotta.
\\
\\ C. Colijn and G. Plazzotta,
\\ "A Metric On Phylogenetic Tree Shapes",
\\ Systematic Biology, volume 67, number 1,
\\ January 2018, pages 113-126, pages 113-126.
\\ https://doi.org/10.1093/sysbio/syx046


\\ return children vector [x,y], or [0,0] if no children (which is n=1)
CP_children_z(n) =
{
  n>=1 || error();
  my(R, L=sqrtint((n-1)<<1,&R));
  if(R>L,  R-=L;L++, R+=L);
  [L,R>>1];
}

\\ A002024 = 1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5,
\\       n = 1  2  3  4
A002024_k_rep_k(n) =
{
  n>=1 || error();
  my(r,s=sqrtint(n<<1,&r)); s + (r>s);
}

A002260_runs_1_to_k(n) =
{
  n>=1 || error();
  my(r,s=sqrtint(n<<1,&r));
  if(r>s, r-s, r+s) >> 1;
}

\\ Return "left" child subtree (larger) of n,
\\ or return 0 for n=1 which has no children.
\\ A002024(n-1), for n>=2
CP_child_L(n) =
{
  n>=1 || error();
  my(r,s=sqrtint((n-1)<<1,&r)); s + (r>s);
}

\\ Return "right" child subtree (smaller) of n,
\\ or return 0 for n=1 which has no children.
\\ A002260(n-1)
CP_child_R(n) =
{
  n>=1 || error();
  \\ CP_children_z(n)[2];
  my(r,s=sqrtint((n-1)<<1,&r));
  if(r>s, r-s, r+s) >> 1;
}

\\-----------------------------------------------------------------------------
\\ Number of Vertices

CP_num_vertices(n) =
{
  \\ plain recursion measuring faster than explicit stack
  n>=1 || error();
  1 + if(n>1, vecsum(apply(self(),
                          CP_children_z(n))));
}
alias(A064002_CP_num_vertices, CP_num_vertices);


CP_num_childless(n) =
{
  n>=1 || error();
  if(n==1,1,
     vecsum(apply(CP_num_childless,CP_children_z(n))));

  \\ my(stack=List([]), ret=1, r);
  \\ while(1,
  \\       while([n,r]=CP_children_z(n),
  \\             ret++;
  \\             listput(stack,r));
  \\       #stack || return(ret);
  \\       n=stack[#stack]; listpop(stack));
}
A064064_num_childless_offset(n) =
{
  n>=0 || error();
  CP_num_childless(n+1);
}

\\ Colijn and Plazzotta v_t(2) for tree t
CP_num_cherries(n) =
{
  n>=1 || error();
  if(n<=2, n==2,
     vecsum(apply(self(), CP_children(n))));
}

\\-----------------------------------------------------------------------------

CP_height(n) =
{
  n>=1 || error();
  my(ret=0); while(n=CP_child_L(n), ret++); ret;

  \\ n--;
  \\ my(ret=0,s,r);
  \\ while(n,
  \\       (sqrtint(n<<3) - 1)>>1 == (s=sqrtint(n<<1,&r)) - (r<=s) || error();
  \\       \\ n = (sqrtint(n<<3) - 1)>>1;
  \\       n=sqrtint(n<<1,&r); n-=(r<=n);
  \\       ret++);
  \\ ret;

  \\ my(stack=List([]),d=0,ret=0);
  \\ while(1,
  \\       ret=max(ret,d);
  \\       my(v=CP_children(n));
  \\       if(#v, n=v[1]; listput(stack,v[2]); d++,
  \\          #stack, n=stack[#stack]; listpop(stack); d--,
  \\          return(ret)));
}



\\-----------------------------------------------------------------------------
\\ Automorphisms

CP_num_orbits(n) =
{
  n>=1 || error();
  if(n==1,1,
     my(x,y); [x,y]=CP_children(n);
     1 + self()(x) + if(x!=y, self()(y)));
}


\\-----------------------------------------------------------------------------
