\\ Copyright 2018, 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);
read("../vpar.gp");


\\ Usage: gp <cospectral-matchnum2.gp
\\
\\ This code makes cospectral pairs of trees as from
\\
\\     Abbe Mowshowitz, "The Characteristic Polynomial of a Graph", Journal
\\     of Combinatorial Theory, series B, volume 12, 1972, pages 177-193.
\\     https://www.sciencedirect.com/science/article/pii/0095895672900238
\\
\\ Cospectral means trees (or graphs) have the same set of eigenvalues (same
\\ spectrum) of their adjacency matrices (vpar_adjacency_matrix()).  This
\\ means they have equal characteristic polynomial of those matrices
\\ (vpar_adjacency_poly()).
\\
\\ Mowshowitz shows the coefficients of the polynomial are number of
\\ matchings (described here in the vpar_adjacency_poly() addhelp string).
\\ So constructing different trees with same number of matchings gives
\\ cospectral.  Mowshowitz's trees T1 and T2 are
\\
\\        T1                T2
\\     u      v         x         y     
\\     *      *         *         *      u,v x,y            
\\      \    /           \       /       many leaf each side
\\     *-*--*-*         *-*--*--*-*
\\      /    \           /       \
\\     *      *         *         *
\\
\\ There are u,v, x,y >=1 many leaf vertices on the respective sides of the
\\ respective trees.  The extra middle in T2 means it is never isomorphic to
\\ T1.


\\ Like vpar_make_bistar() but subdivide the middle edge.
make_bistar_split_middle(a,b) =
{
  my(vpar=[0]);
  vpar = vpar_join(vpar,1,vpar_make_star(a));
  vpar = vpar_join(vpar,1,vpar_make_star(b));
  vpar;
}

T1_uv(u,v) = vpar_make_bistar(u+1,v+1);
T2_xy(x,y) = make_bistar_split_middle(x+1,y+1);


\\ The matchnum (biggest matching) in both trees is always 2.
\\ That follows from their diameter.
{
  for(u=1,6,
     for(v=1,6,
        vpar_matchnum(T1_uv(u,v)) == 2 || error()));
  for(x=1,6,
     for(y=1,6,
        vpar_matchnum(T2_xy(x,y)) == 2 || error()));
}


\\ In T1, the number of matchings of 2 pairs is u*v, as usual for a bistar.
\\
\\ In T2, the number of matchings of 2 pairs is (x+1)*(y+1) - 1 == x*(y+1) + y.
\\ This is x+1 neighbours of the left middle and y+1 neighbours of the right
\\ middle, but not both pairing with the centre vertex.
{
  for(u=1,5,
     for(v=1,5,
        my(matchpoly=vpar_matchpoly(T1_uv(u,v)));
        polcoeff(matchpoly,2) == u*v || error()));
  for(x=1,5,
     for(y=1,5,
        my(matchpoly=vpar_matchpoly(T2_xy(x,y)));
        polcoeff(matchpoly,2) == x*(y+1) + y || error()));
}


\\ Mowshowitz then takes n+1 vertices in each tree, which is total n edges,
\\ so they have the same number of 1-pair matchings.  1-pair matchings are
\\ simply how many edges.  So
\\
\\     u+v = n-1     x+y = n-2  
\\
\\ Equal number 2-pair matchings is then u*v == x*(y+1)+y.  This requires,
\\ after substituting out u and x,
\\
\\     y^2 - v^2 + (n-1)*v - (n-2)*y - (n-2) = 0
\\
\\ which for given n can be solved by taking y = v+1, giving v = (2*n-5)/3.
\\ v is an integer when n=3k+7, for k>=0.  In terms of that k then,

k_to_n(k) = 3*k+7;
k_to_y(k) = k_to_v(k) + 1;
k_to_v(k) = my(n=k_to_n(k)); (2*n-5)/3;
k_to_u(k) = my(n=k_to_n(k)); n-1 - k_to_v(k);
k_to_x(k) = my(n=k_to_n(k)); n-2 - k_to_y(k);

\\ v odd >= 3          u integers >= 3
\\ y=v+1 even >= 4     x integers >= 1
\\
vector(5,k,k--; k_to_x(k)) == [1,2,3,4,5]  || error();
vector(5,k,k--; k_to_y(k)) == [4,6,8,10,12]  || error();
vector(5,k,k--; k_to_u(k)) == [3,4,5,6,7]  || error();
vector(5,k,k--; k_to_v(k)) == [3,5,7,9,11]  || error();

T1_k(k) = T1_uv(k_to_u(k), k_to_v(k));
T2_k(k) = T2_xy(k_to_x(k), k_to_y(k));

\\ k=0 tree pair T1,T2 are 8 vertices each.
\\ 8 vertices is the smallest where cospectrals occur, and these two are the
\\ only 8 vertex cospectrals.
{
  my(k=0);
  print("k="k);
  my(vpar1=T1_k(k),
     vpar2=T2_k(k));
  print("vpar1 = "vpar1"   poly "vpar_adjacency_poly(vpar1));
  print("vpar2 = "vpar2"   poly "vpar_adjacency_poly(vpar2));
  print("free isomorphic = "vpar_is_free_isomorphic(vpar1,vpar2));
}

{  
  for(k=0,10,
     my(vpar1=T1_k(k),
        vpar2=T2_k(k));
     vpar_is_free_isomorphic(vpar1,vpar2)==0 || error();
     vpar_adjacency_poly(vpar1) == vpar_adjacency_poly(vpar2) || error();
  );
}
