\\ Copyright 2018, 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1)
read("../vpar.gp");


\\ Usage: gp <equal01.gp
\\
\\ Requires GP 2.9.x or higher for empty ranges vec[lo..hi].
\\
\\ This is some code acting on and transforming vectors containing 0s and 1s
\\ with equal numbers of 0s and 1s.  Each 1 is reckoned an up-step and each
\\ 0 a down-step so for example
\\
\\        /\
\\     _ /  \/\    /\ _       __ "zero" line
\\             \/\/
\\       110010010110         equal01 vector,  k=2 flaws
\\
\\ Such a vector is not necessarily balanced (cf vpar_is_balanced_binary())
\\ because it can go below the zero line.
\\
\\ The number of vectors is
\\
\\     equal01   = binomial(2*n,n), choose n 1s among 2n positions
\\     balanced  = binomial(2*n,n)/(n+1) = Cat(n) = Catalan numbers
\\                                       = vpar_count_ordered(n,1)
\\
\\ Factor n+1 between these hints at possible relation.  A theorem of
\\ MacMahon shows that the number of equal01 with k many "flaws" is equal to
\\ Cat(n).  A "flaw" means a 0 located below the zero line.
\\
\\     P.A. MacMahon, "Memoir on the Theory of the Partitions of Numbers -
\\     Part IV", Philisophical Transactions of the Royal Society of London
\\     (A), volume 209, 1909, pages 153-175.  Reprinted in Collected Works.
\\     https://www.jstor.org/stable/91034
\\     http://rsta.royalsocietypublishing.org/content/209/441-458/153
\\       (demands cookies to track you)
\\
\\ This is known as the Chung-Feller theorem for their reaching the same
\\ result when tossing coins which end in equality (same heads as tails).
\\
\\     Kai Lai Chung and W. Feller, "On Fluctuations In Coin-Tossing",
\\     Proceedings of the National Academy of Sciences of the United States
\\     of America, volume 35, number 10, 15 October 1949, pages 605-608.
\\     https://www.jstor.org/stable/88260
\\
\\ Various authors give actual one-to-one transformations of equal01
\\ increasing or decreasing its "k" many flaws.  Hodges (and found also by
\\ Chen) steps by 1, Mutze, Standke and Wiechert step by 1 by a minimum
\\ change, and Atkinson and Sack step all the way to balanced k=0.
\\
\\ The interest for vpar is that a random equal01 vector is quite easily
\\ generated, and it can may be turned into a balanced vector somehow, and
\\ such a balanced corresponds to an ordered forest.  Atkinson and Sack
\\ would be closest to suiting such a thing.  The code here is just
\\ transformations (without going on to ordered vpars).
\\


\\-----------------------------------------------------------------------------
\\ Equal 0,1 Vector Iteration
\\
\\ This is a simple iterator for vectors containing equal number of 0s and 1s.
\\ The lex() next vector if formed by transforming lowest run of 1s,
\\
\\     ... 0 1111 0000000    vec
\\     ... 1 00000000 111    next
\\
\\ The top 1 of the run moves up 1 place.  The rest drop down to the bottom,
\\ which is the end of the vector.  (Some bit twiddling could do this in an
\\ integer if desired.)
\\
\\ For interest, iterating balanced binary is similar.  The low run highest
\\ 1 moves up, but the balance condition demands the low bits are set
\\ alternating 101010 instead of 000111.


\\ vec is a vector of even length with equal number of 0 and 1 entries.
\\ Return the lexicographically next higher such.
\\ The first such is equal01_first(n).
equal01_next(vec) =
{
  forstep(h=#vec,1,-1,
     if(vec[h],                    \\ last 1
        forstep(l=h,1,-1,
           if(vec[l],
              vec[l] = 0;          \\ move 1 to end of vec
              vec[l+#vec-h] = 1;
              ,
              vec[l]=1;            \\ except top of run moves 1 place
              vec[l+1+#vec-h]=0;   \\ towards start of vec
              return(vec)));
        break));  \\ vec == 111000, run of 1s is all at start, nothing more
  'none;
}
equal01_first(n) = vector(2*n,i,i>n);

{
  \\ check equal01_next() versus forvec() filtered by hammingweight()
  for(n=0,6,
     my(by_next=List([]),
        vec=equal01_first(n));
     until((vec=equal01_next(vec)) == 'none,
           listput(by_next,vec));
     #by_next==binomial(2*n,n) || error();

     my(by_filter=List([]));
     forvec(vec=vector(2*n,i,[0,1]),
           if(hammingweight(vec)==n, listput(by_filter,vec)));
     by_next == by_filter || error());
}


\\-----------------------------------------------------------------------------
\\ Equal 0,1 to Balanced
\\
\\ Atkinson and Sack give a transformation from a vector of an equal01
\\ vector to a balanced vector,
\\
\\     M.D. Atkinson and J.R. Sack, "Generating Binary Trees At Random",
\\     Information Processing Letters, volume 41, 1992, pages 21-23
\\     http://www.cs.otago.ac.nz/staffpriv/mike/Papers/RandomGeneration/RandomBinaryTrees.pdf
\\
\\ They call equal01 "balanced", and balanced (ie. nested) "well formed".
\\ But here stay with balanced for nested per vpar_is_balanced_binary().
\\
\\ Atkinson and Sack note their transformation is a way to generate a random
\\ binary tree, or equivalently an ordered tree.  A random equal01 can be
\\ made by each position getting a chance of 1 or 0 according to how many 1s
\\ and 0s not yet placed (eg. Knuth volume 2 section 3.4.2 algorithm S,
\\ second edition page 137).  They note this requires numbers only up to 2n,
\\ as opposed to Arnold and Sleep needing n^2 (which is used in the current
\\ vpar_make_random_preorder() implementation).


\\ equal01_to_balanced() is the algorithm of Atkinson and Sack to go from an
\\ equal01 vector to a balanced vector.  The equal01 vector begins with a
\\ shortest nested block,
\\
\\       .....                               First balanced run of vec,
\\      /     \        OR                    being first return to the 
\\      1xxxxx0 rest        0xxxxx1 rest     zero line.  The first step
\\                          \     /          may go up or down.
\\                           .....             
\\
\\ The block can begin with 1 or 0.  Taking those as +1 step up, and -1 step
\\ down, the block ends at the steps first return to the zero line.
\\
\\ An up run is unchanged and a down run is changed:
\\
\\     Transform(1xxxxx0 rest)  =  1xxxxx0 Transform(rest)
\\     Transform(0xxxxx1 rest)  =  1 Transform(rest) 0 Flip(xxxxx)
\\
\\ An up block is 1xxxxx0 and it is already balanced and is unchanged.
\\ A down block is 0xxxxx1.  Flip() means each term changed 1<->0.
\\ Flip(xxxxx) gives a balanced block because xxxxx preceded by 0 first
\\ element.
\\
\\ In the code, s is location to store to the start of ret[] and e is
\\ location to store to the end of ret[].  Up copies its 1xxxxx0 to the
\\ start of ret.  Down puts 1 at ret start and 0 Flip(xxxx) at end.

equal01_to_balanced(vec) =
{
  my(d_start=1,
     s=0,
     e=#vec+1,
     ret=vector(#vec));
  for(i=1,#vec,
     d += if(vec[i],1,-1);
     if(d==0,
        if(vec[d_start],   \\ up
           for(j=d_start,i, ret[s++] = vec[j]);  \\ copy
           ,
           ret[s++] = 1;   \\ down
           forstep(j=i-1,d_start+1,-1, ret[e--] = !vec[j]);  \\ flip
           ret[e--] = 0);
        d_start=i+1));
  d==0 || error;
  ret;
}

{
  my(n=2);
  print("n="n" equal01 vectors to balanced vectors");
  my(vec=equal01_first(n));
  until((vec=equal01_next(vec)) == 'none,
        my(bal=equal01_to_balanced(vec));
        print(vec" -> "bal));
}


\\ is_balanced_binary_vec(vec) is similar to vpar_is_balanced_binary(), but
\\ acts on a vector of 1s and 0s.  Treating vec entries as +1 and -1, the
\\ running total at all positions must stay >=0.
is_balanced_binary_vec(vec) =
{
  for(i=1,#vec,
     d += if(vec[i],1,-1);
     if(d<0, return(0)));
  d==0;
}

{
  \\ validate each balanced vector made by equal01_to_balanced(),
  \\and check that each arises from n+1 input equal01 vectors
  for(n=1,5,
     my(vec=equal01_first(n),
        bals=List([]),      \\ balanced binary vectors
        counts=List([]));   \\ how many of each of those
     until((vec=equal01_next(vec)) == 'none,
           my(bal=equal01_to_balanced(vec));
           is_balanced_binary_vec(bal) || error();

           my(pos=setsearch(bals,bal));
           if(!pos,
              pos=setsearch(bals,bal,1);
              listinsert(bals,bal,pos);
              listinsert(counts,0,pos));
           counts[pos]++);

     for(i=1,#counts,
        counts[i]==n+1 || error()));
}
print();


\\-----------------------------------------------------------------------------
\\ Equal 0,1 Belows

\\ vec is a vector of equal numbers of 0s and 1s.
\\ The 1s are reckoned up steps and 0s down steps.
\\ Return the number of down steps which go to below the zero line.
\\ For example
\\          /\            above
\\    ___  /  \    ______ zero
\\       \/    \  /       below
\\              \/
\\       0111000011
\\       ^     ^^        3 downs going  below the zero
\\
equal01_num_below(vec) =
{
  my(d=0, num_below=0);
  for(i=1,#vec,
     if(vec[i],
        d++,
        d--; if(d<0, num_below++)));     
  num_below;
}

\\ Catalan numbers binomial(2n,n) / (n+1) = OEIS A000108
Catalan_number(n) = binomial(n<<1,n) / (n+1);

\\ Per MacMahon, and Chung and Feller, the number of equal01 vectors with a
\\ given "num_below" is the same (Catalan number)
{
  print("num below histogram, 0 to n");
  for(n=0,6,
     my(vec=equal01_first(n),
        histogram=vector(n+1));
     until((vec=equal01_next(vec)) == 'none,
           my(num_below=equal01_num_below(vec));
           histogram[num_below+1]++);
     print("n="n"  "histogram);
     
     \\ all counts the same
     histogram[1]==Catalan_number(n) || error();
     for(i=2,#histogram, 
        histogram[i]==histogram[1] || error());
  );
}

\\ "func" is a function such as equal01_decrease_below().
\\ Check that it acts on an equal01 vector so as to decrease
\\ equal01_num_below() by 1, and that its output has no duplicates.
check_decrease_below(equal01_decrease_below,name) =
{
  for(n=0,6,
     my(vec=equal01_first(n),
        l=List());
     until((vec=equal01_next(vec)) == 'none,
           my(num_below=equal01_num_below(vec),
              decr_vec=equal01_decrease_below(vec));
           if(num_below==0,
              decr_vec=='none || error();
              ,
              decr_vec!='none || error();
              equal01_num_below(decr_vec) == num_below-1 || error();
              listput(l,decr_vec)));
     #l==#Set(l)  || error();   \\ all decreases are distinct
     #l == n/(n+1)*binomial(2*n,n) || error();
  );
  \\ print("decr checks ok: "name);
}


\\-----------------------------------------------------------------------------
\\ Equal 0,1 Belows Increase -- Hodges

\\ Hodges gives a transformation reducing the "num below" of an equal01,
\\
\\     J. L. Hodges, Jr., "Galton's Rank-Order Test", Biometrika,
\\     volume 42, number 1/2, June 1955, pages 261-262.
\\     http://www.jstor.org/stable/2333442
\\
\\ Hodges counts 0 negative and 1 positive and acts to reduce the number of
\\ steps above the zero line.  It works just as well to flip the above/below
\\ zero sought and so reduce the equal01_num_below() instead.


\\ The following is a simple implementation of Hodges' transformation T.
\\ Input vec is parsed to "AbbCxxxx".
\\ AbbC is non-empty and has C as the first return to zero.
\\ This can be either an excursion above zero or below zero.
\\ xxxx is the rest of vec, possibly empty if vec is single block.
\\
\\ Output is then 
\\     'none            if all AbbCxxxx has nothing below zero line
\\                         (so no belows to decrease)
\\     AbbC T(xxxx)     if xxxx has any below
\\     1 xxxx 0 bb      if xxxx has no below
\\ 
\\ If "xxx no below" then the below of the whole vec is due to AbbC, by
\\ being an excursion below zero, so A=0 and B=1.  The output then strips
\\ those A and B so the remaining bb is 1 fewer below pair.  New initial
\\ 1xxxx0 is all above the zero line still.
\\
equal01_decrease_below(vec) =
{
  if(equal01_num_below(vec) == 0, return('none));

  my(d=0, k=-1);
  for(i=1,#vec,
     if(vec[i], d++, d--);
     if(d==0, k=i; break)); \\ first return to the zero line

  my(after=vec[k+1..#vec]);
  if(equal01_num_below(after) > 0,
     concat(vec[1..k], equal01_decrease_below(after)),  \\ recurse
     concat([1, after, 0, vec[2..k-1]]));
}
check_decrease_below(equal01_decrease_below, "Hodges 1");

\\ Although Hodges expresses T recursively (inductively), the effect is
\\ simply to find AbbC as the last excursion below the zero line.  This
\\ might be the whole of vec, or there might be some above-zero blocks after.
\\ The output swaps that last below block with what follows..  The initial
\\ parts are unchanged.
\\
equal01_decrease_below(vec) =
{
  my(d=0, lo=0, hi=#vec);
  forstep(i=#vec,1,-1,
     if(vec[i], d++, d--);
     if(d==0,
        if(vec[i]==1,
           hi=i-1,          \\ return to zero, but from above
           lo=i; break)));  \\ found end-most return to zero from below

  if(lo==0, return('none));  \\ nothing below zero at all

  \\ lo..hi inclusive is the last below-zero block
  concat([vec[1..lo-1],             \\ initial unchanged
          1, vec[hi+1..#vec], 0,    \\ end moved
          vec[lo+1..hi-1]]);        \\ below zero trimmed to "aa"
}
check_decrease_below(equal01_decrease_below,"Hodges 2");

\\ Repeatedly applying equal01_decrease_below() reaches a different balanced
\\ vector (none below) than Atkinson and Sack.  The first different is on
\\ equal01 vector [0,0,0,1,1,1],
\\
\\ Return vec with equal01_decrease_below() applied repeatedly to reach 0 below.
equal01_decrease_below_all(vec) =
{
  while(1,
        my(decr_vec=equal01_decrease_below(vec));
        if(decr_vec=='none, break);
        vec=decr_vec);
  is_balanced_binary_vec(vec) || error();
  vec;
}
{
  my(vec=[0,0,0,1,1,1]);
  equal01_to_balanced(vec)        == [1,0,1,1,0,0]  || error();
  equal01_decrease_below_all(vec) == [1,0,1,0,1,0]  || error();
}


\\-----------------------------------------------------------------------------
\\ Equal 0,1 Belows Decrease or Increase -- Chen

\\ A similar rearrangement as Hodges is reached by
\\
\\     Young-Ming Chen, "The Chung-Feller Theorem Revisited", Discrete
\\     Mathematics, volume 308, 2008, pages 1328-1329.
\\
\\ Chen's decrease finds the first below zero block and swaps with what
\\ precedes.  This is as opposed to Hodges finding the last block and swap
\\ with what follows.  The two are the same under a reverse and 0<->1 bit
\\ flip.


\\ vec is a vector of equal numbers of 0s and 1s with k many "below".
\\ Return Chen's bijection increasing it to k+1 below.
\\ If k is already the maximum below (#vec/2), meaning everything below
\\ zero, then return symbol 'none as the below cannot be increased.
\\
\\ vec is transformed,
\\
\\     B 1 A 0 C     -->     A 0 B 1 C           increasing flaws
\\       i   j                     
\\
\\ B is entirely below zero and i is the first up returning to zero.
\\ A is entirely above zero and j is its down returning to zero.
\\ The B1 and A0 are swapped.  The resulting 0 B 1 is one extra below.
\\
equal01_increase_below(vec) =
{
  my(d=0);
  for(i=1,#vec,
     if(!vec[i], d--,
        d++;
        if(d>0,               \\ i = first up going above zero
           for(j=i+1,#vec,
              if(vec[j], d++,
                 d--;
                 if(d<=0,     \\ j = following down step returning to zero
                    return(concat([vec[i+1..j],   \\ Ad
                                   vec[1..i],     \\ Bu
                                   if(j<#vec,     \\ C remaining
                                      vec[j+1..#vec], [])]))))))));
  'none; \\ if nothing below
}

\\ Chen's example figure 1
\\
\\     /\                               
\\    /  \          /\        
\\   /    \  -->   /  \  _  -->  /\    _     -->        _       
\\   111000            \/          \/\/           \/\/\/        
\\                 111000        100101           010101
\\                                                              
\\   0 below       1 below       2 below          3 below         
\\
equal01_increase_below([1,1,1,0,0,0]) == [1,1,0,0,0,1]   || error();
equal01_increase_below([1,1,0,0,0,1]) == [1,0,0,1,0,1]   || error();
equal01_increase_below([1,0,0,1,0,1]) == [0,1,0,1,0,1]   || error();

{
  \\ check the "below" resulting from equal01_increase_below()
  for(n=0,6,
     my(vec=equal01_first(n),
        l=List([]));
     until((vec=equal01_next(vec)) == 'none,
           my(num_below=equal01_num_below(vec),
              incr_vec=equal01_increase_below(vec));
           if(num_below==n,
              incr_vec=='none || error(),
              equal01_num_below(incr_vec) == num_below+1 || error();
              listput(l,incr_vec)));
     #l==#Set(l)  || error();   \\ all increases are distinct
  );
}

\\ vec is a vector of equal numbers of 0s and 1s with k many "below".
\\ Return Chen's bijection decreasing it to k-1 below.
\\ If k=0 then there is nothing below zero, so the below cannot be
\\ decreased, and the return is return symbol 'none.
\\
\\     A 0 B 1 C   -->   B 1 A 0 C          decreasing flaws
\\       i   j
\\ A is all above zero line (possibly empty).
\\ "0 B 1" is a block below zero line and j is where it returns to 0.
\\
equal01_decrease_below(vec) =
{
  my(d=0);
  for(i=1,#vec,
     if(vec[i], d++,
        d--;
        if(d<0,              \\ i = first down going below zero
           for(j=i+1,#vec,
              if(!vec[j], d--,
                 d++;
                 if(d>=0,    \\ j = following up step returning to zero
                    return(concat([vec[i+1..j],   \\ Bu
                                   vec[1..i],     \\ Ad
                                   if(j<#vec,     \\ C remaining
                                      vec[j+1..#vec], [])]))))))));
  'none; \\ if nothing below
}
check_decrease_below(equal01_decrease_below,"Chen");


\\ Repeatedly applying equal01_decrease_below() reaches a different balanced
\\ vector (none below) than Atkinson and Sack.  The first different is on
\\ equal01 vector [0,1,1,0],
\\
\\ Return vec with equal01_decrease_below() applied repeatedly to reach 0 below.
equal01_decrease_below_all(vec) =
{
  while(1,
        my(decr_vec=equal01_decrease_below(vec));
        if(decr_vec=='none, break);
        vec=decr_vec);
  is_balanced_binary_vec(vec) || error();
  vec;
}
{
  my(vec=[0,1,1,0]);
  equal01_decrease_below_all(vec) == [1,0,1,0]  || error();
  equal01_to_balanced(vec)        == [1,1,0,0]  || error();
}

\\ count same
\\ not in OEIS: 1,2,4,7,19,44,136,433,1434,
\\
\\ count different
\\ not in OEIS: 0,0,2,13,51,208,788,2999,11436,
\\
\\ (total being all equal01s = binomial(2*n,n) = A000984)
\\ {
\\   for(n=0,8,
\\      my(vec=equal01_first(n),
\\         count_same=0,
\\         count_diff=0);
\\      until((vec=equal01_next(vec)) == 'none,
\\            if(equal01_decrease_below_all(vec)==equal01_to_balanced(vec),
\\               count_same++,
\\               count_diff++));
\\      print1(count_same+count_diff,","));
\\   print();
\\ }


\\-----------------------------------------------------------------------------
\\ Equal 0,1 Belows Increase, Changing 2 Places

\\     Torsten Mutze, Christoph Standke, Veit Wiechert, "A Minimum-Change
\\     Version of the Chung-Feller Theorem For Dyck Paths"
\\
\\ u[c] = num up-steps starting at level c
\\ u1 = num ups from 1 line.
\\ d[c] = num down-steps starting at level c, so u0 = ups from zero line.
\\ d0 = num downs from 0 line.
\\ Flip d0+1'th down step starting or ending at 0 line (d0+d[-1] such).
\\ Flip u1'th down step starting or ending at 1 line (u1+u0 such).

\\ vec is a vector of equal numbers of 0s and 1s with k many "below".
\\ Return Mutze,Standke,Wiechert bijection increasing it to k+1 below.
\\ If k is already the maximum below (#vec/2), meaning everything below
\\ zero, then return symbol 'none as the below cannot be increased.
\\
equal01_minimum_increase_below(vec) =
{
  my(c=0,     \\ depth
     d0=0);   \\ num downs starting at d==0
  for(i=1,#vec,
     if(vec[i], c++,
        if(c==0, d0++); c--));
  \\ print("d0="d0);

  c=0;
  my(down_pos);
  for(i=1,#vec,
     if(vec[i], c++,
        if((c==0 || c==1)     \\ downs starting or ending at 0
           && d0-- == -1,     \\ d0+1'th such
           \\ print("down pos i="i);
           down_pos = i;
           vec[i] = 1; break);
        c--));
  if(d0 != -1, return('none));

  \\ now have vec = g(x)

  c=0;
  my(u1=0);   \\ num ups starting at c==1, after down change
  for(i=1,#vec,
     if(vec[i],
        if(c==1, u1++); c++,
        c--));
  \\ print("u1="u1);

  c=0;
  for(i=1,#vec,
     if(vec[i],
        if((c==1 || c==0)     \\ ups starting or ending at 1
           && u1-- == 0,      \\ u1'th such
           \\ print("up pos i="i);
           i < down_pos || error();   \\ always before the down change pos
           vec[i]=0; break);
        c++,
        c--));

  \\ now have vec = h(g(x))

  vec;
}

\\ Mutze,Standke,Wiechert example figure 2
{
  my(x=[0,1, 0,0,1,1, 1,1,1,0,0,1,1,0,0,0, 1,1,0,1,1,0,0,0, 0,1],
     f=[0,1, 0,0,1,1, 1,1,1,0,0,1,1,0,0,0, 1,1,0,0,1,0,0,1, 0,1]);
  #x == 26 || error();
  equal01_minimum_increase_below(x) == f  || error();
  \\ d0=3  u1=6
}


\\ Mutze, Standke, Wiechert example figure 4, first column.
\\
\\     /\                               
\\    /  \                   
\\   /    \  -->   /\/\  _  -->  /\    _     -->        _       
\\   111000            \/          \/\/           \/\/\/        
\\                 101001        100101           010101
\\                                                              
\\   0 below       1 below       2 below          3 below         
\\
equal01_minimum_increase_below([1,1,1,0,0,0]) == [1,0,1,0,0,1]   || error();
equal01_minimum_increase_below([1,0,1,0,0,1]) == [1,0,0,1,0,1]   || error();
equal01_minimum_increase_below([1,0,0,1,0,1]) == [0,0,0,1,1,1]   || error();

{
  \\ check the "below" resulting from equal01_minimum_increase_below()
  for(n=0,6,
     my(vec=equal01_first(n),
        l=List([]));
     until((vec=equal01_next(vec)) == 'none,
           my(num_below=equal01_num_below(vec),
              incr_vec=equal01_minimum_increase_below(vec));
           if(num_below==n,
              incr_vec=='none || error(),
              equal01_num_below(incr_vec) == num_below+1 || error();
              hammingweight(vec - incr_vec) == 2 || error();
              listput(l,incr_vec)));
     #l==#Set(l)  || error();   \\ all increases are distinct
  );
}


\\ Starting at a balanced vector and applying equal01_minimum_increase_below()
\\ n times gives an entirely below-zero.  Mutze, Standke, Wiechert show
\\ that this is an 0<->1 flip of the original.
{
  for(n=0,6,
     my(vec=equal01_first(n),
        l=List([]));
     until((vec=equal01_next(vec)) == 'none,
           is_balanced_binary_vec(vec) || next;  \\ just balanced binary

           my(incr_vec=vec);                   \\ n times belows increased
           for(i=1,n, incr_vec=equal01_minimum_increase_below(incr_vec));
           my(flip_vec = apply(b->!b, vec));
           incr_vec == flip_vec || error();    \\ is 0<->1 flip of original
          ));
}

\\-----------------------------------------------------------------------------
print("end");
