\\ Copyright 2018, 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1)
read("../vpar.gp");


\\ Usage: gp <equimatchable-count.gp
\\
\\ Count the number of equimatchable trees using the state table of
\\ vpar_is_equimatchable().
\\


\\-----------------------------------------------------------------------------
\\ Equimatchable State Table

\\ States are 1 to 5.
\\
\\ state_table[p,v] = new_state at parent when a state p parent has a state
\\ v child added underneath.  If new_state==0 then that means the
\\ combination is not equimatchable and nothing more above or below could
\\ make it so.
\\
\\ state_table[] is per vpar_INTERNAL_is_equimatchable_table in vpar.gp.
\\ That table is internal use only because it is a Vecsmall to reduce size
\\ and might change if some better idea.  Here state_table[] is a matrix for
\\ convenient use.
\\
state_table = {[
     2, 3, 4, 5, 0;   \\ p=1 row
     2, 0, 2, 0, 0;   \\ p=2
     0, 3, 0, 0, 0;   \\ p=3
     2, 0, 4, 0, 0;   \\ p=4
     0, 0, 0, 0, 0    \\ p=5
\\   -------------
\\ v=1  2  3  4  5  as new child under
]};


\\------------------------------------------------------------------------------
\\ Ordered Trees

\\ The state table gives counts of ordered equimatchable trees by showing
\\ how a given state is built from an existing parent and add a new child
\\ under.  This can be used directly for "ordered" trees where children have
\\ an order under their parent.

limit = 12;

\\ ordered_count[state,n] = number of ordered trees of equimatchable "state"
\\   1 to 5 at its root, and n vertices.
\\ n=1 singleton is state 1 and it is the only n=1 tree.
\\
ordered_count = matrix(5,limit);
ordered_count[1,1] = 1;

\\ An ordered tree of n vertices comprises a parent of p_size many vertices
\\ (the parent and all existing child subtrees), then a new child subtree
\\ being v_size = n - p_size.  Combinations of p_state and v_state which
\\ lead to new_p_state are different ways to make an n vertex new_p_state.
\\
\\ It can be noticed state 5 is simply state 4 one n later.
\\ In state_table[] the only new_state = 5 is a state p=1 with v=4 under it,
\\ and state p=1 is n=1 count 1 only.
{
  for(n=2,limit,
     for(p_size=1,n-1,
        my(v_size=n-p_size);   \\ p_size + v_size == n

        for(p_state=1,5,
           for(v_state=1,5,
              new_p_state = state_table[p_state,v_state];
              if(new_p_state,
                 ordered_count[new_p_state,n] 
                   += ordered_count[p_state,p_size]
                    * ordered_count[v_state,v_size];
   )))));
}

\\ States 1,2,3,5 are equimatchable as roots.
\\ State 4 is not equimatchable as a root.  It is still needed in the state
\\ table since it can lead to equimatchable, by state 4 parent plus state 1
\\ child, or vice versa.
\\
ordered_count_total \
  = vector(limit,n, ordered_count[1,n] + ordered_count[2,n] \
                  + ordered_count[3,n] + ordered_count[5,n]);

{
  print("Counts of rooted ordered equimatchable trees:");
  print("   state  1      2      3      4      5     total (1,2,3,5)");
  print("         ---  ----- ------ ------ ------   -------");
  for(n=1,limit,
     printf("n=%2d",n);
     for(state=1,5,
        printf(" %6d", ordered_count[state,n]));
     printf(" %9d\n", ordered_count_total[n]));
  print();
}

\\ ordered_count_total[] can be verified by an explicit search.
\\ This is only feasible for small n because the number of ordered trees
\\ (vpar_count_ordered()) is the Catalan numbers and they grow as a bit
\\ under 4^n.
\\
{
  for(n=1, min(8,limit),
     my(count=0,
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar))=='none,
           count += vpar_is_equimatchable(vpar));
     count == ordered_count_total[n] || error());
  print("(ordered_count_total[] vs search ok)");
}
print();

\\ The state combinations making ordered_count[] above can be expressed as
\\ products of generating functions.
\\
\\ Each g below is a generating function polynomial (ordinary gf) where term
\\ C*x^n is count C of how many equimatchable ordered trees of n vertices
\\ with root as the respective state 1 to 5.
\\
\\ Ser() and O(x^n) are the Pari t_SER which limits poly products etc to
\\ terms smaller than the given x^n.  Ser() normally takes a vector with
\\ vec[1] as coefficient of x^0, but here ordered_count[] starts at the x^1
\\ term, so multiply up by x.

g1 = Ser(ordered_count[1,]) * x;        \\ ogfs
g2 = Ser(ordered_count[2,]) * x;
g3 = Ser(ordered_count[3,]) * x;
g4 = Ser(ordered_count[4,]) * x;
g5 = Ser(ordered_count[5,]) * x;
gT = Ser(ordered_count_total) * x;   \\ total 

print("Ordered trees generating functions (ogf), by state:");
print("g1 = "g1 + O(x^10));  \\ truncate the printout a bit further
print("g2 = "g2 + O(x^10));
print("g3 = "g3 + O(x^10));
print("g4 = "g4 + O(x^10));
print("g5 = "g5 + O(x^10));

\\ g1 == 1*x^1 since state 1 is the n=1 singleton and nothing else.
\\
\\ g5 = g1*g4 because it is made by a state 1 parent with state 4 child
\\ below, and no other combinations.  Other g have various combinations.
\\
g1 == x                                || error();
g5 == g1*g4                            || error();
g4 == g1*g3 + g4*g3                    || error();
g3 == g1*g2 + g2*g3                    || error();
g2 == g1*g1 + g2*g1 + g2*g3 + g4*g1    || error();

\\ Difference g2-g3 cancels the g2*g3 term in each giving relations
\\
g2 == x^2 + g3 + x*g4                  || error();
g2 == x^2 + g3 + g5                    || error();

\\ Using this for g3+g5 can write total gT in terms of g2.
\\ Resulting total = 2*g2 for n>=3 in seen in the count printouts above too.
\\
gT == g1 + g2 + g3 + g5                || error();
gT == 2*g2 - x^2 + x                   || error();

\\ Some substituting with the above g1 to g5 equations isolates g2 as
\\ satisfying the following cubic.
\\
(1+x)*g2^3 - 2*g2^2 + (1-x+2*x^2)*g2 - x^2 == 0    || error();

\\ The others follow,
g3 == x*g2/(1-g2)                      || error();
g4 == x*g3/(1-g3)                      || error();
g5 == x*g4                             || error();

\\ Substituting g2 in terms of gT into the cubic gives a similar cubic for gT.
\\ It's more complicated than g2 but is the equimatchable count directly.
g2 == (gT - x + x^2)/2                 || error();
{
  (1+x)                                 * gT^3     \\ cubic for gT
  + (-4 - 3*x + 3*x^3)                  * gT^2
  + (1+x)*(4 + 3*x^2 - 6*x^3 + 3*x^4)   * gT
  + x * (-4 - 4*x - 5*x^2 + 6*x^3 - 2*x^5 + x^6)
  == 0 || error();
}

\\ seralgdep() can find a cubic from gf terms instead of working through
\\ substituting.  It solves for 'x so change to variable 'z in the gf for it.
\\
g2_in_z = subst(g2,'x,'z);
seralgdep(g2_in_z,3,2) \
  == (1+z)*x^3 - 2*x^2 + (1-z+2*z^2)*x - z^2   || error();

print();
print();


\\------------------------------------------------------------------------------
\\ Unlabelled Trees

\\ For unlabelled trees, there is no order to consider, only what types of
\\ children go under a parent to make its state.
\\
\\ Equimatchable-ness doesn't depend on child order.  The code in
\\ vpar_is_equimatchable() uses the state table with order, but the state
\\ table only actually accumulates a notion of which and how many child
\\ types are below.  From the state table it can be seen
\\
\\     state 1 = singleton only
\\     state 5 = root above one state 4
\\     state 4 = root above one or more state 3s
\\     state 3 = root above one or more state 2s
\\     state 2 = root above one or more state 1s and zero or more state 3s
\\
\\ For example state 4 is a root of 1 vertex and underneath a collection of
\\ state 3 subtrees totalling n-1 vertices (of whatever various possible
\\ sizes can make n-1).  Repeats among the state 3 trees are allowed, but
\\ their order is ignored.
\\
\\ Such counts without order and with repetitions allowed is the usual
\\ "Euler transform".  Euler_transform_term(a,n) below takes a vector a[]
\\ where a[i] is how many trees of i vertices exist (1 <= i <= n).  The
\\ return is how many ways to collect combinations of such trees to a forest
\\ n vertices total.  Taking a forest and putting a root above is a new
\\ tree.
\\
\\ Euler_transform_term() is similar to vpar.gp internal code
\\ vpar_INTERNAL_Euler_transform(), but here the return is just the
\\ transformed b[n] term, not the whole vector.  For efficiency, would want
\\ to keep the preceding b[] terms for re-use, and keep the auxilliary c[]
\\ too.

Euler_transform_term(a,n) =
{
  my(c=vector(n,k, sumdiv(k,d, d*a[d])),
     b=vector(n));
  b[1]=a[1];
  for(n=2,n, b[n] = (c[n] + sum(k=1,n-1, c[k]*b[n-k])) / n);
  b[n];
}

limit = 12;

\\ unlabelled_count[state,n] = number of unlabelled trees of equimatchable
\\   "state" 1 to 5 at its root, and n vertices.
\\ n=1 singleton is state 1 and it is the only n=1 tree.
\\ n=2 path-2 is state 2 and it is the only n=2 tree.
\\
unlabelled_count = matrix(5,limit);
unlabelled_count[1,1] = 1;
unlabelled_count[2,2] = 1;

\\ State 2 here is calculated by two combinations, either
\\     state 2 size n-1 plus a further singleton, so >=2 singletons
\\   + state 4 size n-1 plus a singleton, so =1 singleton
\\ The effect is that state 2 is 1 + cumulative state 4.
{
  for(n=3,limit,
     unlabelled_count[5,n] = unlabelled_count[4,n-1]; 
     unlabelled_count[4,n] = Euler_transform_term(unlabelled_count[3,],n-1); 
     unlabelled_count[3,n] = Euler_transform_term(unlabelled_count[2,],n-1); 
     unlabelled_count[2,n] = unlabelled_count[2,n-1]
                           + unlabelled_count[4,n-1]);
}
{
  \\ state 2 = 1 + cumulative state 4
  for(n=3,limit,
     unlabelled_count[2,n] == vecsum(unlabelled_count[4,1..n-1]) + 1
        || error());
}

unlabelled_count_total \
  = vector(limit,n, unlabelled_count[1,n] + unlabelled_count[2,n] \
                  + unlabelled_count[3,n] + unlabelled_count[5,n]);

{
  print("Counts of rooted unlabelled equimatchable trees:");
  print("   state  1      2      3      4      5     total (1,2,3,5)");
  print("         ---  ----- ------ ------ ------   -------");
  for(n=1,limit,
     printf("n=%2d",n);
     for(state=1,5,
        printf(" %6d", unlabelled_count[state,n]));
     printf(" %9d\n", unlabelled_count_total[n]));
  print();
}

\\ unlabelled_count_total[] can be verified by an explicit search.
\\ This is only feasible for small n because the number of unlabelled trees
\\ (vpar_count_unlabelled()) grows as a bit under 3^n.
{
  for(n=1,min(8,limit),
     my(count=0,
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar))=='none,
           count += vpar_is_equimatchable(vpar));
     count == unlabelled_count_total[n] || error());
  print("(unlabelled_count_total[] vs search ok)");
}

print();
print();


\\-----------------------------------------------------------------------------
\\ Labelled Trees
\\
\\ Counts for labelled rooted trees can go similar to the unlabelled, but
\\ with the usual "exponential transform" to count ways to collect labelled
\\ child subtrees.
\\
\\ exponential_transform_term() below takes vector a[1..n] where each a[i]
\\ is how many labelled rooted trees of i vertices exist.  The return is how
\\ many labelled rooted forests can be made by combinations of such trees
\\ and labellings to make total n vertices.  Repeats of trees are allowed.
\\ Choosing a vertex as root and putting a forest below is a new labelled
\\ tree.
\\
\\ exponential_transform_term() here is similar vpar.gp code
\\ vpar_INTERNAL_exponential_transform(), but returning just the b[n]
\\ forests of n, not the whole b vector.  For efficiency would keep the b[]
\\ to re-use in subsequent terms.

exponential_transform_term(a,n) =
{
  my(b=vector(n));
  for(n=1,n,
     b[n] = a[n] + sum(t=1,n-1, binomial(n-1,t-1) * a[t] * b[n-t]));
  b[n];
}

limit = 12;

\\ labelled_count[state,n] = number of labelled trees of equimatchable
\\   "state" 1 to 5 at its root, and n vertices.
\\ n=1 singleton is state 1 and it is the only n=1 tree.
\\
labelled_count = matrix(5,limit);
labelled_count[1,1] = 1;

\\ State 5 is choose root vertex n ways, then a single state 4 below.
\\ State 4 is choose root vertex n ways, then a forest of 3s totalling n-1
\\   vertices below.
\\ State 3 likewise but forest of 2s below.
\\ State 2 is choose root vertex n ways, then a forest of 1s and 3s making
\\   n-1 vertices below.  Must have at least one v=1 in that forest.  This
\\   is enforced by taking combinations of 1s or 3s then subtract
\\   combinations of entirely 3s as unwanted.
{
  for(n=2,limit,
     labelled_count[5,n] = n * labelled_count[4,n-1]; 
     labelled_count[4,n] = n * exponential_transform_term(labelled_count[3,],n-1); 
     labelled_count[3,n] = n * exponential_transform_term(labelled_count[2,],n-1); 
     labelled_count[2,n] = n * (exponential_transform_term(labelled_count[1,]
                                                         + labelled_count[3,],n-1)
                              - exponential_transform_term(labelled_count[3,],n-1));
     );
}

labelled_count_total \
  = vector(limit,n, labelled_count[1,n] + labelled_count[2,n] \
                  + labelled_count[3,n] + labelled_count[5,n]);

{
  print("Counts of rooted labelled equimatchable trees:");
  print("   state     1          2          3          4          5    total (1,2,3,5)");
  print("            ---  --------- ---------- ---------- ----------   ----------");
  for(n=1, min(10,limit),
     printf("n=%2d",n);
     for(state=1,5,
        printf(" %10d", labelled_count[state,n]));
     printf(" %12d\n", labelled_count_total[n]));
  print();
}

\\ labelled_count_total[] can be verified by an explicit search.
\\ This is only feasible for small n because the number of labelled trees
\\ (vpar_count_labelled()) grows as n^n.
{
  for(n=1, min(6,limit),
     my(count=0);
     forvec(vec=vpar_forvec_from(n),
           my(vpar=vpar_from_upqueue(vec));
           count += vpar_is_equimatchable(vpar));
     count == labelled_count_total[n] || error(n));
  print("(labelled_count_total[] vs search ok)");
}
print();

\\ Exponential transform suits counts expressed in exponential generating
\\ functions (egf).  The transform is exp(g), and x* is a shift up and
\\ choose root.

L = O(x^(limit+1));
g1 = sum(i=1,limit, labelled_count[1,i] * x^i/i!) + L;   \\ egfs
g2 = sum(i=1,limit, labelled_count[2,i] * x^i/i!) + L;
g3 = sum(i=1,limit, labelled_count[3,i] * x^i/i!) + L;
g4 = sum(i=1,limit, labelled_count[4,i] * x^i/i!) + L;
g5 = sum(i=1,limit, labelled_count[5,i] * x^i/i!) + L;

print("Labelled trees generating functions (egf), by state:");
print("g1 = "g1 + O(x^9)); \\ truncate the printout a bit further
print("g2 = "g2 + O(x^9));
print("g3 = "g3 + O(x^9));
print("g4 = "g4 + O(x^9));
print("g5 = "g5 + O(x^9));

g1 == x                                    || error();
g5 == g1*g4                                || error();
g4 == -x + x*exp(g3)                       || error();
g3 == -x + x*exp(g2)                       || error();
g2 == x*(exp(g3+g1) - exp(g3))             || error();

\\ Substituting gives each separately satisfying,
g2 == x*(1-1/exp(x)) * exp(x * exp(g2) )                    || error();
g3 == -x + x*exp((exp(x)-1)*x * exp(g3) )                   || error();
g4 == -x + x*exp(-x + x*exp((exp(x)-1) * (x+g4) ))          || error();
g5 == -x^2 + x^2*exp(-x + x*exp((exp(x)-1)/x * (x^2+g5)))   || error();

gT = sum(i=1,limit, labelled_count_total[i] * x^i/i!) + L;
gT == g1 + g2 + g3 + g5                    || error();

\\ and gT in terms of g2
gT == - x^2 + x*exp(g2) + ( 1 + (x-1)/exp(x) )*x*exp(x*exp(g2))    || error();

\\------------------------------------------------------------------------------
