\\ Copyright 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);
read("../vpar.gp");


\\ Usage: gp <footprints-preorder.gp
\\
\\ Knuth volume 4A section 7.2.1.6 exercise 30 first edition page 475, or
\\ pre-fascicle 4A "draft of section 7.2.1.6" page 35
\\
\\     http://www-cs-faculty.stanford.edu/~knuth/fasc4a.ps.gz
\\
\\ considers pre-order rooted forests with given childfulness "footprint" of
\\ the vertices.  For vertices 1..n in pre-order, the footprint is a vector
\\
\\     footprint f[vertex] = / 1 if vertex is childful (has children)
\\                           \ 0 if vertex is childless
\\                         = vpar_is_childful()
\\                           or whole vector vpar_is_childful_vector()
\\
\\ The answer in first edition page 788, or pre-fascicle page 53, is
\\ replaced by errata for page 788 dated 23 Apr 2019,
\\
\\     https://www-cs-faculty.stanford.edu/~knuth/all4a.ps.gz
\\
\\ The code and long comments below are a couple of approaches to counting
\\ preorder forests of a given footprint as for exercise 30(b).  The counts
\\ are compared against explicitly iterating vpar_preorder_next() etc.
\\
\\ The build-up is slightly lengthy, but eventually reaches
\\ count_footprint_using_upwards_of_pqvecs().
\\
\\
\\ A notion equivalent to the footprint appears in
\\
\\     Jean-Louis Loday and Maria O. Ronco, "Hopf Algebra of the Planar Binary
\\     Trees", Advances in Mathematics, volume 139, 1998, article number
\\     AI981759, pages 293-309.
\\
\\ The assign +1/-1 to each "external" vertex of a binary tree.  Xavier
\\ Viennot calls this the "canopy" of a binary tree ("Up-Down Sequences of
\\ Permutations, Paths and Canopy of Binary Trees").  Before then, in
\\
\\     Jean-Louis Loday and Maria O. Ronco, "Order Structure On the Algebra
\\     of Permutations and of Planar Binary Trees", J. of Algebraic
\\     Combinatorics, 15, 253-270, 2002
\\
\\ they show too that the binary trees of n vertices with a given canopy
\\ form an interval within the Tamari lattice.  This is equivalent to the
\\ preorder forests of given footprint forming an interval.


\\-----------------------------------------------------------------------------
\\ The footprint of vertices 1..n consists of alternating runs of 1s and 0s,
\\
\\     1,1,1,1,1,  0,0,0,0, ... 1,1,1, 0,0,0
\\        >=0        >=1          >=1   >=1
\\        p[1]       q[1]         p[k]  q[k]    p,q counts 1s and 0s
\\
\\     first run length >=0, rest >=1
\\
\\ Vertex number 1 can be childless, so the first run of 1s is length >=0.
\\ The rest are length >=1.  The last vertex must be childless because there
\\ is nothing after to give it a child.  So all runs of 0s, including the
\\ last, are >=1.
\\
\\ Knuth collects these run lengths as p[i] many 1s and q[i] many 0s, for
\\ i=1 to k.  Or rather Knuth takes those p,q as count - 1, except for p[1].
\\ Such -1 has the effect of making all >=0.  But here they are left full
\\ run lengths, and consequently here total
\\
\\     p[1] + q[1] + ... + p[k] + q[k]  = n     vertices
\\
\\ Pre-order labelling means that a run of p many 1s is a chain down.  If
\\ vertex v is to be childful then next vertex v+1 must go under it to make
\\ that so.  Putting v+1 anywhere higher would prevent anything later going
\\ under v, because labelling is preorder.
\\
\\   top    0  v                     top     0  r
\\             |                                |
\\          1 v+1                            1  *
\\             |\    \                          |
\\          2  * Q ...W  t=3 up              2  *
\\             |\     |                         |
\\          3  * Q    *                      3  *
\\             |\     |                         |
\\          4  *      *                      4  *
\\  p[i]=5     |\\    |              p[i]=5     |\     \
\\             N QQ   *                         N QQQQ  W    t=0 up
\\                    |                                 |
\\         q[i]=4     *                                 *
\\
\\ There are q many childless vertices.  The first is shown N and it must be
\\ under the last p to make that p childful.  The remaining q-1 are shown as
\\ Q's.  They can be assigned under any of the p vertices, at depth levels
\\ no higher than the top vertex of the next run (p[i+1]).
\\
\\ The diagram above at the right is when the next chain p[i+1] is maximum
\\ depth.  W is the top vertex of p[i+1] and is under the lowest of p[i].
\\ It's not under N of course, since N is to be childless.  All Qs must go
\\ under the lowest of p[i] too, only 1 choice.
\\
\\ The diagram at the left is instead next chain W at t=3 many depth levels
\\ upwards from the deepest.  This opens 3 additional places for the Q
\\ vertices to go.
\\
\\ The Qs are thus q-1 vertices going into t+1 many places.  In the usual
\\ way, s things into m places is a binomial choosing m-1 separators from
\\ among these separators plus s things, so binomial(m-1+s,s).  Or
\\ equivalently binomial(m-1+s,m-1) which is choose the things.  So here
\\
\\     binomial(t+1-1+q-1, q-1)
\\       = binomial(t+q-1, q-1)           arrangements of Qs
\\
\\ Maximum t upwards is when the next top W is a new root, depth=0.  How far
\\ up that can be is found by reckoning successive chains going down p and
\\ up t.  Total t must be <= total p at each point.
\\
\\     sum(p[1..i]) >= sum(t[1..i])       all intermediates i
\\
\\ There are some notions of partial ordering of sequences (where one
\\ "dominates" the other) based on cumulatives >= like this, but here just
\\ want to limit t.
\\
\\ At the last p,q, there will be no next chain down so the Qs can go at any
\\ depth level.  This can be reckoned as final t[k] always its maximum, so
\\ going all the way up to depth=0 and so Qs anywhere on the right-most side
\\ of the forest.  So final total t same as total p,
\\
\\     sum(p[1..k]) = sum(t[1..k])        final total
\\
\\ Graphically, this can be thought of as the p[] being stair steps going
\\ each up p[i] and across 1 for total sum(p) up and k-1 across (or k across
\\ if wanted to reckon a fixed initial across 1, or a final across 1).  The
\\ t[] steps are to be a path staying always at or below the p path.
\\
\\                          **
\\                     p[5] ||            p path with
\\                      *---*| t[5]       t path always at or below.
\\                      |    |
\\                 p[4] |*---*            Can have equality,
\\                      || t[4]           as at sum(p[1..3])==sum(t[1..3])
\\                  **==**                here.
\\                  ||
\\             p[3] ||                    Can have individual t[i] > p[i],
\\                  ||                    as at t[3] > p[3] here,
\\              *---*|                    just cumulative must be <=.
\\              |    | t[3]
\\         p[2] |    |
\\              |*---*
\\          *---*|
\\          |    | t[2]
\\     p[1] |*---*
\\          || t[1]
\\          **
\\
\\ For a given sequence of t[] steps up, the total arrangements of the Qs is
\\ product of the binomials choosing the places for each,
\\
\\     count =  prod  binomial(t[i]+q[i]-1, q[i]-1)
\\             i=1..k
\\
\\ A simple, but inefficient, approach for counting forests of given
\\ footprint is to generate all t sequences permitted by the footprint's
\\ p[], and sum the products according to the q[].


\\ pvec is vector of p[i].
\\ tvec is vector of t[i].
\\ Return 1 if tvec is good, meaning all partial sum(t[1..i]) <= sum(p[1..i]),
\\ and final sum(all t) == sum(all p).
tvec_is_good(pvec,tvec) =
{
  my(net=0);
  for(i=1,#tvec,
     net += pvec[i];
     net -= tvec[i];
     if(net<0, return(0)));  \\ each partial sum net pssum-tsum >= 0
  net==0;                    \\ final total equal so net psum-tsum==0
}

\\ Return the lex() smallest tvec for given pvec[].
\\ This is zeros except last term which is p total.
\\    [ 0, 0, ..., 0, 0, sum(pvec) ]
tvec_first(pvec) =
{
  my(tvec=vector(#pvec));
  if(#tvec, tvec[#tvec] = vecsum(pvec));
  tvec;
}

\\ Return lex() next bigger tvec than the given tvec, under footprint pvec.
tvec_next(pvec,tvec) =
{
  \\ try to increment second-end-most position i = k-1 for bigger lex()
  forstep(i=#tvec-1,1,-1,
     tvec[i]++;
     tvec[#tvec]--;    \\ maintain total t's == total p's
     if(tvec_is_good(pvec,tvec), return(tvec));

     \\ this position is too big, drop it to zero and try at i-1
     tvec[#tvec] += tvec[i];    \\ maintain total t's == total p's
     tvec[i] = 0);

  return('none);   \\ no further lex() increase possible
}

\\ Number of ways to arrange childless Qs under this tvec.
tvec_product(tvec,qvec) = \
  prod(i=1,#tvec, my(t=tvec[i], q=qvec[i]); binomial(t+q-1, q-1));

\\ footprint is a vector of 1,0 flags.
\\ Return [pvec,qvec] vector of two vectors pvec and qvec.
footprint_to_pqvecs(footprint) =
{
  my(plist=List([]),
     qlist=List([]),
     i=1);
  while(i<=#footprint,
        my(p=0); while(i<=#footprint && footprint[i]==1, p++;i++);
        my(q=0); while(i<=#footprint && footprint[i]==0, q++;i++);
        listput(plist,p);
        listput(qlist,q));
  [Vec(plist), Vec(qlist)];
}

\\ Return count of preorder forests of given pvec,qvec footprint.
count_footprint_using_tvecs_of_pqvecs(pvec,qvec) =
{
  my(tvec=tvec_first(pvec),
     count=0);
  until((tvec=tvec_next(pvec,tvec))=='none,
        count += tvec_product(tvec,qvec));
  count;
}

\\ Return count of preorder forests of given footprint vector.
count_footprint_using_tvecs(footprint) =
{
  my(pvec,qvec); [pvec,qvec]=footprint_to_pqvecs(footprint);
  count_footprint_using_tvecs_of_pqvecs(pvec,qvec);
}

{
  print("Preorder forests count by \"footprint\"");
  for(n=3,5,
     print("n="n);
     my(total=0);
     forvec(footprint=vector(n,i, [0,i!=n]),
           my(count=count_footprint_using_tvecs(footprint));
           print("  "footprint"  "count);
           total += count);
     print("  total "total" == vpar_count_ordered(n,1)");
     total == vpar_count_ordered(n,1) || error());
  print();
}

\\ vpar is a preorder labelled forest.  Return its childfulness footprint.
\\ This is vpar_is_childful_vector(), but given a name "footprint" here
\\ for clarity.
\\
footprint_of_vpar(vpar) = vpar_is_childful_vector(vpar);

{
  \\ Check count_footprint_using_tvecs() by iterating through all preorder
  \\ vpar forests and counting how many of each footprint.
  \\
  \\ The number of preorder forests is vpar_count_ordered(n,1).  This grows
  \\ roughly as 4^n so iterating here is feasible only for small n.
  \\
  \\ The last entry in a footprint is always 0, and otherwise all 2^(n-1)
  \\ possible combinations occur.  That's simply because always at least one
  \\ arrangement of the Qs is always possible.

  print1("checking ... ");
  for(n=0,8,
     my(footprints=List([]));     \\ all possible footprints of n vertices
     forvec(footprint=vector(n,i, [0,i!=n]),
           listput(footprints,footprint));
     footprints=Set(footprints);  \\ ready for setsearch()
     #footprints == if(n==0,1,2^(n-1)) || error();

     my(counts=vector(#footprints));  \\ of each footprint
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
           my(footprint=footprint_of_vpar(vpar),
              pos=setsearch(footprints,footprint));
           pos || error("oops, footprint not found");
           counts[pos]++);

     for(i=1,#footprints,
        counts[i]!=0 || error();   \\ all footprints occur at least once
        count_footprint_using_tvecs(footprints[i]) == counts[i] || error()));
  print("good");
}


\\-----------------------------------------------------------------------------
\\ Recursion
\\
\\ A given p[i] run has its top at some depth d.  The first run p[1] is
\\ always at d=0 (the depth of the roots).
\\
\\ A recursion can be written based on having the top of run p[i] at a depth
\\ d, then recurse to i+1 at a new depth by p[i] and up by t, for each
\\ permitted t at that i.  The permitted t range is no higher than next top
\\ depth 0, and no lower than d+p[i] (child of lowest vertex of p[i]).
\\
\\ The effect of such recursion is to go through sequences of t[] the same
\\ as tvec iterated above, but the t values are local variables in each
\\ recursive call, instead of forming a whole tvec together.
\\
\\ This is only slightly more efficient than the iteration.  There's less
\\ repeating of binomial()s, but the recursion still enumerates all t[]
\\ combinations.  Some memoize() of the recursion helps (eg. my memoize.gp),
\\ but that can be done more directly in "Depths by Vector" below.

count_footprint_using_recursion_of_pqvecs(pvec,qvec, i,d) =
{
  if(i==#pvec+1, return(if(d==0,1,0)));
  my(p=pvec[i], q=qvec[i]);
  sum(t=0,d+p,
      binomial(t+q-1, q-1)
      * count_footprint_using_recursion_of_pqvecs(pvec,qvec, i+1,d+p-t));
}

count_footprint_using_recursion(footprint) =
{
  my(pvec,qvec); [pvec,qvec]=footprint_to_pqvecs(footprint);
  count_footprint_using_recursion_of_pqvecs(pvec,qvec, 1,0);
}

{
  print1("checking ... ");
  for(n=0,10,
     forvec(footprint=vector(n,i, [0,i!=n]),
           count_footprint_using_recursion(footprint)
             == count_footprint_using_tvecs(footprint) || error()));
  print("good");
}

\\ In the code above, final case if(i==#pvec+1, return(if(d==0,1,0))) means
\\ going back to depth d=0 after all pvec is factor 1, and anything else is
\\ factor 0 and so contributes nothing to the return.  This has the effect
\\ of zeroing out anything except t[k] = d+p[k] in the i=k sum.
\\
\\ The final calls with i=k+1, d!=0 are unnecessary and wasteful.  Can
\\ instead stop the recursion at i==k and return just single desired
\\ binomial on t=d+p.
\\
count_footprint_using_recursion_of_pqvecs(pvec,qvec,i,d) =
{
  my(p=pvec[i], q=qvec[i]);    \\ for #pvec >= 1
  if(i==#pvec, return(binomial(d+p+q-1, q-1)));
  sum(t=0,d+p,
      binomial(t+q-1, q-1)
      * count_footprint_using_recursion_of_pqvecs(pvec,qvec, i+1,d+p-t));
}

{
  print1("checking ... ");
  for(n=1,10,
     forvec(footprint=vector(n,i, [0,i!=n]),
           count_footprint_using_recursion(footprint)
           == count_footprint_using_tvecs(footprint) || error()));
  print("good");
}
print();


\\-----------------------------------------------------------------------------
\\ Binary Integer for Footprint
\\
\\ The above loops iterate footprints of given length using forvec().
\\ Another way is to take an integer "fbin" in binary and interpret it,
\\
\\                      +---+-------+
\\             fbin =   | 1 | xxxxx |
\\                      +---+-------+--+
\\        footprint =       | xxxxxx 0 |           for n>=1 vertices
\\                          +----------+
\\                      drop         add
\\                     high 1       low 0
\\
\\ The high 1 of fbin determines the length, then appending a low 0
\\ satisfies the demand that a footprint ends in 0.  Footprints of n>=1
\\ vertices are all fbin in the range
\\
\\     2^(n-1) <= fbin < 2^n            for n>=1
\\
\\ The effect is to take footprints by length, and among equal length by
\\ lex().
\\
\\ The empty footprint, for 0 vertices, can be taken as fbin=0.  This
\\ doesn't really follow the general case, but offers completeness.

\\ fbin is an integer >=1.  Return a footprint vector of 0s and 1s.
fbin_to_footprint(fbin) =
{
  fbin >= 0 || error();
  if(fbin==0, [],
     concat(binary(fbin)[^1], [0]));
}
fbin_to_footprint(0) == []     || error();
fbin_to_footprint(1) == [0]    || error();
fbin_to_footprint(2) == [0,0]  || error();
fbin_to_footprint(3) == [1,0]  || error();

{
  my(lo=8,hi=31);
  print("Count forests for fbin="lo" to "hi);
  print1("  ");
  for(fbin=lo,hi,
     my(footprint=fbin_to_footprint(fbin));
     print1(count_footprint_using_tvecs(footprint),","));
  print();
  \\ not in OEIS: 1,1,2,1,3,2,3,1,1,1,2,1,3,2,3,1,4,3,5,2,6,3,4,1
}

\\ The number of different tvec which occur in a given footprint can be
\\ counted by explicit loop.  (In the full count preorder forests, the same
\\ result would be had by forcing all q lengths to 1.)
\\
num_tvecs_by_iteration(footprint) =
{
  my(pvec,qvec); [pvec,qvec]=footprint_to_pqvecs(footprint);
  my(tvec=tvec_first(pvec),
     count=0);
  until((tvec=tvec_next(pvec,tvec))=='none, count++);
  count;
}
{
  my(lo=8,hi=31);
  print("num tvecs for fbin="lo" to "hi);
  print1("  ");
  for(fbin=lo,hi,
     my(footprint=fbin_to_footprint(fbin));
     print1(num_tvecs_by_iteration(footprint),","));
  print("\n");
  \\ lo=28 to hi=50 avoids false matches of various 1s and 2s sequences
  \\ not in OEIS: 1,3,1,1,1,1,1,1,1,2,1,1,1,2,2,2,1,3,1,1,1,2,2,2,2,5,2,2
}


\\-----------------------------------------------------------------------------
\\ Depths by Vector
\\
\\ A more efficient count forests by footprint can be made by collecting
\\ binomials where the recursion above repeats an i,d combination.
\\ The idea is to form a sum
\\
\\     counts[d] = sum all binomial products, of positions i and later,
\\                 where the top of p[i] is at depth d
\\
\\ This depth d can also be thought of as extra "headroom" available to its
\\ subsequent chains p[i+1] etc.  p[i+1] can be placed as child of any of
\\ the p[i], and in addition at any of d many places above p[i].
\\
\\ Like the recursion etc above, depth d occurs when earlier t's are smaller
\\ than their maxima, leaving later t's the opportunity to go higher.  So
\\ depth is shortfall of t's over p's so far,
\\
\\     d = sum(p[1..i-1]) - sum(t[1..i-1])      depth
\\
\\ At last position i=k, the greatest depth is d = sum(p[1..k-1]).  This is
\\ all t[1..k-1] = 0.  So i=k is counts[d] for d=0 up to that maximum,
\\
\\     t = d + p[k]                        places at final i=k
\\     counts[d] = binomial(t+q-1,q-1)     arrangements of Qs
\\
\\ The main loop in the code takes counts[] at i+1 and uses them to
\\ construct counts[] at i.  A given d in i gains p steps down then t steps
\\ up.  This means depth d+p-t at i+1, and so that counts[d+p-t], multiplied
\\ by binomial(t+q-1,q-1) arrangement of Qs.  t can range from 0 up to
\\ maximum d+p (same as recursion etc above).  That maximum is depth 0 in
\\ i+1 (because t has taken all of d+p in i).
\\
\\ The maximum depth at position i is sum(p[1..i-1]).  At i=1 this is an
\\ empty sum, so 0, giving at i=1 a sole depth d=0 and this is the desired
\\ total count of forests.  In terms of "headroom", it is no extra headroom
\\ above.
\\
\\ GP vectors start at index 1, but counts[] wants d=0 too.  In the code,
\\ counts[1] is for d=0.  This index offset only shows up in mangling the
\\ "d" index of vector() constructing new counts, and in counts[1+ ...]
\\ fetching from the existing (ie. counts in i+1).  Similarly bins[].
\\
\\ The bins[] vector of binomials is calculated outside the new d vector
\\ loop, since the various binomials by t are the same in each set of d
\\ products, just different ones chosen for the cross products.  bins[]
\\ depends only on q, so if some q occurred more than once in qvec[] then
\\ could keep and re-use its bins[].  A smaller i has smaller ptotal so only
\\ that much of bins[] would be needed (the rest truncated to release
\\ memory).


count_footprint_using_upwards_of_pqvecs(pvec,qvec) =
{
  if (#pvec==0,return(1));  \\ empty footprint is the empty forest only

  \\ counts[d+1] is the count for d, ranging d=0 upwards
  \\ bins[t+1] is the binomial for t, ranging t=0 upwards
  \\ clen = ptotal+1 is the number of counts

  my(clen = vecsum(pvec[^#pvec]) + 1,  \\ excluding last p
     p=pvec[#pvec],  \\ last p,q
     q=qvec[#qvec],
     counts=vector(clen,d,d--;   \\ d=p to vecsum(pvec) inclusive
                   binomial(d+p + q-1, q-1)));

  forstep(i=#pvec-1,1,-1,
     my(p=pvec[i], q=qvec[i],
        bins=vector(clen,t,t--; binomial(t + q-1, q-1)));
     clen -= p;
     counts=vector(clen,d,d--;
                   sum(t=0,d+p,
                       bins[1+ t] * counts[1+ d+p-t])));   \\ reverse crosses
  #counts==1 || error();
  counts[1];
}
count_footprint_using_upwards(footprint) =
{
  my(pvec,qvec); [pvec,qvec]=footprint_to_pqvecs(footprint);
  count_footprint_using_upwards_of_pqvecs(pvec,qvec);
}

{
  print1("checking ... ");
  for(fbin=0,2^12,
     my(footprint=fbin_to_footprint(fbin));
     count_footprint_using_upwards(footprint)
     == count_footprint_using_tvecs(footprint)
       || error());
  print("good");
}

\\ If working in terms of the integer fbin, successive p,q run lengths can
\\ be peeled off the low end of fbin instead of making pvec,qvec.
\\ fbin is without the final 0 for last vertex, hence initial fbin<<=1.
\\ hammingweight(fbin) is ptotal plus the high 1-bit, which is the desired
\\ clen.
\\
\\ When run p[1] is reached, it is for example fbin = 1 111 which is p=3 1s
\\ and a high 1-bit above.  The 1-bit count is 4 which is 1 too big.
\\ Stripping this run leaves fbin==0, and p is decremented in that case.
\\
\\ The valuation() and shifting down here could also be done by some
\\ bit-scanning where look for the next 1-bit or 0-bit above a given
\\ position.  That would be efficient if fbin is a bignum since it's
\\ read-only.  Some for() loop with bittest() would suit, though it's
\\ probably the sort of thing you'd like a builtin or at least a library
\\ function to do.
\\
count_footprint_using_upwards_of_fbin(fbin) =
{
  fbin>=0 || error("count_footprint_using_upwards_of_fbin() is for fbin>=0");
  if (fbin==0,return(1));  \\ empty footprint is the empty forest only

  my(clen=hammingweight(fbin)); fbin<<=1;
  my(q=valuation(fbin,2)); fbin>>=q;                     /* num low 0s */
  my(p=valuation(fbin+1,2)); fbin>>=p; if(fbin==0,p--);  /* num low 1s */
  clen -= p;
  my(counts=vector(clen,d,d--; binomial(d+p + q-1, q-1)));
  while(fbin,
        q=valuation(fbin,2); fbin>>=q;
        p=valuation(fbin+1,2); fbin>>=p; if(fbin==0,p--);
        my(bins=vector(clen,t,t--; binomial(t + q-1, q-1)));
        clen -= p;
        counts=vector(clen,d,d--; sum(t=0,d+p, bins[1+t] * counts[1+d+p-t])));
  #counts==1 || error();
  counts[1];
}

{
  print1("checking ... ");
  for(fbin=0,2^14,
     my(footprint=fbin_to_footprint(fbin));
     count_footprint_using_upwards_of_fbin(fbin)
     == count_footprint_using_upwards(footprint)
       || error());
  print("good");
}


\\-----------------------------------------------------------------------------
\\ Depths by Polynomials
\\
\\ In the code above, it can be noticed the "reverse crosses" is sum over
\\ products of rising/falling indices, of total index d+p.  That's the sort
\\ of thing done by a generating function product (an ordinary generating
\\ function).
\\
\\ The code below adapts the counts[] vector to be a polynomial cpoly where
\\ polcoeff(cpoly,d) == counts[d].  d=0 is the poly constant term.
\\
\\ Initial cpoly=1 is per the first implemenation of
\\ count_footprint_using_recursion_of_pqvecs(), being a 1 for imagined k+1
\\ position going back to depth d=0, opening up the whole right of the
\\ forest, and for any other d the count is 0.  The multiply 0s in the
\\ recursion are no as bad here, being polynomial multiply by 1.  However
\\ binomial_places_gf() is calculated up to ptotal=sum(p[1..k]) and then
\\ immediately its low p[k] many terms are chopped off, which is wasteful.
\\ In fact, each serchop() discards low product terms, which is wasteful.
\\
\\ Do these generating function products hint at some product for a
\\ closed-form answer?  Suspect the chopping-off low terms means not-quite;
\\ and think chopping is necessary to avoid low terms contributing to later
\\ products (at least when working/thinking last to first).


\\ g = test_binomial_places_gf(s,limit)
\\ Return a generating function t_SER where c = polcoeff(g,i) is the number
\\ of ways to put s many things into a list of i+1 many buckets.
\\ The things are undistinguished, the buckets are ordered.
\\ The binomial is per the "s things into m places" described above.
\\ g has "limit" many terms, and first term x^0.
\\
binomial_places_gf(s,limit) = \
  Ser(vector(limit,i,i--; binomial(i+s, s)),,limit);

\\ serchop() is new in GP 2.11, here's enough of a fallback for prior versions
iferr(serchop(1+O(x^2),1),e, \
      eval("serchop=(s,n)->for(i=0,n-1,s-=polcoeff(s,i)*x^i);s"));

count_footprint_using_gfs_of_pqvecs(pvec,qvec) =
{
  my(clen=vecsum(pvec)+1,
     cpoly=1);
  forstep(i=#pvec,1,-1,
     my(p=pvec[i], q=qvec[i]);
     cpoly *= binomial_places_gf(q-1,clen);
     cpoly = serchop(cpoly,p)/x^p;           \\ discard low p many terms
     clen -= p);
  polcoeff(cpoly,0);
}
count_footprint_using_gfs(footprint) =
{
  my(pvec,qvec); [pvec,qvec]=footprint_to_pqvecs(footprint);
  count_footprint_using_gfs_of_pqvecs(pvec,qvec);
}

{
  print1("checking ... ");
  for(fbin=0,2^12,
     my(footprint=fbin_to_footprint(fbin));
     count_footprint_using_gfs(footprint)
       == count_footprint_using_upwards(footprint) || error());
  print("good");
}
print();


\\-----------------------------------------------------------------------------
\\ Exercise 30(b) example footprint is answer 114044694.

{
  my(footprint
   =[1,0, 1,0, 1,1,0, 1,1,1,1,1,1,0,0,0,0, 1,0, 1,0, 1,0,0,0, 1,0, 1,1,0,0,0]);
  print("Knuth 30(b) sample");
  print("  footprint "footprint);
  my(pvec,qvec); [pvec,qvec]=footprint_to_pqvecs(footprint);
  print("  pvec      "pvec);
  print("  qvec      "qvec);
  my(got=count_footprint_using_upwards(footprint));
  print("  answer    ",got);
  got==114044694 || error();
}
print();


\\-----------------------------------------------------------------------------
\\ Tamari Lattice Complementary Pairs
\\
\\ Knuth exercise 30(d) is on complementary pairs of forests in the Tamari
\\ lattice.  In a lattice, two vertices are "complementary" when
\\
\\     min(x,y) = whole lattice minimum             complementary pair
\\     max(x,y) = whole lattice maximum          meet,join at whole min,max
\\
\\ In the Tamari lattice, each vertex is an ordered forest and cover
\\ relations are by a binary tree rotation (usual natural correspondence
\\ between ordered forest and binary tree).
\\
\\ Per Lakser (apparently), two forests x,y are complementary in the Tamari
\\ lattice if and only if their footprints fx,fy are complementary, meaning
\\
\\     fx[v] = ! fy[v]       opposite 0,1 for v = 1 to n-1,
\\     fx[n] = fy[n] = 0     except both last entry always 0
\\
\\ Per Knuth's answer, this occurs essentially because a rotate moves a
\\ subtree to under another vertex.  Such a move makes that vertex childful.
\\ Combinations of moves can thus reach a union of x,y childfulness.
\\ If footprints x,y start as 0,1 complements then that union goes to
\\ everything childful, which is the lattice maximum (a path).
\\ Conversely downwards unrotate towards union of childlessness reaches
\\ lattice minimum everything childless (forest of all singletons).
\\
\\ So the total number of Tamari complementary pairs is a sum over
\\ footprints,
\\
\\        sum     Num(footprint) * Num(complement(footprint))
\\     footprints
\\     starting 0
\\
\\ In the forvec() code below, pairs of footprints are counted just one way
\\ around, so only x,y not also y,x, by forcing the footprint to start 0, so
\\ its complement starts 1.
\\
\\ In general, the number of forests with given footprint is not the same as
\\ the number with the complement.
\\
\\ n=1 is a single forest (singleton vertex).  It can be reckoned
\\ complementary to itself since min(x,x)=minimum and max(x,x)=maximum.
\\ In anything bigger, a vertex is not complementary to itself.
\\
\\ n=2 is two forests (2-singletons, or path-2).  They are a complementary
\\ pair since again min(x,y)=x minimum and max(x,y)=y maximum.


\\ footprint is a vector of 1,0 flags.
\\ Return its 0 <-> 1 complement vector.
footprint_complement(footprint) = \
  for(i=1,#footprint-1, footprint[i] = !footprint[i]); footprint;

\\ A046996 = number of Baxter permutations / 2
A046996(n) =
{
  n>=2 || error();
  sum(k=1,n, binomial(n+1,k-1)*binomial(n+1,k)*binomial(n+1,k+1))
  / (2*binomial(n+1,1)*binomial(n+1,2));
}
A046996_samples = [1,3,11,46,211,1037,5377,29101,163120,941480,5570280];
vector(#A046996_samples,n,n++; A046996(n)) /* its OFFSET=2 */ \
  == A046996_samples  || error();

\\ A001181 = number of Baxter permutations
A001181(n) = if(n<=1,n, 2*A046996(n));
A001181_samples = [0,1,2,6,22,92,422,2074,10754,58202,326240,1882960,11140560];
vector(#A001181_samples,n,n--; A001181(n)) /* its OFFSET=0 */ \
  == A001181_samples || error();

{
  print("Tamari lattice number of complementary pairs");
  print1("  ");
  for(n=1,10,
     my(total=0);
     forvec(footprint=vector(n,i, if(i==1||i==n,[0,0],[0,1])),
           my(comp=footprint_complement(footprint));
           total += count_footprint_using_upwards(footprint)
                  * count_footprint_using_upwards(comp));
     print1(total,",");

     if(n>=2,  total == A046996(n) || error();
             2*total == A001181(n) || error());
  );
  print("  counted one way = A046996 Baxter permutations/2");
  print("  counted both ways = A001181 Baxter permutations");
  print();
}


\\-----------------------------------------------------------------------------
\\ Knuth's answer, before and after errata, is based on converting a
\\ preorder forest to a binary tree by a correspondence
\\
\\     preorder forest      binary tree            in-order traverse
\\     ---------------      -----------               binary tree
\\     previous sibling  =  left child                    =
\\     last child        =  right child         preorder traverse forest
\\
\\ This is not the "natural" correspondence of left = forest first child;
\\ right = forest next sibling; but instead a kind of opposite diagonal.
\\ The effect is binary tree as last root of the forest, and other forest
\\ roots going down the left-most side of the tree.
\\
\\ Vertices of the binary tree can have the following combinations of
\\ left,right children present or absent
\\
\\     N = no children
\\     L = have left child, no right child
\\     R = have right child, no left child
\\     B = have both
\\
\\ In-order traversal of the binary tree is pre-order traversal of the forest.
\\ The binary tree has a "specification" of vertex types
\\
\\     R^p1  N  L^q1   B    R^p2  N  L^q2   B  ...  R^pn  N  L^qn
\\
\\       where ^ means a repeat count
\\
\\ For example, footprint 1,0,1,0 is two forests
\\
\\     1  3                1
\\     |  |      and       |\      preorder forests 1010
\\     2  4                2 3
\\                           |
\\                           4
\\
\\ It is specification = R,N,B,N and the two corresponding binary trees are
\\
\\         3 B             1 R
\\        / \      and      \           binary trees RNBN
\\     R 1   4 N             3 B        in-order vertex numbering
\\        \                 / \
\\         2 N           N 2   4 N
\\
\\ In general, a forest footprint is chains like
\\
\\   top    0  R
\\             |             \
\\          1  R              B
\\             |\    \        |
\\          2  R L    B       R
\\             |\     | \     |
\\          3  R L L  R  L    N
\\             |\     |
\\          4  N L    R
\\                    |
\\                    N
\\
\\ R^p1 is the left-most chain down.  Each such has no prev sibling which
\\ would be left child.  They are footprint=1.
\\
\\ The bottom-most of the chain is an N which has no first child either.
\\ It and L^q1 are the footprint=0 childless vertices.  The L each have a
\\ previous sibling.
\\
\\ B is the top of the next chain.  It has a child, and also has a previous
\\ sibling.  It and possible further R are the second chain down.  And so on
\\ each chain.
\\
\\ Runs of 1s in the footprint are thus all of first p1, and then p2 plus
\\ preceding B so p2+1, p3+1, etc.  Runs of 0s in the footprint are each q+1
\\ being L and preceding N.  So Knuth's counts p1,q1, p2,q2, are footprint
\\ run lengths each -1 except for p1.
\\
\\ Knuth's answer in errata dated 23 Apr 2019 is firstly k=1 only chain is
\\ binomial(p1+q1,q1) where each L can be placed.  Then for k>=2, sum
\\ three cases
\\
\\     f(p1,q1,...) =   f(p1-1,q1, ...)                 drop first R
\\                    + f(p1,q1,...) * f(pj,qj,...)     split
\\                    + f(..., pn,qn-1)                 drop last L
\\
\\ In the binary tree, these cases are the vertex type of the root, and then
\\ ways to form left and right child sub-trees.
\\
\\ "Drop first R" is binary tree root R.  The specification is in-order so
\\ this is the first R, and applies when the specification starts with an R
\\ so p1>=1.  This R can be dropped and count the ways for the rest as its
\\ right sub-tree.  In terms of a forest, this is a forest of just one root
\\ so count ways to arrange the rest as a forest underneath it.
\\
\\ "Split" is binary tree root B.  j is which B in the specification is the
\\ root and the product is ways in the left and right sub-trees.  In the
\\ forest, this is >=2 roots and the last root having childfulness
\\ footprint=1 and being top of chain j.  A root is depth=0 so it does not
\\ restrict arrangements of the chains preceding, so split for product of
\\ ways before and ways in the sub-forest below it.
\\
\\ "Drop last L" is binary tree root L.  The specification is in-order so
\\ this is the last L, and applies when the specification ends with an L so
\\ qn>=1.  This L can be dropped and count ways for the rest as its left
\\ sub-tree.  In terms of a forest, this is >=2 roots and the last root
\\ childless footprint=0.  Being depth=0, it does not restrict anything
\\ preceding, so count ways to arrange the rest.
\\
\\ The code below is Knuth's f().  Within in it each p[i],q[i] is in the way
\\ described by Knuth so each other than p[1] has -1 so is the R,L counts.
\\
\\ The recursions by f() variously repeat smaller p,q vectors.  Would want
\\ to cache or otherwise arrange terms for speed.  My memoize.gp can be
\\ applied by f=memoize(f) to do that easily, but memoizing on arguments
\\ pvec,qvec is not particularly memory efficient.

f(pvec,qvec) =
{
  my(k=#pvec);
  if(k==0, return(1));
  if(k==1, return(binomial(pvec[1]+qvec[1], pvec[1])));

  if(pvec[1]>=1,
     my(pvecsub1=pvec); pvecsub1[1]--;
     f(pvecsub1, qvec))
  +
  sum(j=1,k-1, f(pvec[1..j], qvec[1..j])
             * f(pvec[j+1..k], qvec[j+1..k]))
  +
  if(qvec[k]>=1,
     my(qvecsub1=qvec); qvecsub1[k]--;
     f(pvec, qvecsub1));
}

count_footprint_using_f(footprint) =
{
  my(pvec,qvec); [pvec,qvec]=footprint_to_pqvecs(footprint);
  \\ each p[i]-1 and q[i]-1, except p[1] unchanged
  for(i=2,#pvec, pvec[i]--);
  for(i=1,#qvec, qvec[i]--);
  f(pvec,qvec);
}

{
  print1("checking ... ");
  for(fbin=0,2^10,
     my(footprint=fbin_to_footprint(fbin));
     count_footprint_using_f(footprint)
       == count_footprint_using_upwards(footprint) || error());
  print("good");
}


\\-----------------------------------------------------------------------------
\\ Balanced Binary
\\
\\ vpar_to_balanced_binary() codes a forest as balanced binary by the usual
\\ preorder traversal.  Each bit is
\\
\\      1    a vertex, and descend to next greater depth level
\\      0    ascend to smaller depth level
\\
\\ So each vertex is
\\
\\      11   vertex with a child below it
\\      10   vertex with no child below it
\\
\\ and hence the footprint vector is obtained from balanced binary by taking
\\ the bit below each 1-bit.

balanced_binary_to_footprint(bal) =
{
  my(footprint=vector(hammingweight(bal)),  \\ hammingweight = num vertices
     upto=0);
  forstep(i=logint(max(bal,1),2),1,-1, \\ bit pos of highest 1-bit, and down
     if(bittest(bal,i),
        footprint[upto++] = bittest(bal,i-1))); \\ bit below each 1-bit
  footprint;
}
{
  print1("checking ... ");
  for(n=0,8,
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
           my(bal=vpar_to_balanced_binary(vpar));
           footprint_of_vpar(vpar) == balanced_binary_to_footprint(bal)
             || error()));
  print("good");
}

\\ The binary footprint coding of a balanced binary is therefore a high
\\ 1-bit then the bit below each 1-bit of the balanced binary, except the
\\ last 1-bit (the lowest).  The lowest 1-bit is the last vertex and always
\\ has a 0 below it (possibly below the binary point) and is skipped for the
\\ binary footprint code.
\\
\\ This bit selection is OEIS A209859.  Antti Karttunen gives that as a
\\ transform (of any integer, not just balanced binary),
\\
\\     00...00 1  -> 0           replacement, for 1 or more 0s above
\\         ...00  -> nothing     low 0-bits discarded
\\
\\ A 1-bit without one or more 0-bits above it is retained.  This includes
\\ the most significant bit which is reckoned with nothing at all above.
\\ The bit patterns are sought only in the original n, so the 0 from a
\\ replacement is not a 0 above another 1-bit too (that would knock out all
\\ 1-bits).
\\
\\ The replacement can also be thought of as splitting n by its 1-bits then
\\ form A209859(n) by starting with a 1-bit and for each gap between 1-bits
\\ of n if there are no 0s put a 1-bit to A209859(n) or if there are 0s then
\\ put a 0-bit to A209859(n).
\\
\\              n =  1 00 1 1 1 000 1 00     binary
\\     A209859(n) =  1  0  1 1   0           binary
\\          high 1 --^  ^  ^ ^   ^
\\                      rest between 1..1 pairs in n
\\
\\ Or equivalently, start with a 1-bit in A209859(n) and at each 1-bit of n
\\ from most significant down to second-least significant, take the bit
\\ below it.  Or at each 1-bit of n from second-most significant to least
\\ significant, take the bit above it.
\\
\\ In all cases, the bit length of A209859(n) is the number of 1-bits in n.
\\
\\ The 0s between 1s of a balanced binary n are where the depth of the next
\\ vertex decreases so vertex v which is 1-bit before does not have the next
\\ v+1 of the next 1-bit as a child.

A209859(n) =
{
  \\ The return is going to have hammingweight(n) total bits.
  \\ vec[1]=1 is the only initialization needed, since the rest are written
  \\ by the loop, but it's convenient to let vector() do them all so no need
  \\ to test n!=0 before setting vec[1].
  \\
  \\ The bit testing here is the "bit above each 1-bit" style, from second
  \\ most-significant down to least significant, in the way described above.
  \\
  \\ The forstep() loop could instead be while(upto<#vec, ...), since each
  \\ bittest() steps "upto" until hammingweight()==#vec total many 1-bits.
  \\ But that's one of those slightly worrying things where if for some
  \\ strange reason the bittest and hammingweight didn't agree then you'd
  \\ get an infinite loop bittesting above the high end of n.  Of course
  \\ they should never disagree, but unless you're squeezing the last drop
  \\ out of some assembler then no chance of an infinite loop has some
  \\ merit, for instance in case n is some unforeseen data type.

  my(vec=vector(hammingweight(n),i,1),
     upto=#vec+1);     \\ position in vec, ready for pre-decrement
  for(i=0,logint(max(n,1),2)-1,
     if(bittest(n,i),
        vec[upto--] = bittest(n,i+1)));  \\ bit above the 1-bit in n
  fromdigits(vec,2);
}

A209859_samples={[   \\ OFFSET=0
   0, 1, 1, 3, 1, 2, 3, 7, 1, 2, 2, 5, 3, 6, 7, 15, 1, 2, 2, 5, 2, 4, 5,
   11, 3, 6, 6, 13, 7, 14, 15, 31, 1, 2, 2, 5, 2, 4, 5, 11, 2, 4, 4, 9, 5
   ]};
vector(#A209859_samples,n,n--; A209859(n)) == A209859_samples   || error();

bitlength(n) = if(n==0,0, logint(n,2)+1);
bitlength(0) == 0 || error();
bitlength(4) == 3 || error();
bitlength(7) == 3 || error();
{
  for(n=0,256,
     bitlength(A209859(n)) == hammingweight(n) || error());
}

\\ This form builds the return with bit power p instead of into a vector.
A209859_by_p(n) =
{
  if(n==0,0,
     my(ret=0, p=1);
     for(i=0,logint(n,2)-1,
        if(bittest(n,i),
           if(bittest(n,i+1),ret+=p);
           p<<=1));
     ret+p);
}
vector(256,n,n--; A209859_by_p(n)) == vector(256,n,n--; A209859(n)) || error();

\\ This form is bit-twiddling to locate successive 1-bits of n.  Good in a
\\ single machine word, but scanning for bits will be better in a bignum.
A209859_by_b(n) =
{
  if(n==0,0,
     my(ret=0, p=1, b);
     while(b=bitand(n,-n); n-=b,
           if(bitand(n,(b<<1)),ret+=p);
           p<<=1);
     ret+p);
}
vector(256,n,n--; A209859_by_b(n)) == vector(256,n,n--; A209859(n)) || error();


\\ The count of how many forests have a given footprint is how many times
\\ A209859(bal) = fbin, over all balanced binary "bal".
{
  my(bal=0,                 \\ start empty forest
     n = 8,                 \\ num vertices <= n
     bal_limit = 4^n,
     counts=vector(2^n));   \\ counts[1+fbin] = how many forests fbin
  while(bal < bal_limit,
        my(fbin=A209859(bal));
        counts[1+fbin]++;   \\ fbin>=0 so 1+ for 1-based vector counts[]
        bal=vpar_balanced_binary_next(bal));

  my(counts_by_upwards=vector(2^n,fbin,fbin--;
                              count_footprint_using_upwards_of_fbin(fbin)));
  counts == counts_by_upwards  || error();
}


\\-----------------------------------------------------------------------------
\\ Footprint Counts from Balanced Binary
\\
\\ In balanced binary, a vertex with a child has a 1 following and a vertex
\\ without a child has a 0 following.  So for example
\\
\\     0  0  1110  0  0  110  0        footprint
\\     10 10 11110 10 10 1110 10       balanced binary form
\\       ^  ^     ^  ^  ^    ^  ^ ---- places for remaining 0s
\\
\\ This balanced binary form is what the footprint demands, and then
\\ additional 0s for balance are added to it to make a complete balanced
\\ binary.  These additional 0s can go after any of the 0s which the
\\ footprint demands, subject to the balance condition that nowhere
\\ preceding 0s exceed preceding 1s.
\\
\\ In the example above, the 1110 must be followed by at least two 0s, but
\\ they can be either immediately after, or after the final 10.
\\
\\ count_footprint_using_tvecs() etc effectively takes the places for
\\ additional 0s in groups of the 11..11 childful and their following
\\ childless, with binomial to count the ways to arrange 0s between the
\\ childless.
\\
\\ Doing it without the binomials means imagining each childless is its own
\\ single q[i]=1 and corresponding p[i] is how many childful precede it,
\\ possibly 0.
\\
\\ Stair-step paths like drawn above widen accordingly, being various p=0 so
\\ the path top does not go up.
\\

footprint_to_bvec(footprint) =
{
  my(num_childless=sum(i=1,#footprint, footprint[i]==0),
     bvec=vector(num_childless),
     p=0,           \\ how many childful immediately preceding i
     upto=0);       \\ position in bvec[]
  for(i=1,#footprint,
     if(footprint[i], p++,
        bvec[upto++]=p; p=0));
  bvec;
}

count_footprint_using_bb(footprint) =
{
  my(bvec=footprint_to_bvec(footprint),
     tvec=tvec_first(bvec),
     count=0);
  until((tvec=tvec_next(bvec,tvec))=='none,
        count++);
  count;
}
{
  print1("checking ... ");
  for(fbin=0,2^10,
     my(footprint=fbin_to_footprint(fbin));
     count_footprint_using_f(footprint)
       == count_footprint_using_bb(footprint) || error());
  print("good");
}

\\ A vector of all q=1 can go with bvec, and they are then behave as a pqvec
\\ pair.
count_footprint_using_bbpq(footprint) =
{
  my(bvec=footprint_to_bvec(footprint),
     qvec=vector(#bvec,i, 1));
  count_footprint_using_upwards_of_pqvecs(bvec,qvec);
}
{
  print1("checking ... ");
  for(fbin=0,2^10,
     my(footprint=fbin_to_footprint(fbin));
     count_footprint_using_f(footprint)
       == count_footprint_using_bbpq(footprint) || error());
  print("good");
}

\\ For computer calculation, binomial on childless together is likely to be
\\ more efficient, just on the basis of size and crunching of vectors.
\\ The worst case in both is probably quadratic in number of vertices,
\\
count_footprint_using_bb_of_fbin(fbin) =
{
  fbin>=0 || error("count_footprint_using_bb_of_fbin() is for fbin>=0");
  if (fbin==0,return(1));  \\ empty footprint is the empty forest only

  my(clen=hammingweight(fbin));
  my(p=valuation(fbin+1,2)); fbin>>=p; if(fbin==0,p--);  /* num low 1s */
  clen -= p;
  my(counts=vector(clen,d, 1));
  while(fbin,
        bitand(fbin,1)==0 || error();
        fbin>>=1;                                         /* take one low 0 */
        p=valuation(fbin+1,2); fbin>>=p; if(fbin==0,p--);  /* num low 1s */
        clen -= p;
        counts=vector(clen,d,d--; sum(t=0,d+p, counts[1+d+p-t])));
  #counts==1 || error();
  counts[1];
}
{
  print1("checking ... ");
  for(fbin=0,2^10,
     my(footprint=fbin_to_footprint(fbin));
     count_footprint_using_bb_of_fbin(fbin)
       == count_footprint_using_upwards(footprint)  || error());
  print("good");
}

print();


\\-----------------------------------------------------------------------------
\\ Balanced Binary Range

\\ The first, ie. smallest, balanced binary number of a given footprint is
\\ found by taking the remaining 0s as high as possible.
\\
\\     0  0  1110     0  0  110     0     footprint
\\     10 10 11110    10 10 1110    10    balanced binary form
\\                000           00        remaining 0s as high as possible
\\
\\ In terms of the forest, this is a return to depth 0 every time, so the
\\ first of each p run is a new root (and path down), and each childless
\\ (apart from the first after the p run) is also a new root (a singleton).
\\
\\              /\
\\             /  \           /\
\\            /    \         /  \         mountain range always returning
\\     /\ /\ /      \ /\ /\ /    \ /\     to sea-level
\\

\\ footprint is a vector of 0s and 1s.
\\ Return a balanced binary integer.
footprint_to_balanced_binary_first(footprint) =
{
  my(vec=vector(2*#footprint),
     d=0,       \\ net depth
     upto=0);
  for(v=1,#footprint,
     vec[upto++] = 1;
     d++;
     if(footprint[v]==0,
        \\ this vertex childless, put 0s so the next is at the top again
        upto+=d;
        d=0));
  fromdigits(vec,2);
}

\\ Return an integer which has decimal digits 0,1 for the binary bits of n.
\\ So printing to_binary(n) shows n written in binary.
to_binary(n) = fromdigits(binary(n));

{
  print("Balanced binary first of footprint");
  print1("  ");
  for(fbin=0,15,
     my(footprint=fbin_to_footprint(fbin));
     print1(footprint_to_balanced_binary_first(footprint)","));
  print("     (A166751)");
  print1("  binary ");
  for(fbin=1,7,
     my(footprint=fbin_to_footprint(fbin));
     print1(to_binary(footprint_to_balanced_binary_first(footprint))","));
  print();
}

footprint_to_balanced_binary_first([]) == 0  || error();
{
  \\ check footprint_to_balanced_binary_first() by searching through all
  \\ balanced binary and recording which is the first of each "fbin"
  my(max_vertices=7,
     found_first=vector(2^max_vertices - 1),  \\ 1 to 1[max_vertices-1]
     bal=2);  \\ balanced binary
  while(#binary(bal) <= 2*max_vertices,
        my(fbin=A209859(bal));
        if(!found_first[fbin], found_first[fbin]=bal);
        bal=vpar_balanced_binary_next(bal));
  for(fbin=1,#found_first,
     my(footprint=fbin_to_footprint(fbin));
     footprint_to_balanced_binary_first(footprint) == found_first[fbin]
       || error(fbin));
}

\\ First balanced binary is OEIS A166751.
\\
A166751(n) =
{
  \\ This inspects the bits of fbin=n, then the output vec is the same as in
  \\ footprint_to_balanced_binary_first() above.
  \\
  \\ The bits of n=fbin tested are the second highest, down to one below the
  \\ radix point.  That last is bittest(n,-1) which is always 0 and is the
  \\ last vertex omitted from fbin.

  my(k=if(n==0,-1,logint(n,2)),
     vec=vector(2*k+2),
     upto=0,
     d=0);
  while(k-- >= -1,
        vec[upto++]=1;
        d++;
        if(!bittest(n,k),
           upto+=d;
           d=0));
  upto == #vec || error();   \\ final 0s would be upto end of vec
  fromdigits(vec,2);
}
{
  my(A166751_samples = [
    2,10,12,42,44,50,56,170,172,178,184,202,204,226,240,682,684,690,696,
    714,716,738,752,810,812,818,824,906
  ]);
  vector(#A166751_samples,n, A166751(n)) == A166751_samples  || error();
}
{
  for(n=0,256,
     A166751(n) == footprint_to_balanced_binary_first(fbin_to_footprint(n))
       || error());
}


\\ The last, ie. biggest, balanced binary number of a given footprint is
\\ found by taking the remaining 0s as low as possible, which means all at
\\ the end.
\\
\\     0  0  1110  0  0  110  0            footprint
\\     10 10 11110 10 10 1110 10           balanced binary form
\\                               000 00    remaining 0s as low as possible
\\
\\ The effect on the footprint vector is to map
\\
\\     1 -> 1           unchanged
\\     0 -> 10
\\
\\ and then low 0s to balance, which is one 0 for each childful 1 in the
\\ footprint.
\\
\\ In terms of the forest, this is every chain down p having all its
\\ childless at the bottom and the next chain down commencing at the bottom
\\ there too.

\\ footprint is a vector of 0s and 1s.
\\ Return a balanced binary integer.
footprint_to_balanced_binary_last(footprint) =
{
  my(vec=vector(2*#footprint),
     upto=0);
  for(v=1,#footprint,
     vec[upto++] = 1;
     if(footprint[v]==0, upto++));  \\ another 0
  fromdigits(vec,2);
}

{
  print("Balanced binary last of footprint");
  print1("  ");
  for(fbin=0,15,
     my(footprint=fbin_to_footprint(fbin));
     print1(footprint_to_balanced_binary_last(footprint)","));
  print("     (A071162)");
  print1("  binary ");
  for(fbin=1,7,
     my(footprint=fbin_to_footprint(fbin));
     print1(to_binary(footprint_to_balanced_binary_last(footprint))","));
  print();
}

{
  \\ Check footprint_to_balanced_binary_last() by searching through all
  \\ balanced binary and recording which is the last of each "fbin".
  my(max_vertices=7,
     found_last=vector(2^max_vertices - 1),  \\ 1 to 1<max_vertices-1>
     bal=2);  \\ balanced binary
  while(#binary(bal) <= 2*max_vertices,
        my(fbin=A209859(bal));
        found_last[fbin]=bal;
        bal=vpar_balanced_binary_next(bal));
  for(fbin=1,#found_last,
     my(footprint=fbin_to_footprint(fbin));
     footprint_to_balanced_binary_last(footprint) == found_last[fbin]
       || error(fbin));
}

\\ Last balanced binary is OEIS A071162.
\\ Antti Karttunen in that sequence gives transform rule on the fbin bits
\\
\\     0 -> 01
\\     1X -> 1(rewrite X)0      where X everything after the 1
\\
\\ This X has the effect of putting low 0s at the end.  The high 1-bit of
\\ fbin, which indicates the length but is not a footprint bit as such,
\\ means everywhere is one 1-bit late, hence 0->01 instead of 0->10 as in
\\ the footprint vector form described above.
\\
A071162(n) =
{
  my(k=if(n==0,-1,logint(n,2)),
     vec=vector(2*k+2),
     upto=0,
     d=0);
  forstep(i=k-1,-1,-1, vec[upto++]=1; if(!bittest(n,i), upto++));
  fromdigits(vec,2);
}
{
  my(A071162_samples = [
   0,2,10,12,42,44,52,56,170,172,180,184,212,216,232,240,
   682,684,692,696,724,728,744,752,852,856,872,880,936,944
  ]);
  vector(#A071162_samples,n,n--; A071162(n)) == A071162_samples  || error();
}
{
  for(n=0,256,
     my(bal=A071162(n));
     vpar_is_balanced_binary(bal) || error();
     bal == footprint_to_balanced_binary_last(fbin_to_footprint(n)) || error();

     \\ Every childful vertex is child of the preceding childful vertex, or
     \\ root if no preceding childful.
     my(vpar=vpar_from_balanced_binary(bal),
        is_childful=vpar_is_childful_vector(vpar), \\ the footprint
        want_p=0);
     for(v=1,#vpar,
        vpar[v] == want_p  || error();
        if(is_childful[v],
           \\ new childful for everything to go under
           want_p=v));
  );
}


\\ It can be noted that the footprints of successive balanced binary are not
\\ in ascending order, in general.  For example, the following printout has
\\ the 4-vertex forests of footprints 12 and 13 intermingled.
\\
\\     12,13,12,12,13
\\
{
  print("fbin of balanced binary");
  print1("  ");
  my(bal=2);
  while(bal < 2^8,
     print1(A209859(bal),",");
     bal=vpar_balanced_binary_next(bal));
  print();
}


\\-----------------------------------------------------------------------------
print("end");
