\\ Copyright 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);
read("../vpar.gp");


\\ Usage: gp <footprints-restricted.gp
\\
\\ This is a variation on footprint-preorder.gp (see references there)
\\ considering Knuth's original answer before errata of 23 Apr 2019.
\\
\\ The pre-errata answer was too small.  It counted over only some preorder
\\ forests, not all of them.


\\-----------------------------------------------------------------------------
\\ These helpers are the same as in footprint-preorder.gp.
\\ See comments in that file.

\\ footprint is a vector of 1,0 flags.
\\ Return [pvec,qvec] vector of two vectors pvec and qvec.
footprint_to_pqvecs(footprint) =
{
  my(plist=List([]),
     qlist=List([]),
     i=1);
  while(i<=#footprint,
        my(p=0); while(i<=#footprint && footprint[i]==1, p++;i++);
        my(q=0); while(i<=#footprint && footprint[i]==0, q++;i++);
        listput(plist,p);
        listput(qlist,q));
  [Vec(plist), Vec(qlist)];
}

\\ b is an integer.  Return a footprint vector of 0s and 1s.
b_to_footprint(b) =
{
  b >= 1 || error();
  concat(binary(b)[^1], [0]);
}

\\ vpar is a preorder labelled forest.  Return its childfulness footprint.
footprint_of_vpar(vpar) = vpar_is_childful_vector(vpar);


\\------------------------------------------------------------------------------
\\ count_restricted_footprint() is Knuth's pre-errata answer to exercise
\\ 30(b) counting forests of given footprint.

\\ Catalan numbers (A000108)
Catalan_number(n) = binomial(2*n,n) / (n+1);

\\ 7.2.1.6 exercise 30(b) answer first edition page 788,
\\ or draft fasc4a.ps.gz page 53
count_restricted_footprint(footprint) =
{
  \\ footprint_to_pqvecs() is full lengths of 1,0 runs.
  my(pvec,qvec); [pvec,qvec]=footprint_to_pqvecs(footprint);
  my(k = #pvec);
  for(i=2,k, pvec[i]--);  \\ reduce all -1 except pvec[1]
  for(i=1,k, qvec[i]--);
  prod(i=1,k, binomial(pvec[i]+qvec[i], qvec[i])) * Catalan_number(k-1);
}

{
  \\ Exercise 30(b) question footprint and pre-errata answer 240240.
  my(footprint
   =[1,0, 1,0, 1,1,0, 1,1,1,1,1,1,0,0,0,0, 1,0, 1,0, 1,0,0,0, 1,0, 1,1,0,0,0]);
  my(got=count_restricted_footprint(footprint));
  print("Knuth 30(b) pre-errata answer "got);
  got==240240  || error();
}

{
  my(lo=8,hi=31);
  print("Count restricted for b="lo" to "hi);
  for(b=lo,hi,
     my(footprint=b_to_footprint(b));
     print1(count_restricted_footprint(footprint),","));
  print("\n");

  \\ not in OEIS: 1,1,1,1,3,1,3,1,1,1,1,1,1,2,2,1,4,2,1,1,6,1,4,1,
}

\\ The first footprint where the restricted formula differs from all
\\ preorders is n=4 vertices
\\
\\     footprint = [0, 1, 0, 0]
\\
\\ The restricted formula is, with -1s applied, p[1]=0, q[1]=0,
\\ p[2]=0, q[2]=1.
\\
\\     binomial(0+0,0) * binomial(0+1,1) * Catalan_number(1) = 1
\\
\\ But there are 2 preorder forests with this footprint, being 2 places to
\\ put the last vertex 4.
\\
\\     1 2       1 2 4
\\       |\        |
\\       3 4       3
\\
\\ Also in n=4 vertices, footprint=[1,0,1,0] is count 1 vs 2 in a similar
\\ way.  It is, with -1s applied, p[1]=1, q[1]=0, p[2]=0, q[2]=0 for same
\\ binomials (swapped).  The forests of this footprint are
\\
\\     1 3       1
\\     | |       |\
\\     2 4       2 3
\\                 |
\\                 4


\\-----------------------------------------------------------------------------
\\ Restricted Total
\\
\\ The way the restricted formula is too small can be seen in its grand
\\ total over all footprints of length n.  All preorders would be
\\ Catalan(n), but the restricted formula total is smaller.
\\
\\ Footprints must have last entry 0, and other entries any 0,1 for 2^(n-1)
\\ combinations.  Total count trees of all these footprints is OEIS A086622.

A086622_samples = \
  [1, 2, 5, 12, 30, 76, 197, 520, 1398, 3820, 10594, 29768, 84620, 243000];
{
  print("Total restricted length n, OEIS A086622");
  for(n=1,10,
     my(total=0);
     forvec(footprint=vector(n,i, [0,i!=n]),
           total += count_restricted_footprint(footprint));
     print1(total,",");
     total == A086622_samples[n] || error());
  print("\n");
}

\\ Jose Luis Ramirez Ramirez in OEIS A086622 gives the following formula for
\\ A086622, adapted here to sum with index k starting k=1.
total_restricted(n) = \
  sum(k=1,floor((n+1)/2), 2^(n+1-2*k)*binomial(n-k,k-1)*Catalan_number(k-1));
{
  for(n=1,#A086622_samples,
     A086622_samples[n] == total_restricted(n) || error());
}
\\
\\ The sum is over k many parts in the footprint.
\\ Each part is r>=2 vertices in the n+1 tree form, so k <= floor((n+1)/2).
\\
\\ After taking 2*k vertices for those r>=2 minimums, there are n+1-2*k left
\\ to assign to the k parts, which by s things into m places is
\\
\\     binomial(k-1+n+1-2*k, k-1) = binomial(n-k, k-1)
\\
\\ Each part is r=p+q for all combinations p>=1 and q>=1.  Sum of binomials
\\ of those, with -1, is a whole row of Pascal's triangle,
\\
\\     r = p+q
\\     sum q=0 to r-2 of binomial(r-2, q-1)  = 2^(r-2)
\\
\\ Total of these r is sum(r) = n+1 all tree vertices, so product these
\\ powers-of-2 over all is 2^(n+1-2*k).


\\ For interest, the formula without the Catalan(k-1) factor would be ways
\\ to rearrange footprint 1s and 0s within their blocks "111000".
\\
count_restricted_footprint_sans_Catalan(footprint) =
{
  my(pvec,qvec); [pvec,qvec]=footprint_to_pqvecs(footprint);
  my(k = #pvec);
  for(i=2,k, pvec[i]--);
  for(i=1,k, qvec[i]--);
  prod(i=1,k, binomial(pvec[i]+qvec[i], qvec[i]));
}
\\ {
\\   my(lo=8,hi=31);
\\   for(b=lo,hi,
\\      my(footprint=b_to_footprint(b),
\\         k=length(footprint_to_pqvecs(footprint)[1]));
\\      print1(count_restricted_footprint(footprint)/Catalan_number(k-1),","));
\\   print();
\\   \\ not in OEIS: 1,1,1,1,3,1,3,1,1,1,1,1,1,1,2,1,4,2,1,1,6,1,4,1,
\\ }
\\
\\ The total restricted sans Catalan is the Pell numbers (A000129).
\\ Think the sum over binomials and powers of 2 is one of the many
\\ moderately usual forms for the Pell numbers.
\\
sqrt2 = quadgen(8);
Pell(n) = imag( (1+sqrt2)^n );  \\ per GP code by Michael Somos in A000129
{
  print1("checking ... ");
  for(n=1,10,
     my(total=0);
     forvec(footprint=vector(n,i, [0,i!=n]),
           total += count_restricted_footprint_sans_Catalan(footprint));
     total == Pell(n)  || error());
  print("good");
}


\\-----------------------------------------------------------------------------
\\ A class of binary trees counted by the restricted formula can be formed
\\ by following Knuth's R,N,L,B specification and putting the k-1 many Bs as
\\ a binary tree and underneath, at its k many "external" positions, put
\\ chains of R,L down.
\\
\\ Chains of R,L down to bottom-most N are the "degenerate" binary trees
\\ described in Knuth's exercise 31 (immediately following exercise 30).
\\ A given number of Rs and given number of Ls can be arranged with the Rs
\\ and Ls in any sequence down the chain.  In-order traversal is the same
\\ "RR N LL" for all such.  For example,
\\
\\     R           R         L          L
\\      \           \       /          /         in-order
\\       R           L     R          L         R,R, N, L,L
\\        \         /       \        /
\\         L       L         R      R
\\        /       /           \      \
\\       L       R             L      R
\\      /         \           /        \
\\     N           N         N          N
\\
\\ In the restricted formula, binomial() chooses positions of the Ls among
\\ all the Rs+Ls.  The Catalan(k-1) factor is the ways to form the k-1
\\ vertex tree of Bs at the top.
\\
\\ Bs at the top can be thought of as non-increasing number of children
\\ going down the tree.  For a binary tree this merely means a 2-child
\\ vertex cannot be under a 1-child parent.  Does this class of binary trees
\\ have a name?  There are A086622(n) many of n vertices.
\\
\\ In terms of ordered forests, in the left=prev right=last form described
\\ in footprints-preorder.gp, a childful with prev sibling must have its
\\ binary tree parent both childful and having prev sibling.  The binary
\\ tree parent is next sibling when it has one, and if not (because a last
\\ child) then the forest parent.
\\
\\     v as binary tree left child         v as binary tree right child
\\      is when have next sibling           is when no next sibling
\\
\\            p                                p
\\           /      forest                      \        forest
\\          v     p=next sibling of v            v     p=parent of v
\\         / \                                  / \    (v last child of p)
\\

\\ vpar is a preorder forest.
\\ Return 1 if it is the "Bs at top" form described above.
is_preorder_Btop(vpar) =
{
  my(num_children=vpar_num_children_vector(vpar),
     next_sibling=vpar_next_sibling_vector(vpar),
     prev_sibling=vpar_next_sibling_vector(vpar,1),
     binary_parent=vector(#vpar,v,
                          if(next_sibling[v],next_sibling[v],vpar[v])));
  for(v=1,#vpar,
     if(num_children[v] && prev_sibling[v],             \\ v is type B
        my(p=binary_parent[v]);
        if(p,     \\ no parent at binary tree root
           if(! (num_children[p] && prev_sibling[p]),   \\ p must be type B
               return(0)))));
  1;
}

{
  print1("checking ... ");
  for(n=2,10,
     my(footprints=List([]));    \\ all possible footprints of n vertices
     forvec(footprint=vector(n,i, [0,i!=n]),
           listput(footprints,footprint));
     footprints=Set(footprints);
     my(counts=vector(#footprints));
     #footprints == 2^(n-1) || error();

     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
           is_preorder_Btop(vpar) || next;           \\ of "Bs at top" type
           my(footprint=footprint_of_vpar(vpar),
              pos=setsearch(footprints,footprint));
           pos || error("oops, footprint not found: "footprint);
           counts[pos]++);

     for(i=1,#footprints,
        counts[i]!=0 || error();
        count_restricted_footprint(footprints[i]) == counts[i] || error());
  );
  print("good");
  print();
}

\\-----------------------------------------------------------------------------
