\\ Copyright 2018, 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(strictargs,1)
read("../vpar.gp");


\\ Usage: gp <most-maximum-matchings.gp
\\
\\ This is some code creating the trees with most maximum matchings per
\\
\\     Clemens Heuberger and Stephan Wagner, "The Number of Maximum
\\     Matchings In a Tree", Discrete Mathematics, volume 311, issue 21,
\\     November 2011, pages 2512-2542.
\\     https://arxiv.org/abs/1011.6554
\\
\\     https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3226351/
\\     full text html
\\
\\ Maximum matchings in a tree are those of the maximum size possible in
\\ that tree.  This size is the match number of the tree, per
\\ vpar_matchnum().  The match number and count of such matchings are
\\ calculated together by vpar_matchnum_and_count().
\\
\\ Thse are also the highest term count*x^matchnum in the match polynomial
\\ vpar_matchpoly(), or the lowest term count*x^(n-2*matchnum) in
\\ vpar_adjacency_poly().  But vpar_matchnum_and_count() is more efficient
\\ for getting just that term.
\\
\\ Heuberger and Wagner determine the form of the tree of n vertices which
\\ has the most maximum matchings.  There is a unique tree of most maximum
\\ matchings, except at n=6 and n=34 where a second "other" tree has equal
\\ most.
\\
\\ See also my Graph::Maker::MostMaximumMatchingsTree module for creating
\\ the trees in Perl Graph.pm (or any compatible class) too,
\\
\\     http://user42.tuxfamily.org/graph-maker-other/index.html
\\
\\ This module includes coordinates suitable for drawing the trees in a
\\ layout similar to Heuberger and Wagner, though presently this is secret
\\ undocumented will change, etc.  The docs include a list of the House of
\\ Graphs entries for the trees.
\\


\\-----------------------------------------------------------------------------

default(recover,0);
default(strictargs,0)

\\     1   singleton
L_vpar = [0];

\\     1=root            F fork
\\     |
\\     2                 r=1 is root
\\    / \
\\   3   4
F_vpar = [0,1,2,2];

\\
\\   6       1=root      chain part
\\    \      |
\\     5--4--2*--3       s=1 is root
\\    /                  r=2 is below where further tree T
\\   7                        underneath this C will go
C_vpar = [0,1,2,2,4,5,5];

\\ add_C() adds k many copies of "C" above the root of vpar and returns the
\\ result.
\\
\\           1 = root in the return
\\   *\      |
\\     *--*--*--*            \
\\   */      |                |  chain k many copies of C
\\           *                |
\\   *\      |                |
\\     *--*--*--*            /
\\   */      |
\\          root of given vpar
\\
add_C(k,vpar) = for(i=1,k, vpar=vpar_join(C_vpar,2,vpar)); vpar;

\\ make_CL() and make_CF() creates a chain of k copies of C and an L or F
\\ respectively underneath.
\\
\\           1 = root                     1 = root
\\   *\      |                    *\      |
\\     *--*--*--*                   *--*--*--*
\\   */      |                    */      |
\\           *                            *
\\   *\      |                    *\      |
\\     *--*--*--*                   *--*--*--*
\\   */      |                    */      |
\\           *  L below                   *
\\                                       / \    F below
\\                                      *   *
\\
make_CL(k) = add_C(k,L_vpar);
make_CF(k) = add_C(k,F_vpar);


\\ make(n) builds and returns the tree of n vertices.  Special n=6.5 or
\\ n=34.5 are the "other" trees of n=6 or 34 vertices respectively.
\\
\\ The several cases n mod 7 and the "k" many "C" parts to concatenate make
\\ the code a bit big, but it's the sort of thing where you can start with
\\ something logical like this and consider later if there's a choice of
\\ root or labelling which would be more uniform.
\\
\\ The code illustrates joining pieces of trees to make a big whole.  The
\\ the copying of intermediate vectors for this isn't particularly
\\ efficient.  For greatest efficiency, the idea would be to make one vpar
\\ vector of the final length and fill it, keeping track of where "upto" etc.
\\ There's a certain number of chains of C, which can be joined up or get F
\\ ends in appropriate ways.
\\
\\ Vertex numbering is unspecified.
\\
make(n) =
{
  my(vpar);
  if(
     \\ initial exceptions
     n<=3,
     vpar = vpar_make_path(n);
     ,
     n==6,  \\ T6,1
     vpar = vpar_make_star(6);
     ,
     n==6.5,  \\ T6,2
     \\           /-*
     \\ *--*--*--*
     \\           \-*
     vpar = [0,1,2,3,4,4];
     ,
     n==10,
     \\        L        figure 5(c)   N = 10
     \\        |        different from N==3 mod 7
     \\   F -- * -- F
     vpar=[2,0];
     vpar = vpar_join(vpar,1, F_vpar);
     vpar = vpar_join(vpar,1, F_vpar);
     ,
     n==13,
     \\      *     *     *
     \\      |     |     |         figure 5(d) T13
     \\   *--*--*--*--*--*--*      different from F ends of 6 mod 7
     \\      |     |     |
     \\      *     *     *
     vpar=vpar_make_star(5);
     vpar = vpar_join(vpar,2, vpar_make_star(4));
     vpar = vpar_join(vpar,3, vpar_make_star(4));
     ,
     n==20,
     \\      *     *     CL
     \\      |     |     |         figure 5(e) T20
     \\   *--*--*--*--*--*--*      different from F ends of 6 mod 7
     \\      |     |     |
     \\      *     *     *
     vpar=make_CL(1);
     vpar = vpar_join(vpar_make_star(4),1,vpar);
     vpar = vpar_reroot(vpar,2);
     vpar = vpar_join(vpar_make_star(4),1,vpar);
     vpar = vpar_reroot(vpar,2);
     vpar = vpar_join(vpar_make_star(4),1,vpar);
     ,
     n==34.5,  \\ T34,2 is 4(d) with k0=1 rest =0 (cf general case k6=1 rest 0)
     vpar=[2,0];
     vpar = vpar_join(vpar,1, F_vpar);
     vpar = vpar_join(vpar,1, F_vpar);
     vpar = vpar_join(vpar,1, F_vpar);
     vpar = add_C(1,vpar);  \\ k_0 == 1
     vpar = vpar_join([0],1, vpar);
     vpar = vpar_join(vpar,1, F_vpar);
     vpar = vpar_join(vpar,1, F_vpar);
     vpar = vpar_join(vpar,1, F_vpar);
     ,
     \\--------------
     \\ general forms
     \\
     \\ At n=30 there are special branch lengths different from the general
     \\ pattern for n==2 mod 7.
     \\
     n%7==1,
     vpar = make_CL((n-1)/7);
     ,
     n%7==2 && n>=9,
     \\       C3L        C4L        figure 4(a)    N = 2 mod 7
     \\        |          |
     \\   L -- m -- C0 -- r -- L
     \\        |          |
     \\       C1L        C2L
     my(kfunc = (j)->if(j==0,  max(0, floor((n-37)/35)),
                        n>=37, floor((n-2+7*j)/35),
                               floor((n-9+7*j)/35)));
     vpar=[2,0];
     vpar = vpar_join(vpar,1, make_CL(kfunc(3)));
     vpar = vpar_join(vpar,1, L_vpar);
     vpar = vpar_join(vpar,1, make_CL(kfunc(1)));
     vpar = add_C(kfunc(0),vpar);
     vpar = vpar_join([0],1, vpar);
     vpar = vpar_join(vpar,1, make_CL(kfunc(4)));
     vpar = vpar_join(vpar,1, L_vpar);
     vpar = vpar_join(vpar,1, make_CL(kfunc(2)));
     ,
     n%7==3 && n>=17,
     \\         C2F        figure 4(b)  N == 3 mod 7
     \\          |
     \\   C1F -- m -- C3F
     \\          |
     \\         C0F
     my(kfunc = (j)->floor((n-17+7*j)/28));
     vpar=[0];
     vpar = vpar_join(vpar,1, make_CF(kfunc(0)));
     vpar = vpar_join(vpar,1, make_CF(kfunc(1)));
     vpar = vpar_join(vpar,1, make_CF(kfunc(2)));
     vpar = vpar_join(vpar,1, make_CF(kfunc(3)));
     ,
     n%7==4,
     vpar = make_CF((n-4)/7);
     ,
     n%7==5,  \\ 4(c)
     my(kfunc = (j)->floor((n- 5+7*j)/21));
     vpar = [0];
     vpar = vpar_join(vpar,1, make_CL(kfunc(0)));
     vpar = vpar_join(vpar,1, make_CL(kfunc(1)));
     vpar = vpar_join(vpar,1, L_vpar);
     vpar = vpar_join(vpar,1, make_CL(kfunc(2)));
     ,
     n%7==6 && n>=27,
     \\         C5F        C6F        figure 4(d)  N == 6 mod 7
     \\          |          |
     \\   C3F -- m -- C0 -- r -- C4F
     \\          |          |
     \\         C1F        C2F
     my(kfunc = (j)->floor((n-27+7*j)/49));
     vpar=[2,0];
     vpar = vpar_join(vpar,1, make_CF(kfunc(5)));
     vpar = vpar_join(vpar,1, make_CF(kfunc(3)));
     vpar = vpar_join(vpar,1, make_CF(kfunc(1)));
     vpar = add_C(kfunc(0),vpar);
     vpar = vpar_join([0],1, vpar);
     vpar = vpar_join(vpar,1, make_CF(kfunc(6)));
     vpar = vpar_join(vpar,1, make_CF(kfunc(2)));
     vpar = vpar_join(vpar,1, make_CF(kfunc(4)));
     ,
     n%7==0,
     \\    L        figure 4(e) N == 0 mod 7
     \\    |
     \\    m -- CF
     \\    |
     \\    L
     my(k = floor((n-7)/7));
     vpar = [0];
     vpar = vpar_join(vpar,1, L_vpar);
     vpar = vpar_join(vpar,1, L_vpar);
     vpar = vpar_join(vpar,1, make_CF(k));
     ,
     error("oops, n=",n," n%7 = ",n%7));

  vpar;
}


\\-----------------------------------------------------------------------------
\\ Print some sample counts.

{
  print("Tree greatest number of maximum matchings per Heuberger and Wagner");
  print("  Size   Maximum Matchings   Match Number");
  print("             (A333347)        (A333348)");
  for(n=0,35,
     my(vpar=make(n),
        matchnum_and_count=vpar_matchnum_and_count(vpar));
     printf("  n=%2d       %7d             %2d\n",
            n, matchnum_and_count[2], matchnum_and_count[1]);
     if(n==6 || n==34,
        my(vpar=make(n+.5),
           matchnum_and_count=vpar_matchnum_and_count(vpar));
        printf("  n=%2d(b)      %5d             %2d\n",
               n, matchnum_and_count[2], matchnum_and_count[1]));
  );
}


\\-----------------------------------------------------------------------------
\\ To verify that the tree made really is the most, do a search.  Matchings
\\ don't depend on root or labelling, so the search is over free trees.
\\
\\ If n=6 or n=34 was tried here then search() would return the two trees of
\\ equal most maximum matchings.  The check would have to allow for that.
\\ But n=34 is beyond what the speed of gp code can reach.  (Some C code can
\\ just get there if really want brute force that far.)
\\
\\ Heuberger and Wagner show various necessary conditions leading up to
\\ their final unique tree.  Some of those might reduce a brute-force search
\\ space if desired.
\\
search(n) =
{
  my(vpar=vpar_free_first(n),
     max_list=List([]),
     max_count=-1);
  until((vpar=vpar_free_next(vpar)) == 'none,
        my(count=vpar_matchnum_and_count(vpar)[2]);
        if(count==max_count,
           listput(max_list,vpar);
           ,
           count>max_count,
           max_list=List([vpar]);
           max_count=count));
  Vec(max_list);
}

print();
{
  print("Check make() against a brute force search ...");
  for(n=7,14,
     my(vpar=make(n),
        vec=search(n));
     for(i=1,#vec,
        vpar_is_free_isomorphic(vec[i],vpar)
        || error("oops, expected found tree to be isomorphic to make()");
        );
     print("  n="n" search() vs make(), good"));
}

print();


\\-----------------------------------------------------------------------------
\\ Heuberger and Wagner in their proof consider vertices of type A and B.
\\ Type B vertices are matched in every maximum matching in the tree.
\\ Type A vertices are not.
\\
\\ The optimal trees have vertex types A,B alternating, so every edge is
\\ from some A to some B.  The leaves are all type A.  The optimal trees are
\\ thus of the "bipartite leaves" or "even distance leaves" type which is
\\ where any pair of leaves are an even distance apart, or equivalently the
\\ tree considered as a bipartite graph has all leaves in the same bipartite
\\ half.
\\
\\ For a given type A vertex, there is one or more maximum matching where A
\\ is unmatched.  For example in the fork F,
\\
\\     A1
\\       \                  A = not always matched
\\        B --- A3 -...     B = always matched
\\       /
\\     A2
\\
\\ A maximum matching is B,A1 with A2 unmatched.  Or vice versa B,A2 with A1
\\ unmatched.  So there exists some maximum matching with A1 unmatched, and
\\ likewise some maximum matching with A2 unmatched.
\\
\\ In any "bipartite leaves" tree, type B vertices must be matched.  This
\\ follows from a general rule for matchings.  If a given vertex (B) has one
\\ non-leaf neighbour (A3) and one or more leaf neighbours (A1,A2) then a
\\ maximum matching is attained by pairing B with a leaf (B to A1).
\\
\\ If B was unmatched then that would not be a maximal matching.  If B is
\\ paired with a leaf then the remainder of the tree still includes A3 and
\\ that gives matchings which are either the same or bigger than if A3 is
\\ taken with B already.
\\
\\
\\ This pairing of all B leads to a straightforward calculation for the
\\ match number in the optimal trees.  Each chain part C contains 2 B's
\\ among 7 vertices, hence (2/7)*n, and then a fixed number of additional B
\\ according to the vertices between chains as n mod 7, after initial
\\ exceptions.

matchnum_general(n) = floor((2*n+2)/7);
matchnum(n) = if(n==2,1, n==6,1, n==13,3, n==20,5, matchnum_general(n));

{
  print1("matchnum  ");
  for(n=0,20,
     print1(matchnum(n),","));
  print("...   (A333348)\n");
}
{
  for(n=7,100,
     my(vpar=make(n));
     vpar_matchnum(vpar) == matchnum(n)  || error());
}


\\-----------------------------------------------------------------------------
\\ Heuberger and Wagner calculate counts by choosing a root vertex and
\\ keeping two counts there.  In the code here these are column vector,
\\
\\     [ any,            count maximum matchings where v matched or not
\\       without ]       count maximum matchings where v not matched
\\
\\     any = with + without
\\
\\ The current code in vpar_matchnum_and_count() does essentially the same,
\\ but keeps counts "with, without" instead of any=with+without.
\\
\\ At a type A root, the size of maximum matchings "with" and "without" are
\\ the same, and "any" is the total number of both.
\\
\\ At a type B root, the size of maximum matchings "without" is 1 smaller
\\ than "with".  "Any" is still the total number of both.


\\ counts_of_vpar_A() returns a column vector of two counts [any, without]~
\\ at the root of vpar.  The root must be a type A vertex.
\\ "any" is the count as from vpar_matchnum_and_count().
\\ "without" is the count of maximum matchings when the root vertex is
\\ deleted so it cannot be in a matching.
\\
counts_of_vpar_A(vpar) =
{
  my(sans_root=if(#vpar==0,[],vpar_delete_vertex(vpar,vpar_root(vpar))));
  [vpar_matchnum_and_count(vpar)[2],
   vpar_matchnum_and_count(sans_root)[2] ]~;
}


\\ For the singleton L, the empty matching is the only matching possible.
\\ This matchings has the L vertex unmatched the matching is a "without".
counts_L = {[ 1,       \\ 1 any
              1 ]~};   \\ 1 without

\\     root
\\      |         F subtree
\\      B
\\     / \
\\    A   A
counts_F = {[ 3,      \\ any
              2 ]~};  \\ without root, only the 2 others
counts_F == counts_of_vpar_A([0,1,2,2])    || error();

\\ Heuberger and Wagner show in lemma 6. shows that when a C is added above
\\ an A, the new two counts are
\\
\\                                      new_A_pair = M * old_A_pair
\\     *       new A      \          
\\      \        |         |  new C above old A
\\       *---*---*---*     |
\\      /        |        /
\\     *         |
\\             old A                    old_A_pair = [any,without]~
\\
\\ where the matrix M multiplied is
\\
M = {[8, 3;
      5, 3]};

\\ counts in a chain of k many C above an L or above an F respectively
counts_CL(k) = M^k * counts_L;
counts_CF(k) = M^k * counts_F;
{
  for(k=0,20,
     counts_CL(k) == counts_of_vpar_A(make_CL(k)) || error();
     counts_CF(k) == counts_of_vpar_A(make_CF(k)) || error());
}

\\ Heuberger and Wagner note the eigenvalues of M are the following l,h.
\\ sqrt85 is GP type t_QUAD so l,h are represented exactly.
sqrt85 = quadgen(4*85);
l = (11 + sqrt85)/2;      \\ their lambda
h = (11 - sqrt85)/2;      \\ their overline(lambda), quadratic conjugate
{
  sqrt85^2 == 85 || error();
  my(e=mateigen(M,1)[1]);
  abs(e[1] - h) < 1e-10 || error();
  abs(e[2] - l) < 1e-10 || error();
}

\\ l,h are the roots of the characteristic polynomial of M.
\\
P(x) = x^2 - 11*x + 9;
print("M = "M" charpoly P(x) = ",P(x));
print("Eigenvalues of M (roots of P)");
printf("  l = %11.8f...    (A333345)\n", l);
printf("  h = %11.8f...\n", h);

charpoly(M) == P('x)   || error();
P(l) == 0   || error();
P(h) == 0   || error();

\\ Heuberger and Wagner work also in terms of a G(k) which is the following
\\ powers.  G(k) is the basic linear recurrence arising from multiplying
\\ matrix M^k.
G(k) =
{
  k >= 0 || error("G(k) is intended for k>=0, though negatives would work");
  (l^k - h^k) / (l-h);
}

\\ G is roughly 1 new decimal digit for each k since l = 10.109772... is
\\ close to 10.
{
  print1("G(k) = ");
  for(k=0,6, print1(G(k),", "));
  print("...    (A190872)");
}

\\ The denominator l-h used in G is as follows, and some other relations
\\ between l and h too.  It's straightforward to work either in terms of l
\\ or h or sqrt85.  Believe l is most convenient since the various counts
\\ grow as a power of l.
\\
h == 9/l                  || error();
l-h == sqrt85             || error();
2*l-11 == sqrt85          || error();
l-h == 2*l-11             || error();
1/(l-h) == sqrt85 / 85    || error();
1/(l-h) == (2*l-11)/85    || error();

\\ The coefficients in the characteristic polynomial of M are the recurrence
\\ coefficients for G(k), in the usual way.
\\
vector(50,k,k--; G(k+2) - 11*G(k+1) + 9*G(k)) == \
vector(50,k,k--; 0)                     || error();
vector(50,k,k--; G(k+2)) == \
vector(50,k,k--; 11*G(k+1) - 9*G(k))  || error();

\\ The generating function for G(k) is given by Philippe Deleham in OEIS
\\ A190872 and follows just from the recurrence.  A gf numerator x/(...) is
\\ normal for a recurrence starting 0,1.  An initial G(0)=0 arises in the CL
\\ and CF below so the initial 0 suits.
\\
gG(x) = x/(1 - 11*x + 9*x^2);
gG(x) + O(x^200) == Ser(vector(200,k,k--; G(k)))  || error();

polrecip(denominator(gG(x))) == P(x)  || error();

\\ Some polmod can calculate G too if preferred, instead of powering
\\ t_QUAD's l and h.
vector(50,k,k--; G(k)) == \
vector(50,k,k--; polcoeff(lift(Mod(x, (x^2-11*x+9))^k), \
                          1))   || error();
vector(50,k,k--; G(k)) == \
vector(50,k,k--; polcoeff(lift(Mod(1/x, (1-11*x+9*x^2))^(k-1)), \
                          0))   || error();

\\ The CL and CF chain matchings counts are, in terms of G, and per lemma
\\ 6.2 again,
vector(50,k,k--; counts_CL(k)) == \
vector(50,k,k--; [ G(k+1),        \
                   G(k+1) - 3*G(k) ]~)     || error();

vector(50,k,k--; counts_CF(k)) ==      \
vector(50,k,k--; [ 3*G(k+1) - 3*G(k),  \
                   2*G(k+1) - G(k)   ]~)   || error();


\\-----------------------------------------------------------------------------
\\ At a given vertex p, a new vertex v and subtree below v are to be
\\ attached under p.  p has a pair of counts [any,without]~ and v has a pair
\\ of counts [any,without]~.  The new counts at p with v added underneath
\\ are calculated as follows.

\\ For p vertex type A, and v vertex type B underneath p.
\\
\\     new A without = without A * with B
\\
\\ This new "without" uses "with B" since every B must be matched to be a
\\ maximum matching in the final tree, and a child "without B" will have no
\\ further opportunity to be matched.
\\
\\     new A with = with A * with B
\\                + without A + without B
\\
\\ Again with A must be with B since every B must be matched.  Without A and
\\ without B are where a new pairing A,B is made.  The total is then,
\\
\\     new A any = with + without
\\               =   with A * with B
\\                 + without A + without B
\\                 + without A * with B
\\               = any A * with B
\\                 + without A + without B
\\
\\ Here "any A * with B" follows arithmetically, or in terms of matchings is
\\ simply that as long as B is matched then the final is maximum.
\\
\\ p type A, v type B
counts_under_A(pcounts,vcounts) =
{
  my(with_v = vcounts[1] - vcounts[2]);
  [pcounts[1] * with_v          \\ with p * with v
   + pcounts[2] * vcounts[2]    \\ without p * without v, new pair
   ,
   pcounts[2] * with_v          \\ without p * with v
  ]~;
}

\\ For p vertex type B, and v vertex type A underneath.
\\
\\     new without B = without B * any A
\\
\\ All "any A" are the same size, so "without B" one smaller than "with B"
\\ is maintained.
\\
\\     new with B = with B * any A
\\                + without B + without A
\\
\\ Again "any A" suits since all its matchings are the same size.  Then
\\ without B and without A are where a new pair B,A is made.  Then total,
\\
\\     new any B = with + without
\\               =   with B * any A
\\                 + without B + without A
\\                 + without B * any A
\\               = any B * with A
\\                 + without B + without A
\\
\\ Here "any B * any A follows" arithmetically, or in terms of matchings is
\\ simply that "any A" matchings are all the same size so all maintain the
\\ without-one-smaller in any B.
\\
\\ p vertex type A, and v vertex type B
counts_under_B(pcounts,vcounts) =
{
  [pcounts[1] * vcounts[1]      \\ with p * any v
   + pcounts[2] * vcounts[2]    \\ without p * without v, new pair
   ,
   pcounts[2] * vcounts[1]      \\ without p * any v
  ]~;
}

\\ For interest, the current vpar_matchnum_and_count() code uses a
\\ "without_is_smaller" to record when size withs = size withouts, versus
\\ size withs = size withouts +1, and take consequent action.  That suits
\\ general trees but the bipartite leaves tree like the optimal trees here
\\ can explicitly apply respective counts_under_A() or counts_under_B().


\\ Path-2 is not actually a bipartite leaves tree, but its root vertex can
\\ be reckoned as a B ready to add more underneath and that comes out right.
\\
counts_path2B = counts_under_B(counts_L,counts_L);

\\    B  root
\\   / \             star-3 with root as middle type B
\\  A   A
counts_star3B = counts_under_B(counts_path2B,counts_L);

counts_F == counts_under_A(counts_L,counts_star3B)  || error();

{
  \\ The following shows how matrix M arises from putting C over the top of
  \\ some counts x,y below.  counts_under_A() and counts_under_B() can act
  \\ on the counts below as symbols 'x and 'y.  The result from that is the
  \\ new top A is M*[x,y]~.
  \\
  \\     *         A = final c
  \\      \        |
  \\       *---F---B---L
  \\      /        |
  \\     *         xy
  \\
  my(c=counts_L);                 \\ at B
  c=counts_under_B(c,['x,'y]);    \\ whatever x,y below
  c=counts_under_B(c,counts_F);   \\ F to its left
  c=counts_under_B(c,counts_L);   \\ L to its right
  c=counts_under_A(counts_L,c);   \\ at top A the B is below
  c == M*['x,'y]~    || error();
}


\\-----------------------------------------------------------------------------
\\ Tree n == 1 mod 7 is CL
\\
\\ The number of maximum matchings is G(k+1) which is the "any" component of
\\ counts_CL().

\\ most maximum matchings for n = 7*k+1
count_7k_plus_1(k) = counts_CL(k)[1];

vector(50,k,k--; my(n=7*k+1); vpar_matchnum_and_count(make(n))[2]) == \
vector(50,k,k--; count_7k_plus_1(k))          || error();

gcount_7k_plus_1(x) = 1/(1 - 11*x + 9*x^2);

gcount_7k_plus_1(x) == 1/x * gG(x)            || error();
gcount_7k_plus_1(x) + O(x^200) == \
  Ser(vector(200,k,k--; count_7k_plus_1(k)))  || error();

{
  print();
  my(g=gcount_7k_plus_1(x),
     deg=poldegree(denominator(g)));
  print("count n = 1 mod 7, recurrence order ",deg);
  deg == 2  || error();
}


\\-----------------------------------------------------------------------------
\\ Tree n == 4 mod 7 is CF
\\
\\ The number of maximum matchings is 3*(G(k+1)-G(k)) which is the "any"
\\ component of counts_CF().

\\ most maximum matchings for n = 7*k+4
count_7k_plus_4(k) = counts_CF(k)[1];

vector(50,k,k--; my(n=7*k+4); vpar_matchnum_and_count(make(n))[2]) == \
vector(50,k,k--; count_7k_plus_4(k))          || error();

gcount_7k_plus_4(x) = (3 - 3*x)/(1 - 11*x + 9*x^2);

gcount_7k_plus_4(x) == 3/x*gG(x) - 3*gG(x)    || error();
gcount_7k_plus_4(x) + O(x^200) == \
  Ser(vector(200,k,k--; count_7k_plus_4(k)))  || error();

{
  my(g=gcount_7k_plus_4(x),
     deg=poldegree(denominator(g)));
  print("count n = 4 mod 7, recurrence order ",deg);
  deg == 2  || error();
}


\\-----------------------------------------------------------------------------
\\ Tree n == 0 mod 7 is
\\
\\    L        figure 4(e)  n == 0 mod 7
\\    |
\\    m -- CF
\\    |
\\    L
\\
\\ This is two extra L above a CF chain.  That CF is terms in G(k) and the
\\ counts_under_B() add and multiply so as to give G(k) still but with
\\ different bigger coefficients than plain CF.

\\ most maximum matchings for n = 7*k + 7
count_7k_plus_7(k) =
{
  my(c=counts_L);
  c=counts_under_B(c,counts_CF(k));
  c=counts_under_B(c,counts_L);
  c=counts_under_B(c,counts_L);
  c[1] - c[2];
}
vector(50,k,k--; my(n=7*k+7); vpar_matchnum_and_count(make(n))[2]) == \
vector(50,k,k--; count_7k_plus_7(k))   || error();

\\ The extra vertex m and two Ls extend the CF counts to become, in terms of
\\ G(k),
vector(50,k,k--; count_7k_plus_7(k)) == \
vector(50,k,k--; 8*G(k+1) - 7*G(k))  || error();

gcount_7k_plus_7(x) = (8 - 7*x)/(1 - 11*x + 9*x^2);

gcount_7k_plus_7(x) == 8/x*gG(x) - 7*gG(x)  || error();
gcount_7k_plus_7(x) + O(x^200) == \
  Ser(vector(100,k,k--; count_7k_plus_7(k)))  || error();

{
  my(g=gcount_7k_plus_7(x),
     deg=poldegree(denominator(g)));
  print("count n = 0 mod 7, recurrence order ",deg);
  deg == 2  || error();
}


\\------------------------------------------
\\ Tree n == 5 mod 7 is
\\
\\         C2L         figure 4(c)  N == 5 mod 7
\\          |
\\   C1L -- m -- L
\\          |
\\         C0L
\\
\\ For n = 7*k + 5, this tree has CL branches of three lengths
\\ floor((k+j)/3) for j=0,1,2.
\\
\\ C0L is floor(k/3) = k\3 (writing k\3 for floor(k/3) in the GP way), which
\\ is terms in l^(k\3) and h^(k\3) with certain coefficients.  C1L and C2L
\\ may be one bigger power, which means certain bigger coefficients.
\\
\\ When counts_under_B() joins the three CL at m, it multiplies terms so as
\\ to result in four powers
\\
\\     (l^(k\3))^3                        biggest
\\     (l^(k\3))^2 * (h^(k\3))
\\     (l^(k\3))   * (h^(k\3))^2
\\                   (h^(k\3))^3          smallest
\\
\\ Again these are with various coefficients.  The biggest term is
\\
\\     (l^(k\3))^3 = l^k * (some coefficient according to k mod 3)
\\
\\ A set of four power combinations like this is a linear recurrence of
\\ order 4 and the floor((k+j)/3) branch lengths are a pattern of 3 for
\\ when one or two branches are longer and hence bigger coefficients.
\\ So count_7k_plus_5() is a recurrence of order at most 3*4 = 12.
\\
\\ To find recurrence coefficients, or generating function, it's convenient
\\ to get some matchings counts from code like count_7k_plus_5(), or generic
\\ counting like vpar_matchnum_and_count() on make(n), and give the values
\\ to a generic recurrence guesser (like my recurrence-guess.gp).
\\
\\ Knowing upper bound 12 for the recurrence order means 2*12 values are
\\ enough to determine the correct coefficients.  In the case of
\\ count_7k_plus_5(), the actual recurrence order is 8.
\\
\\ \\ from my recurrence-guess.gp
\\ recurrence_guess(vector(200,k,k--; count_7k_plus_5(k)))


\\ most maximum matchings for n = 7*k + 5
count_7k_plus_5(k) =
{
  my(kfunc = (j)->floor((k+j)/3),
     c=counts_L);
  c=counts_under_B(c,counts_CL(kfunc(0)));
  c=counts_under_B(c,counts_CL(kfunc(1)));
  c=counts_under_B(c,counts_CL(kfunc(2)));
  c=counts_under_B(c,counts_L);
  c[1] - c[2];
}

vector(100,k,k--; my(n=7*k+5); vpar_matchnum_and_count(make(n))[2]) == \
vector(100,k,k--; count_7k_plus_5(k))  || error();

\\ Generating function, expressed in partial fractions.
\\ Growth is per the /(1 - 11*x + 9*x^2) term which is l = 10.109...
\\ That root is bigger in magnitude than roots from /(1 - 99*x^3 + 729*x^6).
gcount_7k_plus_5(x) =
{
  (349/85 - 63/17*x)/(1 - 11*x + 9*x^2)
  + (-9/85 - 39/85*x - 93/85*x^2 + 729/85*x^3 + 2916/85*x^4 + 972/17*x^5)
    / (1 - 99*x^3 + 729*x^6);
}
gcount_7k_plus_5(x) + O(x^200) == \
Ser(vector(200,k,k--; count_7k_plus_5(k)))  || error();

{
  my(g=gcount_7k_plus_5(x),
     deg=poldegree(denominator(g)));
  print("count n = 5 mod 7, recurrence order ",deg);
  deg == 8  || error();
}

\\ The smaller term in gcount_7k_plus_5() is a spread G and power of 9,
\\ essentially since h = 9/l.
subst(1/x*gG(x),x, 9*x^3) == 1/(1 - 99*x^3 + 729*x^6)         || error();
\\
1/(1 - 99*x^3 + 729*x^6) + O(x^200) == \
  Ser(vector(200,n,n--; if(n%3==0, my(k=n/3); 9^k*G(k+1))))   || error();

\\ This allows counts to be written as a function of G(k) and G(k\3).
\\ The coeffiencients from the generating function numerators are partly
\\ worked into the powers of 9, then a common denominator of 85.
count_7k_plus_5_using_G(k) =
{
  my(t=k\3);
  (349*G(k+1) - 315*G(k)
   + if(k%3==0,  - 9^(t+1)*G(t+1) +     9^(t+2)*G(t),
        k%3==1,  -39 * 9^t*G(t+1) +  4* 9^(t+2)*G(t),
        k%3==2,  -93 * 9^t*G(t+1) + 60* 9^(t+1)*G(t))
  ) / 85;
}
vector(500,k,k--; count_7k_plus_5_using_G(k)) == \
vector(500,k,k--; count_7k_plus_5(k))                         || error();


\\------------------------------------------
\\ Trees n == 3 mod 7 are
\\
\\         C2F        figure 4(b)  N == 3 mod 7
\\          |
\\   C1F -- m -- C3F
\\          |
\\         C0F
\\
\\ This is like 5 mod 7 above, but 4 branches of CF type.
\\ Notice this case begins only for n>=17, hence 7*k+17.

\\ most maximum matchings for n = 7*k + 17
count_7k_plus_17(k) =
{
  my(kfunc = (j)->floor((k+j)/4),
     c=counts_L);
  c=counts_under_B(c,counts_CF(kfunc(0)));
  c=counts_under_B(c,counts_CF(kfunc(1)));
  c=counts_under_B(c,counts_CF(kfunc(2)));
  c=counts_under_B(c,counts_CF(kfunc(3)));
  c[1] - c[2];
}

vector(50,k,k--; my(n=7*k+17); vpar_matchnum_and_count(make(n))[2]) == \
vector(50,k,k--; count_7k_plus_17(k))  || error();

\\ Each CF arm is a length like k\4, with the floor((k+j)/4) making some +1
\\ bigger according to j.  Similar to 5 mod 7 above, their counts are
\\ multiplied at the middle m so giving terms in powers, with various
\\ coefficients,
\\
\\     (l^(k\4))^4
\\     (l^(k\4))^3 * (h^(k\4))
\\     (l^(k\4))^2 * (h^(k\4))^2
\\     (l^(k\4))   * (h^(k\4))^3
\\                   (h^(k/4))^4
\\
\\ The biggest is l^k, the others are l^(3/4)*h, l^(2/4)*h^(2/4), etc.
\\
\\ Branch lengths are a pattern of 4 according to floor (k-j)/4, and then 5
\\ powers l and h, so a recurrence of order at most 4*5 = 20.  The actual
\\ recurrence length is 14.

\\ Generating function, expressed in partial fractions.
\\ Growth is per the /(1 - 11*x + 9*x^2) term which is l = 10.109...
\\ This is bigger than the magnitudes of the roots from other terms.
gcount_7k_plus_17(x) =
{
  -69/170 / (1 - 3*x)
  + 3/578 / (1 + 3*x)
  + 3/17  / (1 + 9*x^2)
  + (313848/1445 - 279396/1445*x) / (1 - 11*x + 9*x^2)
  + (-117/85 - 9*x + 324/85*x^2 + 243/10*x^3)          / (1 - 33*x^2 + 81*x^4)
  + (117/289 + 414/289*x + 324/289*x^2 + 2187/578*x^3) / (1 + 33*x^2 + 81*x^4);
}
gcount_7k_plus_17(x) + O(x^200) == \
Ser(vector(200,k,k--; count_7k_plus_17(k)))  || error();

{
  my(g=gcount_7k_plus_17(x),
     deg=poldegree(denominator(g)));
  print("count n = 3 mod 7, recurrence order ",deg);
  deg == 14  || error();
}


\\------------------------------------------
\\ Trees n == 2 mod 7 are
\\
\\       C3L        C4L        figure 4(a)     N = 2 mod 7
\\        |          |
\\   L -- m -- C0 -- r -- L
\\        |          |
\\       C1L        C2L
\\
\\ This case is like the above 5 mod 7 or 3 mod 7 in respect of the branches
\\ L, C1L, C3L meeting at m.  Then the middle C0 is applied as a chain
\\ "above" m.  C1L,C3L counts are terms in l^(k\5) and h^(k\5) then at m
\\ they multiply, then C0 across is M^(k\5) which is further factors l^(k\5)
\\ and h^(k\5), and finally again factors l^(k\5) and h^(k\5) for the C2L
\\ and C4L.  The result is terms in 6 power combinations
\\
\\     (l^(k\5))^5
\\     (l^(k\5))^4 *  h^(k\5)
\\     (l^(k\5))^3 * (h^(k\5))^2
\\     (l^(k\5))^2 * (h^(k\5))^3
\\      l^(k\5)    * (h^(k\5))^4
\\                   (h^(k\5))^5
\\
\\ So the recurrence for count_7k_plus_2_ge_37() is order at most 6*5 = 30.
\\ The actual order is 22.

\\ most maximum matchings for n = 7*k + 2 and n>=37, so k>=5
count_7k_plus_2_ge_37(k) =
{
  k>=5 || error("oops, count_7k_plus_2_ge_37() is for k>=5");
  my(kfunc = (j)->floor((k+j)/5),
     c=counts_L);  \\ at m
  c=counts_under_B(c,counts_CL(kfunc(1)));
  c=counts_under_B(c,counts_CL(kfunc(3)));
  c=counts_under_B(c,counts_L);
  c=counts_under_A(counts_L,c);    \\ type A right of m
  c = M^(kfunc(0)-1) * c;          \\ C0 chains across
  c=counts_under_B(counts_L,c);    \\ r
  c=counts_under_B(c,counts_CL(kfunc(2)));
  c=counts_under_B(c,counts_CL(kfunc(4)));
  c=counts_under_B(c,counts_L);
  c[1] - c[2];
}
vector(100,k,k+=4; my(n=7*k+2); vpar_matchnum_and_count(make(n))[2]) == \
vector(100,k,k+=4; count_7k_plus_2_ge_37(k))    || error();

\\ recurrence_guess(vector(200,k,k--; if(k>=5, count_7k_plus_2_ge_37(k))))

gcount_7k_plus_2_ge_37(x) =
{
  -13/9 - 134/9*x - 1375/9*x^2 - 14036/9*x^3 - 142417/9*x^4
  + (2191/1445 - 9729/7225*x)/(1 - 11*x + 9*x^2)
  + (4/2601 - 1672/65025*x - 21587/65025*x^2 - 144379/65025*x^3
     - 514901/65025*x^4 - 107892/1445*x^5 - 1708776/7225*x^6
     - 2601801/7225*x^7 - 3202497/7225*x^8 - 3445983/7225*x^9)
    / (1 - 9306*x^5 + 59049*x^10)
  + (-106/1445 - 3018/7225*x - 13723/7225*x^2 - 38536/7225*x^3
     - 80154/7225*x^4 + 55404/1445*x^5 + 1531629/7225*x^6 + 8282169/7225*x^7
     + 14610618/7225*x^8 + 25141752/7225*x^9)
    / (1 - 891*x^5 + 59049*x^10);
}
gcount_7k_plus_2_ge_37(x) + O(x^200) == \
Ser(vector(200,k,k--; if(k>=5, count_7k_plus_2_ge_37(k))))  || error();

{
  my(g=gcount_7k_plus_2_ge_37(x),
     deg=poldegree(denominator(g)));
  print("count n = 2 mod 7, recurrence order ",deg);
  deg == 22  || error();
}

\\------------------------------------------
\\ Trees n == 6 mod 7 are
\\
\\         C5F        C6F        figure 4(d)  N == 6 mod 7
\\          |          |
\\   C3F -- m -- C0 -- r -- C4F
\\          |          |
\\         C1F        C2F


\\ most maximum matchings for n = 7*k + 27
count_7k_plus_27(k) =
{
  my(kfunc = (j)->floor((k+j)/7),
     c=counts_L);  \\ at m
  c=counts_under_B(c,counts_CF(kfunc(1)));
  c=counts_under_B(c,counts_CF(kfunc(3)));
  c=counts_under_B(c,counts_CF(kfunc(5)));
  c=counts_under_A(counts_L,c);    \\ type A right of m
  c = M^kfunc(0) * c;              \\ C0 chains across
  c=counts_under_B(counts_L,c);    \\ r
  c=counts_under_B(c,counts_CF(kfunc(2)));
  c=counts_under_B(c,counts_CF(kfunc(4)));
  c=counts_under_B(c,counts_CF(kfunc(6)));
  c[1] - c[2];
}
vector(100,k,k--; my(n=7*k+27); vpar_matchnum_and_count(make(n))[2]) == \
vector(100,k,k--; count_7k_plus_27(k))  || error();

\\ recurrence_guess(vector(200,k,k--; count_7k_plus_27(k)))

gcount_7k_plus_27(x) =
{
    (723074688/122825 - 643701168/122825*x) / (1 - 11*x + 9*x^2)
  + (-5379291/122825 - 46954404/122825*x - 79918569/24565*x^2
     - 3280493439/122825*x^3 - 1499335272/7225*x^4 - 180078880293/122825*x^5
     - 1035976799187/122825*x^6 + 27766152/122825*x^7 + 237987153/122825*x^8
     + 402655131/24565*x^9 + 16510808988/122825*x^10
     + 128264879673/122825*x^11 + 906176523771/122825*x^12
     + 5213130099984/122825*x^13)
    / (1 - 950499*x^7 + 4782969*x^14)
  + (-248589/24565 - 9520983/122825*x - 65398428/122825*x^2
     - 456136596/122825*x^3 - 148878972/7225*x^4 - 3096215361/24565*x^5
     - 53541952479/122825*x^6 + 78712317/122825*x^7 + 574920747/122825*x^8
     + 3835586844/122825*x^9 + 26251059636/122825*x^10 + 5796958428/4913*x^11
     + 885083630481/122825*x^12 + 3060234442611/122825*x^13)
    / (1 - 83754*x^7 + 4782969*x^14)
  + (-137052/122825 - 951588/122825*x - 1276722/24565*x^2
     - 36256653/122825*x^3 - 11544444/7225*x^4 - 472431366/122825*x^5
     - 1070322174/122825*x^6 + 124357194/122825*x^7 + 814521906/122825*x^8
     + 205608618/4913*x^9 + 27207653436/122825*x^10 + 136477237446/122825*x^11
     + 322936500942/122825*x^12 + 721376950518/122825*x^13)
    / (1 - 8019*x^7 + 4782969*x^14);
}
gcount_7k_plus_27(x) + O(x^200) == \
Ser(vector(200,k,k--; count_7k_plus_27(k)))  || error();

{
  my(g=gcount_7k_plus_27(x),
     deg=poldegree(denominator(g)));
  print("count n = 6 mod 7, recurrence order ",deg);
  deg == 44  || error();
}


\\-----------------------------------------------------------------------------
\\ The above cases and inital exceptions can then be combined as follows.

gcount(x) =
{
    gcount_7k_plus_7(x^7)       * x^7
  + gcount_7k_plus_1(x^7)       * x^1
  + gcount_7k_plus_2_ge_37(x^7) * x^2
  + gcount_7k_plus_17(x^7)      * x^17
  + gcount_7k_plus_4(x^7)       * x^4
  + gcount_7k_plus_5(x^7)       * x^5
  + gcount_7k_plus_27(x^7)      * x^27

  + vpar_matchnum_and_count(make(0))[2] * x^0       \\ n = 0

  + vpar_matchnum_and_count(make( 2))[2] * x^2      \\ n = 2 mod 7
  + vpar_matchnum_and_count(make( 9))[2] * x^9
  + vpar_matchnum_and_count(make(16))[2] * x^16
  + vpar_matchnum_and_count(make(23))[2] * x^23
  + vpar_matchnum_and_count(make(30))[2] * x^30

  + vpar_matchnum_and_count(make( 3))[2] * x^3      \\ n = 3 mod 7
  + vpar_matchnum_and_count(make(10))[2] * x^10

  + vpar_matchnum_and_count(make( 6))[2] * x^6      \\ n = 6 mod 7
  + vpar_matchnum_and_count(make(13))[2] * x^13
  + vpar_matchnum_and_count(make(20))[2] * x^20;
}

{
  \\ The combined generating function for the counts has denominator degree
  \\ 574, which is the linear recurrence order.
  \\
  my(g=gcount(x),
    deg=poldegree(denominator(g)));

  print();
  print("count all n, recurrence order ",deg);

  my(initial = poldegree(numerator(g)) - deg);
  print("  gf numerator is ",initial," longer than denominator,",
        " so recurrence starts n=",initial+1);
  print();

  poldegree(numerator(g))   == 604  || error();
  poldegree(denominator(g)) == 574  || error();

  \\ The polynomial can be factored over complex numbers by factor(p*I)
  \\ instead, though doing so only breaks
  \\     x^14 + 9  ==  (x^7 + 3*I) * (x^7 - 3*I)
  \\ and factor() is noticeably slower this way than over reals.
  \\
  my(p=polrecip(denominator(g)),
     m=factor(p),
     len=matsize(m)[1],
     rvec=vector(len,i, vecmax(abs(polroots(m[i,1])))),
     seq=vecsort(rvec,,1+4));
  print("charpoly factors (over rationals)");
  for(s=1,#seq,
     my(i=seq[s]);
     printf("  %-30s biggest root %.6f", m[i,1], rvec[i]);
     if(m[i,1]==P(x^7),
        print1("   is P(x^7)"));
     print();
     m[i,2]==1 || error());
}


\\ gcount_by_partial_fractions() is partial fractions as printed by
\\ recurrence_guess_INTERNAL_print_gf(gcount(x)) which is an internal part
\\ of my recurrence-guess.gp.  lk_numerator is picked out for separate use.

lk_numerator = {
  7/9 - 5/9*x^7           \\ pairs of terms k and k+7
  + x
  + 2191/1445*x^2 - 9729/7225*x^9
  + 9212/4335*x^3 - 1640/867*x^10
  + 3*x^4 - 3*x^11
  + 349/85*x^5 - 63/17*x^12
  + 699776/122825*x^6 - 124592/24565*x^13;
}

gcount_by_partial_fractions(x) =
{
  \\ initial exceptions
  2/9 - 4/9*x^2 + 1/9*x^9 + 2*x^13 + 2/9*x^16 + 4*x^20 + 4/9*x^23 - 154/9*x^30

  \\ term in l^k
  + lk_numerator / (1 - 11*x^7 + 9*x^14)

  - 23/510*x^3/(1 - 3*x^7)
  + 1/1734*x^3/(1 + 3*x^7)
  - 1/51*x^3/(1 + 9*x^14)
  + (-4/85*x^3 - 3/10*x^10 + 3/17*x^17 + 9/10*x^24)
    / (1 - 33*x^14 + 81*x^28)
  + (-4/289*x^3 - 27/578*x^10 - 15/289*x^17 - 63/578*x^24)
    / (1 + 33*x^14 + 81*x^28)
  + (-9/85*x^5 - 39/85*x^12 - 93/85*x^19 + 729/85*x^26 + 2916/85*x^33
     + 972/17*x^40)
    / (1 - 99*x^21 + 729*x^42)
  + (4/2601*x^2 - 1672/65025*x^9 - 21587/65025*x^16 - 144379/65025*x^23
     - 514901/65025*x^30 - 107892/1445*x^37 - 1708776/7225*x^44
     - 2601801/7225*x^51 - 3202497/7225*x^58 - 3445983/7225*x^65)
    / (1 - 9306*x^35 + 59049*x^70)
  + (-106/1445*x^2 - 3018/7225*x^9 - 13723/7225*x^16 - 38536/7225*x^23
     - 80154/7225*x^30 + 55404/1445*x^37 + 1531629/7225*x^44
     + 8282169/7225*x^51 + 14610618/7225*x^58 + 25141752/7225*x^65)
    / (1 - 891*x^35 + 59049*x^70)
  + (-26817/122825*x^6 - 189459/122825*x^13 - 1089936/122825*x^20
     - 5379291/122825*x^27 - 46954404/122825*x^34 - 79918569/24565*x^41
     - 3280493439/122825*x^48 + 832059/122825*x^55 + 1709748/122825*x^62
     + 6278877/122825*x^69 + 27766152/122825*x^76 + 237987153/122825*x^83
     + 402655131/24565*x^90 + 16510808988/122825*x^97)
    / (1 - 950499*x^49 + 4782969*x^98)
  + (-1212/4913*x^6 - 185049/122825*x^13 - 639819/122825*x^20
     - 248589/24565*x^27 - 9520983/122825*x^34 - 65398428/122825*x^41
     - 456136596/122825*x^48 + 6803676/122825*x^55 + 17517141/122825*x^62
     + 45448047/122825*x^69 + 78712317/122825*x^76 + 574920747/122825*x^83
     + 3835586844/122825*x^90 + 26251059636/122825*x^97)
    / (1 - 83754*x^49 + 4782969*x^98)
  + (-28534/122825*x^6 - 67518/122825*x^13 - 150822/122825*x^20
     - 137052/122825*x^27 - 951588/122825*x^34 - 1276722/24565*x^41
     - 36256653/122825*x^48 + 32558598/122825*x^55 + 68995476/122825*x^62
     + 139119444/122825*x^69 + 124357194/122825*x^76 + 814521906/122825*x^83
     + 205608618/4913*x^90 + 27207653436/122825*x^97)
    / (1 - 8019*x^49 + 4782969*x^98);
}
gcount_by_partial_fractions(x) == gcount(x)  || error();


\\ The limit constants c_0 through c_6 given by Heuberger and Wagner are in
\\ the numerator of generating function term "l^k" which is "lk_numerator".

\\ Without the x^7 spread, 1/(1 - 11*x + 9*x^2) is terms in l^(k+1)/(l-h)
\\ and h^(k+1)/(l-h)
\\
1/(1 - 11*x + 9*x^2) + O(x^100) == \
Ser(vector(100,k,k--; (l^(k+1) - h^(k+1)) / (l-h)))   || error();

\\ So the two 0 mod 7 terms, ie. x^0 and x^7, of lk_numerator are Heuberger
\\ and Wagner's c_0
\\
c0 = (67*l - 71)/765;  \\ Heuberger and Wagner

c0 == (7/9*l - 5/9)/(l-h)   || error();    \\ lk_numerator

\\ Further c_1 etc constants are shifted down by denominator l^(1/7),
\\ l^(2/7), etc.  The lk_numerator coefficients are before that.
\\
c_coeffs_sans_shift = {         \\ Heuberger and Wagner table 1
 [ c0,
   (11*l - 18)/85,                  \\ c1
   (101047*l - 90171)/614125,       \\ c2
   (4996*l - 4448)/21675,           \\ c3
   (27*l - 21)/85,                  \\ c4
   (3209*l - 2817)/7225,            \\ c5
   (6451616*l - 5743408)/10440125   \\ c6
 ];
}
{
  for(i=1,7,
     my(c=c_coeffs_sans_shift[i],
        j=i-1);
     c == (polcoeff(lk_numerator,j)*l + polcoeff(lk_numerator,j+7))
          / (l-h)    || error());
}

\\-----------------------------------------------------------------------------
print("end");
