\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Usage: gp <terminal-Wiener-maximum.gp
\\
\\    Ivan Gutman, Boris Furtula, Miroslav Petrovic, "Terminal Wiener
\\    Index", Journal of Mathematical Chemistry, volume 46, 2009, pages
\\    522-531
\\
\\ define the terminal Wiener index as the total distance between all pairs
\\ of terminal vertices in a tree (or graph), where terminal means degree 1,
\\ so a leaf.  In vpar.gp this is vpar_Wiener_index() with flag=1.
\\
\\ They determine the maximum terminal Wiener index possible for trees of n
\\ vertices, or trees of n vertices among which are exactly k are leaves,
\\ and they construct the trees which attain these maxima.  The code here
\\ calculates these things.


default(recover,0);
default(strictargs,1);
read("../vpar.gp");


\\------------------------------------------------------------------------------
\\ n vertices among which k leaves is possible only for certain combinations
\\ of n,k.
\\
\\ The general case is 2 <= k <= n-1 which is extremes k=2 path, and k=n-1
\\ star.  At n=0,1,2 vertices, the sole tree for each is k=n leaves.

nk_is_good(n,k) = \
  n>=0 && if(n<=2, k==n, 2 <= k && k <= n-1);
nk_validate(n,k) = \
  nk_is_good(n,k) || error("oops, bad n=",n,",k=",k," vertices and leaves");

\\ Return a vector of the possible k values for the given n.
n_good_ks(n) = n>=0||error(); if(n<=2,[n], [2..n-1])

n_good_ks(0) == [0]     || error();
n_good_ks(5) == [2..4]  || error();
for(n=0,20, n_good_ks(n) == select(k->nk_is_good(n,k),[0..n]) || error());


\\------------------------------------------------------------------------------
\\ n,k Maximum Index

\\ max_TW_nk(n,k) is the maximum Wiener index possible for n vertices of
\\ which k are leaves, as per Gutman, Furtula, Petrovic theorem 4.
\\
max_TW_nk(n,k) =
{
  nk_validate(n,k);
  k*(k-1) + (n-1-k) * floor(k/2)*ceil(k/2);
}

{
  print("max_TW_nk() maximum terminal Wiener index n vertices k leaves");
  my(nlo=2,nhi=9);
  print1("       ");
  for(k=2,nhi-1,
     printf(" %3s", Str("k=",k)));
  print();
  for(n=nlo,nhi,
     printf("  n=%2d: ",n);
     for(k=2,if(n==2,n,n-1),
        printf(" %3s", if(nk_is_good(n,k), Str(max_TW_nk(n,k),","), "")));
     print());
  print();
}

\\------------------------------------------------------------------------------
\\ n,k Maximum Index Count

\\ countmax_TW_nk(n,k) is the count of how many free trees have the
\\ max_TW_nk() maximum index, as per Gutman, Furtula, Petrovic theorem 4.
\\
\\ The k=3 case is all trees of k=3 leaves, and otherwise a single tree when
\\ k even and various when k is odd >= 5.
\\
countmax_TW_nk(n,k) =
{
  nk_validate(n,k);
  if(n==1, 1,              \\ singleton
     k==3, (n-1)^2\/12,    \\ round((n-1)^2 / 12)
     k%2==0, 1,
     ceil((n-k)/2));
}

{
  print("countmax_TW_nk() number of trees attaining the maximum");
  my(nlo=2,nhi=9);
  print1("       ");
  for(k=nlo,nhi-1,
     printf(" %3s", Str("k=",k)));
  print();
  for(n=nlo,nhi,
     printf("  n=%2d: ",n);
     for(k=2,if(n==2,n,n-1),
        printf(" %3s",
               if(nk_is_good(n,k), Str(countmax_TW_nk(n,k),","), "")));
     print());
  print();
}

\\-----------------------------------------------------------------------------
\\ n,k Construction
\\
\\ The trees of n vertices and k leaves which attain the maximum can be
\\ constructed, again as per Gutman, Furtula, Petrovic theorem 4.
\\ k=3 leaves are all star-like and others are caterpillars (a spine which
\\ is a path and then leaves hanging from it).

\\ make_starlike(arms) return a vpar vector which is a star-like tree
\\ comprising a root vertex r=1 and under it the vector arms[] specifies
\\ lengths of paths to attach.  For example,
\\
\\        1 root      make_starlike([2,1,3])
\\      / | \         is vpar = [0, 1,2, 1, 1,5,6]
\\     2  4  5              v =  1  2 3  4  5 6 7
\\     |     |
\\     3     6
\\           |
\\           7
\\
make_starlike(arms) =
{
  my(vpar=vector(vecsum(arms)+1),v=1);
  for(i=1,#arms,
     if(arms[i],
        vpar[v++]=1;
        for(j=2,arms[i], vpar[v+1] = v; v++)));
  vpar;
}

\\ The vertex numbering from make_starlike() is each arm after the other.
\\ The first vertex of an arm has parent the root r=1 and then successively
\\ the parent of each vertex is its own predecessor.
{
  \\          vertex numbers 1  2 3 4  5 6 7 8  9 10...
  make_starlike([3,4,5]) == [0, 1,2,3, 1,5,6,7, 1, 9,10,11,12]  || error();
}

\\ For k even, a single tree attains the maximum terminal Wiener index.
\\ It has k/2 leaves at each end of a path,
\\
\\      *                   *
\\       \                 /          n=11 vertices
\\     *--s---s---s---s---s--*        k=6 leaves
\\       /                 \          spine "s"  n-k = 5 vertices
\\      *                   *         caterpillar [3,0,0,0,3]
\\
\\ vpar_make_caterpillar() is a convenient way to make a tree like this.
\\ It takes a vertor which says for each spine vertex "s" how many leaves
\\ are to attach to it.  cat_nk_even() is as easy as k/2 at the first and
\\ last of the spine of n-k vertices.
\\
cat_nk_even(n,k) =
{
  my(s=n-k,        \\ spine length
     ends = k/2);  \\ how many vertices at each end
  vector(s,i, if(i==1||i==s, ends));
}
cat_nk_even(11,6) == [3,0,0,0,3]  || error();

\\ For k odd >= 5, multiple trees attain the maximum.  They are a
\\ caterpillar like the even case, and have the even part k-1 vertices at
\\ ends of a spine, but then the odd vertex "X" can float anywhere along the
\\ spine.
\\
\\      *     X             *
\\       \    |            /          n=12 vertices
\\     *--A---B---C---s---s--*        k=7 leaves,
\\       /                 \          "X" attaches at A or B or C
\\      *                   *
\\
\\ "X" attached anywhere along the spine is the same maximum Wiener index.
\\ But only A or B or C result in distinct free trees.  The other "s" spine
\\ vertices are the same up to free isomorphism (being simply reversals).
\\
\\ cats_nk_odd() returns a vector of caterpillar specification vectors which
\\ have the X at each of its possible positions (ceil((n-k)/2) of them
\\ altogether, as per countmax_TW_nk(n,k)).
\\
cats_nk_odd(n,k) =
{
  my(s=n-k,            \\ spine length
     ends = (k-1)/2);  \\ how many leaves each end, excluding "X"
  vector(ceil(s/2),X,
         vector(s,i, if(i==1||i==s, ends) + (i==X)));
}
{
  cats_nk_odd(12,7) == [ [4,0,0,0,3],   \\ vector of vectors
                         [3,1,0,0,3],
                         [3,0,1,0,3] ]  || error();
}

\\ make_max_TW_nk_vpars(n,k) returns a vector of vpar vectors which are all
\\ the free trees which have n vertices, among which k leaves, and attain
\\ the maximum possible terminal Wiener index for any tree of such n,k
\\ vertices.  The number of trees, so the length of the return vector, is
\\ countmax_TW_nk(n,k).
\\
make_max_TW_nk_vpars(n,k) =
{
  nk_validate(n,k);
  if(n<=2 || k==n-1,
     \\ the only tree which exists for these n,k combinations
     [ vpar_make_star(n) ];
     ,
     k==3,
     \\ every tree of n,3 vertices
     apply(arms->make_starlike(Vec(arms)), partitions(n-1,,[3,3]));
     ,
     k%2==0,
     [ vpar_make_caterpillar(cat_nk_even(n,k)) ];
     ,
     apply(vpar_make_caterpillar, cats_nk_odd(n,k)));
}

{
  \\ Check that make_max_TW_nk_vpars() does indeed return countmax_TW_nk()
  \\ many trees, and that the terminal Wiener index of each is max_TW_nk().
  for(n=0,20,
     for(k=0,n,
        nk_is_good(n,k) || next;
        my(vpars = make_max_TW_nk_vpars(n,k));
        #vpars == countmax_TW_nk(n,k)  || error();
        my(want = max_TW_nk(n,k));
        for(i=1,#vpars,
           vpar_Wiener_index(vpars[i],1) == want  || error())));
}


\\-----------------------------------------------------------------------------
\\ Maximum Index by Search
\\
\\ A brute-force search can find the trees with maximum index, as a way to
\\ verify make_max_TW_nk_vpars.  The number of free trees grows rapidly so
\\ that this search is feasible only up to relatively small n.  Increase
\\ "nmax" if you have a fast computer and want to try more.

\\ canonical_vpars(vec) takes a vector of vpar vectors and returns them
\\ relabelled, re-rooted, and sorted, so a canonical form.
canonical_vpars(vec) = vecsort(apply(vpar_to_free,vec));

{
  my(nmax=12);
  for(n=3,nmax,
     my(seen_max_TW_vec = vector(n-1,i, -1),  \\ dummy lower than actual max
        seen_vpars_vec = vector(n-1,i, []),   \\ no vpars initially
        all_by_k = vector(n-1));              \\ k<=n-1
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(tw=vpar_Wiener_index(vpar,1),
              k=vpar_num_leaves(vpar));
           all_by_k[k]++;

           if(tw==seen_max_TW_vec[k],
              \\ equal maximum
              seen_vpars_vec[k] = concat(seen_vpars_vec[k], [vpar]);
              ,
              tw > seen_max_TW_vec[k],
              \\ new greater maximum
              seen_max_TW_vec[k] = tw;
              seen_vpars_vec[k] = [vpar]));

     \\ each k from 2..n-1 does occur, and k=1 does not
     vecmin(all_by_k[2..n-1]) == 1           || error();
     all_by_k[1] == 0                        || error();
     vecsum(all_by_k) == vpar_count_free(n)  || error();

     for(k=2,n-1,
        seen_max_TW_vec[k] == max_TW_nk(n,k)        || error();
        #seen_vpars_vec[k] == countmax_TW_nk(n,k)   || error();
           canonical_vpars(seen_vpars_vec[k])
        == canonical_vpars(make_max_TW_nk_vpars(n,k))  || error();
        );
     );
}

\\-----------------------------------------------------------------------------
\\ Maximum n Vertices
\\
\\ The maximum terminal Wiener index for any tree of n vertices is the
\\ maximum over its various possible k leaves.  The result is by certain
\\ cases according as n mod 3 once past the stars for n<=9

\\ max_TW(n) is the maximum Wiener index possible for n vertices, as per
\\ Gutman, Furtula, Petrovic theorem 5.
max_TW(n) =
{
  n>=0 || error();
  my(s=n\3);
  if(n<=1, 0,
     n==2, 1,
     n<=9, (n-1)*(n-2),
     n%3==0, s^3 + 3*s^2 + s - 1,
     n%3==1, s^3 + 4*s^2 + 3*s,
     n%3==2, s^3 + 5*s^2 + 6*s + 2);
}

{
  print("max_TW()");
  print1("  = ");
  for(n=0,13,
     print1(max_TW(n),","));
  print();
  print();
}

\\ The conditionals in max_TW() can be written a few different ways, since
\\ for instance n=10 has the same answer from (n-1)*(n-2) as from the n%3==1
\\ case.  But this is only at initial special cases, since the general forms
\\ are the only way for n>=11.
\\
my(n=10,s=n\3); (n-1)*(n-2) == s^3 + 4*s^2 + 3*s  || error();

\\ max_TW_nk_row(n) returns a vector which is all of the max_TW_nk(n,k)
\\ values for those k possible for the given n, so a "row" of max_TW_nk()
\\ reckoned as an n,k table.
\\
max_TW_nk_row(n) = apply(k->max_TW_nk(n,k), n_good_ks(n));
\\
\\ max_TW() is then the maximum over these max_TW_nk_row() values
\\
for(n=0,100, max_TW(n) == vecmax(max_TW_nk_row(n))   || error());

\\ In general, values in the max_TW_nk_row() have a maximum within the row
\\ and drops to smaller values each side of that.  The max_TW_nk() formula
\\ is a cubic in k and can be plotted with something like the following.
\\ This is left disabled since the a little big, and you might prefer a
\\ graphical "ploth" anyway.  Enable by changing to "if(1,".
\\
if(0, my(n=100); plot(k=2,99, max_TW_nk(n,floor(k))))

\\ The table printout for small n doesn't go far enough to see values
\\ dropping at the largest k.  The following is a portion of bigger n,k.
{
  print("bigger max_TW_nk() table and \"*\" where row maximum");
  my(nlo=18,nhi=23, klo=13,khi=nhi-1);
  print1("       ");
  for(k=klo,khi,
     printf(" %5s", Str("k=",k)));
  print();
  for(n=nlo,nhi,
     my(row_max=max_TW(n));
     printf("  n=%2d: ",n);
     for(k=klo,khi,
        nk_is_good(n,k) || next;
        my(m=max_TW_nk(n,k));
        printf(" %5s", if(nk_is_good(n,k),
                          Str(if(m==row_max,"*"," "), m, ","),
                          "")));
     print());
  print();
}


\\-----------------------------------------------------------------------------
\\ Which k Attain Maximum Index

\\ Return a vector of the values of k number of leaves which attain
\\ max_TW(n), as per Gutman, Furtula, Petrovic theorem 5.
\\
max_TW_at_ks(n) =
{
  n>=0 || error();
  my(s=n\3);
  if(n<=2,   [ n            ],  \\ sole tree of n vertices
     n<=9,   [ n-1          ],  \\ star
     n%3==0, [ 2*s+2        ],
     n%3==1, [ 2*s+2, 2*s+3 ],  \\ two k where equal maximum
     n%3==2, [        2*s+3 ]);
}
{
  \\ check these k's against max_TW_nk() and max_TW()
  for(n=0,50,
     my(want = max_TW(n));
     max_TW_at_ks(n)
     == select(k->max_TW_nk(n,k)==want, n_good_ks(n))   || error(n));
}

\\ The cases in max_TW_at_ks() are written in the way of theorem 5, but it
\\ can be noticed n=8 and n=9 have the same result from the n mod 3 general
\\ cases as they do from the "n-1" star, giving little options in the
\\ implementation if desired.

my(n=8,s=n\3);  (n%3==2 && n-1==2*s+3)  || error();
my(n=9,s=n\3);  (n%3==0 && n-1==2*s+2)  || error();


\\-----------------------------------------------------------------------------
\\ Count Trees of Maximum n
\\
\\ The number of trees attaining max_TW() is the n,k counts at the relevant
\\ max_TW_at_ks() numbers of leaves.  The result is initial special cases
\\ then according as n mod 3.

\\ Return the number of free trees of n vertices which attain the max_TW()
\\ maximum terminal Wiener index, as per Gutman, Furtula, Petrovic
\\ theorem 5.
countmax_TW(n) =
{
  my(s=n\3);
  if(n<=9,   1,
     n%3==0, 1,
     n%3==1, ceil(s/2),
     n%3==2, ceil((s-1)/2));
}

{
  print("countmax_TW() (which for n>=4 is OEIS A133188 up to \"offset\")");
  print1("  = ");
  for(n=0,30,
     print1(countmax_TW(n),","));
  print();
  print();
}
{
  \\ check countmax_TW() against countmax_TW_nk() of each of max_TW_at_ks()
  for(n=0,50,
     countmax_TW(n)
     == vecsum(apply(k->countmax_TW_nk(n,k), max_TW_at_ks(n)))  || error());
}

\\ countmax_TW() is OEIS sequence A133188(n+1) for n>=4 and with that
\\ sequence taken as "flat" instead of its rows interpretation.
{
  my(A133188_data=[0,0,1,0,1,1,1,1,1,1,2,1,1,2,2,1,3,2,1,3,3,1,4,3,1,4,4]);
  for(n=4,#A133188_data-1,
     countmax_TW(n) == A133188_data[n+1]   || error());
}


\\-----------------------------------------------------------------------------
\\ Maximum Vpars
\\
\\ The vpars of n vertices attaining the maximum index are as easy as
\\ collecting the make_max_TW_nk_vpars(n,k) for appropriate k's.

make_max_TW_vpars(n) = \
  concat(apply(k-> make_max_TW_nk_vpars(n,k), max_TW_at_ks(n)));

{
  \\ Check that make_max_TW_vpars(n) does indeed return countmax_TW(n) many
  \\ trees, and that the terminal Wiener index of each is max_TW(n).
  for(n=0,20,
     my(vpars = make_max_TW_vpars(n));
     #vpars == countmax_TW(n)  || error();
     my(want = max_TW(n));
     for(i=1,#vpars,
        vpar_Wiener_index(vpars[i],1) == want  || error()));
}

\\-----------------------------------------------------------------------------
print("end");