\\ Copyright 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1);
read("../vpar.gp");


\\ Usage: gp <transpose.gp
\\
\\ This is some sample forests put through vpar_transpose() to show its
\\ effect.


\\ The simplest illustration of transpose is a vertex v=1 with child v=2 and
\\ next sibling v=3.  The 2 and 3 swap positions,
\\
\\                                 binary tree natural correspondence
\\       1..3        1..2                  1           1
\\       |      =>   |                    / \    =>   / \
\\       2           3                   2   3       3   2               
\\
\\ If say 3 is absent then 2 just swaps its position.
\\
vpar_transpose([0,1,0]) == [0,0,1]       || error();
vpar_transpose([0,0,1]) == [0,1,0]       || error();
vpar_transpose([0,1]) == [0,0]       || error();
vpar_transpose([0,0]) == [0,1]       || error();

\\ As noted in the addhelp string, a path chaining down becomes a row of
\\ siblings, and vice versa.  When it's a path down from a root, the row is
\\ roots (reckoned siblings of each other).
\\
\\       1        1..2..3        1
\\       |    =>            =>   |
\\       2                       2
\\       |                       |
\\       3                       3
\\
vpar_transpose([0,1,2]) == [0,0,0]       || error();
vpar_transpose([0,0,0]) == [0,1,2]       || error();

\\ If a vertex has further sibling and/or child then they move with it, and
\\ transpose within them too.  If a diagonal is drawn South-East down from
\\ top-left corner then the whole structure mirrors across that line.  In
\\ the "natural correspondence" to binary trees, that line is a middle
\\ vertical and transpose is a left<->right mirror image in the binary tree.
\\
\\                                  binary tree natural correspondence    
\\       1..3         1..2..4               1           1                 
\\       |\      =>   |  |                 / \    =>   / \                
\\       2..5         3  5                2   3       3   2               
\\       |                               / \             / \
\\       4                              4   5           5   4
\\
vpar_transpose([0,1,0,2,1]) == [0,0,1,0,2]       || error();
vpar_transpose([0,0,1,0,2]) == [0,1,0,2,1]       || error();


\\ Also noted in the addhelp is that two transposes returns to the original
\\ vpar if and only if vpar_is_downwards().  The examples above are all
\\ downwards so are restored by a second transpose.
\\
\\ The reason non-downwards anywhere is not restored is essentially that
\\ when a chain down becomes siblings, the siblings become in asending
\\ order, not what the chain down was.  So transposing again sorts the path,
\\ and that means different parents.
\\
\\       1        1..2..3        1
\\       |    =>            =>   |
\\       3                       2
\\       |                       |
\\       2                       3
\\
vpar_transpose([0,3,1]) == [0,0,0]       || error();
vpar_transpose([0,0,0]) == [0,1,2]       || error();


\\ The "opp" parameter for opposite transpose is by thinking of sibling
\\ relations going from the biggest to smallest.  The biggest child and the
\\ previous sibling swap positions.
\\
\\ A correspondence to binary tree can be made in this way too, by
\\     binary left = prev sibling
\\     binary right = biggest child
\\ Opposite transpose is left<->right mirror of such binary tree.
\\
\\ This form has pre-order forest traversal corresponding to in-order binary
\\ tree traversal.  Knuth uses this in considering forests of given
\\ pre-order "footprints".  See footprints-preorder.gp here.
\\
\\                                   binary tree by prev,last
\\       1..3        2..3                  3           3
\\          |   =>      |                 / \    =>   / \
\\          2   opp     1                1   2       2   1
\\
vpar_transpose([0,3,0],1) == [3,0,0]       || error();

\\-----------------------------------------------------------------------------