/* Copyright 2021 Kevin Ryde

   This file is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This file is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with this file.  See the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>.
*/

/* Kevin Ryde, May 2021

   Usage: ./a.out [-v] n n lo-hi

   Generate terms of A999999 by iterating through all rooted forests of n
   vertices, finding their automorphism orbits with Nauty, and counting the
   partitions of n which occur as orbit sizes.  This is a brute-force
   approach, suitable only for small n.  Created for use with Nauty 2.7r1.

   The default is to try all n from 1 to 32.  Individual n or ranges can be
   given on the command line

          ./a.out 2 5-7 13

   Output is like

       n=1  a(n) = 1
       n=2  a(n) = 2
       n=3  a(n) = 3
       n=4  a(n) = 5
       n=5  a(n) = 6

   Command-line option -v prints some more verbose information.

   Compiling
   ---------

   Put this file in a Nauty build directory which has had its "./configure"
   and "make nauty1.a", then compile to an executable a.out,

       gcc -O2 A999999-nauty.c nauty1.a

   If Nauty is installed in system locations (such as Debian or similar),
   then #include "nauty.h" below must be #include <nauty/nauty.h>, and

       gcc -O2 A999999-nauty.c -lnauty1

   The code uses 1-word graphs so that the "nauty1.a" version of the library
   suffices, and is a little faster than the general "nauty.a".  1-word
   graphs limit to n <= WORDSIZE, where WORDSIZE is either 32 or 64 bits.
   More than 32 is expected to be too slow anyway.  (To attempt more,
   increase SEEN_SIZE defined below, and on a 32-bit machine compile with
   -DWORDSIZE=64 and link to nautyL1.a.)

   Notes
   -----

   A rooted forest is made into an undirected graph for Nauty, with the
   forest roots distinguished by putting them in their own initial partition.
   This seems faster than giving Nauty a directed graph with edges from
   child to parent (or vice versa).

   Nauty's densenauty() returns automorphism orbits in the form of an array
   orbits[v] = u which means vertex v is in the same orbit as vertex u, and
   u is the smallest vertex in that orbit.  Here this is turned into the
   size of each orbit (number of vertices in it), and those orbit sizes are
   sorted.  Duplicate orbit sizes vectors are detected by building a seen[]
   tree.

   An orbit sizes vector is a partition of n (sum of sizes is n) so the
   number of vectors is at most the number of partitions of n (A000041).
   The number of partitions grows much slower than the number of rooted
   forests, which allows the seen[] storage to be a relatively modest size.
   Not every partition occurs, but that doesn't reduce the size of the final
   seen[] structure by very much, in the form constructed here.
*/

/* Change this to WANT_ASSERTS 1 to run the assert() statements for various
   slow-ish consistency checks.  */
#define WANT_ASSERTS  0

/* Change this to WANT_DEBUG 1 for mucho debug prints. */
#define WANT_DEBUG  0


/* The code here is intended for 1-word graph storage and operations, so set
   MAXN accordingly.  Do this before nauty.h so as to get from it some
   1-word-only versions of macros.
*/
#define MAXN  WORDSIZE

#if ! WANT_ASSERTS
#define NDEBUG         /* for assert.h */
#endif

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "nauty.h"

#if WANT_DEBUG
#define DEBUG(expr)  expr
#else
#define DEBUG(expr)
#endif

int verbose = 0;

#define MIN(x,y)  ((x)<(y) ? (x) : (y))


/* ------------------------------------------------------------------------- */
/* seen[] is a search tree recording the partitions of n which have been
   seen as orbit sizes vectors.

   Each node in this tree is a vector of pointers t[].  Orbit sizes are >=1
   and each size s descends to sub-node t[s-1], successively.  If
   t[s-1]==NULL then there's no such node yet, so this is a new orbit sizes
   vector, and new nodes are created for its descents.

   SEEN_SIZE must be big enough for all the nodes created.  For n vertices,
   an upper bound is SEEN_SIZE <= A026905(n) = the cumulative partition
   numbers.  Eg. for n=32 at most A026905(32) = 43819.

   If every partition of n occurred then seen[] space used would be exactly
   A026905(n).  Not every partition occurs, but this seems to reduce the
   space needed by only a small amount.
*/
#define SEEN_SIZE  65536
typedef void *POINTER;
POINTER seen[SEEN_SIZE];
POINTER *seen_end;  /* pointer into seen[], immediately after used space */

void
init_seen (int n)
{
  memset(seen, '\0', sizeof(seen));  /* all NULLs */
  seen_end = seen + n;   /* after first node of sizes 1..n */
}

/* orbitsizes[0 .. num_orbits-1] is a partition of n, so that sum
   orbitsizes[0] + ... + orbitsizes[num_orbits-1] == n, with terms in
   descending order so that orbitsizes[i] >= orbitsizes[i+1].
   If orbitsizes[] is not already in seen[], then add it and return 1.
   If orbitsizes[] is already in seen[], then do nothing and return 0.
*/
int
add_seen (int n, const int *orbitsizes, int num_orbits)
{
  int is_new = 0;
  POINTER *t = seen;
  int i;
  for (i = 0; i < num_orbits; i++) {
    int size = orbitsizes[i];
    assert (n > 0);
    assert (t != NULL);
    assert (1 <= size && size <= n);
    assert (i==0 || orbitsizes[i-1] >= orbitsizes[i]);  /* decreasing */

    n -= size;  /* new remaining total in orbitsizes[] */
    assert (n >= 0);

    if (t[size-1]) {
      t = t[size-1];   /* go to existing node */

    } else {
      /* New node_size is at most as big as "size" again, since sizes in
         descending order, but is also at most the remaining "n", so
         whichever is smaller.
      */
      int node_size = MIN (n, size);
      DEBUG(printf("  seen insert i=%d size=%d at seen_end=%p\n", i, size, seen_end));

      if (seen_end >= seen + SEEN_SIZE - node_size) {
        fprintf(stderr, "oops, exceeded SEEN_SIZE\n");
        exit(1);
      }

      is_new = 1;
      t = t[size-1] = seen_end;
      seen_end += node_size;
    }
  }
  assert (n == 0);  /* after subtracting all orbit sizes */
  return (is_new);
}

/* ------------------------------------------------------------------------- */
/* Rooted forests are iterated here in the manner of Beyer and Hedetniemi.

       Terry Beyer and Sandra Mitchell Hedetniemi, "Constant Time Generation
       of Rooted Trees", SIAM Journal of Computing, volume 9, number 4,
       November 1980, pages 706-712.

   A rooted forest with vertices v = 0..n-1 is represented by a pre-order
   depths vector depths[0..n-1].  The roots are depth 0, their children are
   depth 1, and so on.  Being pre-order means a vertex v at depth[v] has
   parent p which is the closest p < v at depths[p] = depths[v] - 1.
*/

/* Set depths[0..n-1] to the first rooted forest of n vertices.
   This is depths 0,1,2,...,n-1, representing a path-n rooted at its start.
*/
void
rooted_depths_first (int *depths, int n)
{
  int i;
  for (i=0; i<n; i++) depths[i] = i;
}

/* depths[0..n-1] is a rooted forest.
   Change depths[] to the next rooted forest and return 1, or if there's no
   more rooted forests then return 0.
*/
int
rooted_depths_next (int *depths, int n)
{
  int p, d, q, len;
  DEBUG(printf("  vpar_rooted_forest_next()\n"));
  assert (n<1 || depths[0] == 0);

  for (p = n-1; ; p--) {    /* find p end-most with depths[p] > 0 */
    if (p < 0) return(0);
    if ((d=depths[p]) > 0) break;
  }
  for (q = p-1; ; q--) {    /* find q = parent of p */
    assert (q >= 0);
    if (depths[q] < d) break;
  }
  DEBUG(printf("    p=%d (depth d=%d) q=%d len=%d\n", p,d,q,p-q));

  /* Replace depths[p..n-1] with copies of depths[q..p-1] and possible
     partial copy at the end.
  */
  while (p < n) {
    depths[p++] = depths[q++];
  }
}


/* ------------------------------------------------------------------------- */
/* Progress messages are printed roughly every PROGRESS_INTERVAL seconds.
   num_rooted_forests[n] is the total number of rooted forests of n
   vertices, which is used to show progress as a percentage of total work.

   The test for whether or not to print a message is based on a
   progress_remaining counter, so as to avoid the slowdown of say a time()
   system call for every forest.
*/

/* A000081 */
static const double num_rooted_forests[] = {
  1.0, 1.0, 2.0, 4.0, 9.0, 20.0, 48.0, 115.0, 286.0, 719.0,
  1842.0, 4766.0, 12486.0, 32973.0, 87811.0, 235381.0, 634847.0,
  1721159.0, 4688676.0, 12826228.0, 35221832.0, 97055181.0,
  268282855.0, 743724984.0, 2067174645.0, 5759636510.0,
  16083734329.0, 45007066269.0, 126186554308.0, 354426847597.0
};
#define NUM_ROOTED_FORESTS_LEN \
  (sizeof(num_rooted_forests) / sizeof(num_rooted_forests[0]))
#define PROGRESS_INTERVAL  5  /* seconds between prints */
time_t progress_lasttime;
int progress_block = 100;  /* how many forests between progress prints */
int progress_remaining;    /* how many remaining to the next print */

#define PROGRESS(n,forests_count)       \
  do {                                  \
    if (--progress_remaining < 0)       \
      progress_print(n,forests_count);  \
  } while (0)

void
progress_print (int n, double forests_count)
{
  if (n < NUM_ROOTED_FORESTS_LEN && num_rooted_forests[n])
    printf("n=%d, %.0lf forests, %.1lf%%\r",
           n, forests_count,
           100.0 * forests_count / num_rooted_forests[n]);
  else
    printf("n=%d, %.0lf forests\r", n, forests_count);

  {
    /* The previous progress_block many forests took elapsed_time to run.
       Recalculate progress_block to be about 5 seconds worth of forests.
       But try to beware of clock jumps or process stops
     */
    time_t t = time(NULL);
    double elapsed_time = abs(t - progress_lasttime);
    if (elapsed_time) {
      int per_second = progress_block / elapsed_time;
      progress_block = MIN(PROGRESS_INTERVAL*per_second+1,
                           2*progress_block+1);
    } else {
      /* took 0 seconds for the last block, try doubling */
      progress_block = 2*progress_block + 1;
    }
    if (progress_block < 0) progress_block = 100;
    progress_remaining = progress_block;
    progress_lasttime = t;
  }
}

/* ------------------------------------------------------------------------- */

/* depths[0..n-1] is a rooted forest as pre-order depths.
   Convert depths[] to an undirected Nauty graph g,m,n.
   Undirected means arcs both ways for edges, and the roots of the forest
   not distinguished in the graph.
   g must have space for n vertices.
   Any existing contents in g are replaced.
*/
void
depths_to_graph (const int *depths, graph *g, int m, int n)
{
  int v, prev[MAXN];
  assert (n >= 0);
  assert (n <= MAXN);
  assert (m >= SETWORDSNEEDED(n));

  EMPTYGRAPH(g, m,n);
  for (v = 0; v < n; v++) {
    int d = depths[v];
    assert(d >= 0);
    assert(d < n);
    if (d > 0) {
      int to = prev[d-1];
      ADDONEEDGE(g, v,to, m);
    }
    prev[d] = v;
  }
}

/* depths[0..n-1] is a rooted forest as pre-order depths.  Set lab[] and
   ptn[] to an initial partition which distinguishes the roots for Nauty.

   lab[] is set to list the root vertex numbers first (those where
   depths[v]==0), followed by all other vertex numbers.  ptn[] is set to 1
   except a 0 at the last root and a 0 at the last of the others (which is
   v=n-1), to mark the ends of the partitions.

   As a note, it would be straightforward to partition by all depths, so the
   depths[v]==0 are separate, the depths[v]==1 are separate, and so on.  But
   doing that is no measurable speedup over just distinguishing the roots
   and letting Nauty do the rest.
*/
void
depths_to_lab_ptn (const int *depths, int n, int *lab, int *ptn)
{
  int v, rpos, opos;
  int num_roots = 0;
  for (v=0; v<n; v++) num_roots += (depths[v]==0);
  {
    int rpos = 0;          /* roots to rpos */
    int opos = num_roots;  /* others to opos */
    for (v = 0; v < n; v++) {
      if (depths[v]) lab[opos++] = v;
      else           lab[rpos++] = v;
    }
    assert(opos == n);
    assert(rpos == num_roots);
  }
  for (v = 0; v < n; v++) ptn[v] = 1;
  assert(num_roots-1 >= 0);
  assert(n-1 >= 0);
  ptn[num_roots-1] = 0;   /* mark last of the roots */
  ptn[n-1] = 0;           /* mark last of the others */

#if 0
  int histogram[MAXN];
  int v, max_depth = -1;
  int d, dpos[MAXN+1];

  memset(histogram, '\0', n*sizeof(histogram[0]));
  for (v = 0; v < n; v++) {
    int d = depths[v];
    assert (d >= 0);
    assert (d < n);
    if (d > max_depth) {
      assert (d == max_depth+1);
      max_depth = d;
      histogram[d] = 1;
    } else {
      histogram[d]++;
    }
  }

  /* ptn[] is all 1s except a 0 marker at the last of each colour class. */
  for (v = 0; v < n; v++) ptn[v] = 1;
  dpos[0] = 0;
  for (d = 0; d <= max_depth; d++) {
    int after = dpos[d] + histogram[d];
    dpos[d+1] = after;

    assert (after-1 >= 0 && after-1 < n);
    ptn[after-1] = 0;
  }
  assert (dpos[max_depth+1] == n);  /* total vertices at all depths */

  /* lab[] is a list of the vertices 0..n-1 with those of each colour
     contiguous.  Here this is the vertices of successive depth levels.
  */
  for (v = 0; v < n; v++) {
    int d = depths[v];
    lab[dpos[d]++] = v;
  }
  assert (dpos[max_depth] == n);  /* advances to after all of n */
#endif
}

/* xp and yp are pointers to ints */
int
compare_int_descending (const void *xp, const void *yp)
{
  return ((int) * (const int *) yp - (int) * (const int *) xp);
}

static DEFAULTOPTIONS_GRAPH(options);

int
process_depths (const int *depths, int n)
{
  const int m = SETWORDSNEEDED(MAXN);
  setword g[MAXN*SETWORDSNEEDED(MAXN)];
  setword g_out[MAXN*SETWORDSNEEDED(MAXN)];
  int lab[MAXN];
  int ptn[MAXN];
  int orbits[MAXN];
  int orbitsizes[MAXN];
  int v;
  statsblk stats;

  DEBUG(printf("process_depths() n=%d\n", n));
  assert(n <= MAXN);                  /* g[] array size */
  assert(m <= SETWORDSNEEDED(MAXN));  /* g[] array size */
  assert(m >= SETWORDSNEEDED(n));
  DEBUG(printf("  depths ");
        for(v=0; v<n; v++) printf(" %d", depths[v]);
        printf("\n"));

  depths_to_graph(depths, g,m,n);
  depths_to_lab_ptn (depths,n, lab,ptn);
  densenauty ((graph*)g, lab, ptn, orbits, &options, &stats, m,n, g_out);

  memset (orbitsizes, '\0', n*sizeof(orbitsizes[0]));
  for (v = 0; v < n; v++) {
    int u = orbits[v];  /* smallest vertex in the orbit of v */
    assert (0 <= u && u < n);
    orbitsizes[u]++;    /* how many in the orbit of u */
  }

  DEBUG(printf("    orbitsizes ");
        for(v=0; v<n; v++) printf(" %d", (int) orbitsizes[v]);
        printf("\n"));

  qsort (orbitsizes, n, sizeof(orbitsizes[0]), compare_int_descending);

  DEBUG(printf("    sorted     ");
        for(v=0; v<n; v++) printf(" %d", (int) orbitsizes[v]);
        printf("\n"));
#if WANT_ASSERTS
  /* stats.numorbits many non-0, rest of orbitsizes[] all 0 */
  for(v=stats.numorbits; v<n; v++) assert(orbitsizes[v]==0);
#endif

  return (add_seen(n, orbitsizes, stats.numorbits));
}

/* calculate and print A337114(n) */
void
calculate_n (int n)
{
  double forests_count = 0;
  int depths[MAXN];
  int num_seen = 0;

  if (! (1 <= n && n <= MAXN)) {
    fprintf(stderr, "n out of range, must be 1 to %d, got %d\n", MAXN, n);
    exit(1);
  }
  {
    int m = SETWORDSNEEDED(n);
    nauty_check(WORDSIZE,m,n,NAUTYVERSIONID);
  }
  if (verbose) {
    printf("run n=%d\n", n);
  }

  init_seen(n);
  rooted_depths_first(depths,n);
  do {
    PROGRESS(n,forests_count);
    forests_count++;
    num_seen += process_depths(depths,n);
  } while (rooted_depths_next(depths,n));

  if (verbose) {
    printf("n=%d end, rooted tree count %.0lf, seen array %u used, a(%d) = %d\n",
           n, forests_count, seen_end - seen, n, num_seen);
    printf("\n");

  } else {
    printf("n=%d  a(n) = %d                             \n", n, num_seen);
  }
  if (n < NUM_ROOTED_FORESTS_LEN)
    assert (forests_count == num_rooted_forests[n]);

}

int
main (int argc, char **argv)
{
  int seen_n = 0;
  int i;
  setbuf(stdout,NULL);
  options.defaultptn = 0;

  for (i = 1; i < argc; i++) {
    const char *arg = argv[i];
    int n, len;
    unsigned lo, hi;
    if (strcmp(arg,"-v") == 0) {
      verbose = 1;
      printf("NAUTYVERSION is %s\n", NAUTYVERSION);
      printf("SEEN_SIZE %u is %u bytes\n", SEEN_SIZE, sizeof(seen));
      printf("\n");
    } else if (sscanf(arg, "%d%n", &n, &len) == 1
               && len == strlen(arg)) {
      seen_n = 1;
      calculate_n(n);
    } else if (sscanf(arg, "%u-%u%n", &lo, &hi, &len) == 2
               && len == strlen(arg)) {
      seen_n = 1;
      for (n = lo; n <= hi; n++)
        calculate_n(n);
    } else {
      fprintf(stderr, "Unrecognised command line option: %s\n", arg);
      exit(1);
    }
  }

  if (!seen_n) {
    int n;
    for (n = 1; n <= MAXN; n++) {
      calculate_n(n);
    }
  }
  return 0;
}
